Package: mpoly
Type: Package
Title: Symbolic Computation and More with Multivariate Polynomials
Version: 1.1.2
URL: https://github.com/dkahle/mpoly
BugReports: https://github.com/dkahle/mpoly/issues
Authors@R: person("David", "Kahle", email = "david@kahle.io", role = c("aut", "cre"), comment = c(ORCID = "0000-0002-9999-1558"))
Description: Symbolic computing with multivariate polynomials in R.
Imports: stringr (>= 1.0.0), stringi, partitions, plyr, stats, ggplot2,
        polynom, orthopolynom, tidyr
Suggests: testthat (>= 2.1.0), magrittr, dplyr, covr
License: GPL-2
RoxygenNote: 7.3.2
Encoding: UTF-8
NeedsCompilation: no
Packaged: 2025-06-09 20:40:29 UTC; david_kahle
Author: David Kahle [aut, cre] (ORCID: <https://orcid.org/0000-0002-9999-1558>)
Maintainer: David Kahle <david@kahle.io>
Repository: CRAN
Date/Publication: 2025-06-09 21:20:02 UTC
Built: R 4.3.3; ; 2025-06-19 05:06:11 UTC; unix

# Generated by roxygen2: do not edit by hand

S3method("!=",mpoly)
S3method("*",mpoly)
S3method("*",mpolyList)
S3method("+",mpoly)
S3method("+",mpolyList)
S3method("-",mpoly)
S3method("-",mpolyList)
S3method("==",mpoly)
S3method("[",mpoly)
S3method("^",mpoly)
S3method("^",mpolyList)
S3method(as.mpoly,alm)
S3method(as.mpoly,default)
S3method(as.mpoly,lm)
S3method(as.mpoly,numeric)
S3method(as.mpoly,polynomial)
S3method(base::as.function,bernstein)
S3method(base::as.function,bezier)
S3method(base::as.function,mpoly)
S3method(base::as.function,mpolyList)
S3method(base::plot,mpoly)
S3method(coef,mpoly)
S3method(deriv,mpoly)
S3method(print,mpoly)
S3method(print,mpolyList)
S3method(reorder,mpoly)
S3method(round,mpoly)
S3method(terms,mpoly)
export(LC)
export(LCM)
export(LM)
export(LT)
export(as.mpoly)
export(basis_monomials)
export(bernstein)
export(bernsteinApprox)
export(bernstein_approx)
export(bezier)
export(bezierFunction)
export(bezier_function)
export(burst)
export(chebyshev)
export(chebyshev_roots)
export(coef_lift)
export(dehomogenize)
export(eq_mp)
export(exponents)
export(gradient)
export(hermite)
export(homogeneous_components)
export(homogenize)
export(insert)
export(is.bernstein)
export(is.bezier)
export(is.chebyshev)
export(is.constant)
export(is.homogeneous)
export(is.linear)
export(is.mpoly)
export(is.mpolyList)
export(is.unipoly)
export(is.wholenumber)
export(jacobi)
export(laguerre)
export(legendre)
export(lissajous)
export(make_indeterminate_list)
export(monomials)
export(mp)
export(mpoly)
export(mpolyList)
export(multideg)
export(normalize_coefficients)
export(partitions)
export(permutations)
export(plug)
export(solve_unipoly)
export(swap)
export(totaldeg)
export(tuples)
export(vars)
importFrom(ggplot2,aes)
importFrom(ggplot2,ggplot)
importFrom(graphics,contour)
importFrom(graphics,par)
importFrom(orthopolynom,chebyshev.c.polynomials)
importFrom(orthopolynom,chebyshev.s.polynomials)
importFrom(orthopolynom,chebyshev.t.polynomials)
importFrom(orthopolynom,chebyshev.u.polynomials)
importFrom(orthopolynom,glaguerre.polynomials)
importFrom(orthopolynom,hermite.h.polynomials)
importFrom(orthopolynom,hermite.he.polynomials)
importFrom(orthopolynom,jacobi.g.polynomials)
importFrom(orthopolynom,jacobi.p.polynomials)
importFrom(orthopolynom,legendre.polynomials)
importFrom(partitions,parts)
importFrom(plyr,ddply)
importFrom(polynom,polynomial)
importFrom(stats,coef)
importFrom(stats,deriv)
importFrom(stats,reorder)
importFrom(stringi,stri_c)
importFrom(stringi,stri_length)
importFrom(stringi,stri_replace_all_fixed)
importFrom(stringi,stri_replace_all_regex)
importFrom(stringi,stri_sub)
importFrom(stringr,"str_sub<-")
importFrom(stringr,fixed)
importFrom(stringr,str_c)
importFrom(stringr,str_count)
importFrom(stringr,str_detect)
importFrom(stringr,str_dup)
importFrom(stringr,str_extract)
importFrom(stringr,str_extract_all)
importFrom(stringr,str_locate)
importFrom(stringr,str_locate_all)
importFrom(stringr,str_remove)
importFrom(stringr,str_remove_all)
importFrom(stringr,str_replace)
importFrom(stringr,str_replace_all)
importFrom(stringr,str_split)
importFrom(stringr,str_sub)
importFrom(stringr,str_trim)
importFrom(tidyr,gather)

<!-- README.md is generated from README.Rmd. Please edit that file -->

# mpoly

<!-- badges: start -->

[![CRAN
status](https://www.r-pkg.org/badges/version/mpoly)](https://cran.r-project.org/package=mpoly)
[![Travis build
status](https://travis-ci.org/dkahle/mpoly.svg?branch=master)](https://app.travis-ci.com/dkahle/mpoly)
[![AppVeyor build
status](https://ci.appveyor.com/api/projects/status/github/dkahle/mpoly?branch=master&svg=true)](https://ci.appveyor.com/project/dkahle/mpoly)
[![Coverage
status](https://codecov.io/gh/dkahle/mpoly/branch/master/graph/badge.svg)](https://app.codecov.io/github/dkahle/mpoly?branch=master)
<!-- badges: end -->

## Specifying polynomials

**mpoly** is a simple collection of tools to help deal with multivariate
polynomials *symbolically* and functionally in R. Polynomials are
defined with the `mp()` function:

``` r
library("mpoly")
mp("x + y")
# x  +  y

mp("(x + 4 y)^2 (x - .25)")
# x^3  -  0.25 x^2  +  8 x^2 y  -  2 x y  +  16 x y^2  -  4 y^2
```

[Term
orders](https://en.wikipedia.org/wiki/Lexicographical_order#Monomials)
are available with the reorder function:

``` r
(p <- mp("(x + y)^2 (1 + x)"))
# x^3  +  x^2  +  2 x^2 y  +  2 x y  +  x y^2  +  y^2

reorder(p, varorder = c("y","x"), order = "lex")
# y^2 x  +  y^2  +  2 y x^2  +  2 y x  +  x^3  +  x^2

reorder(p, varorder = c("x","y"), order = "glex")
# x^3  +  2 x^2 y  +  x y^2  +  x^2  +  2 x y  +  y^2
```

Vectors of polynomials (`mpolyList`’s) can be specified in the same way:

``` r
mp(c("(x+y)^2", "z"))
# x^2  +  2 x y  +  y^2
# z
```

## Polynomial parts

You can extract pieces of polynoimals using the standard `[` operator,
which works on its terms:

``` r
p[1]
# x^3

p[1:3]
# x^3  +  x^2  +  2 x^2 y

p[-1]
# x^2  +  2 x^2 y  +  2 x y  +  x y^2  +  y^2
```

There are also many other functions that can be used to piece apart
polynomials, for example the leading term (default lex order):

``` r
LT(p)
# x^3

LC(p)
# [1] 1

LM(p)
# x^3
```

You can also extract information about exponents:

``` r
exponents(p)
# [[1]]
# x y 
# 3 0 
# 
# [[2]]
# x y 
# 2 0 
# 
# [[3]]
# x y 
# 2 1 
# 
# [[4]]
# x y 
# 1 1 
# 
# [[5]]
# x y 
# 1 2 
# 
# [[6]]
# x y 
# 0 2

multideg(p)
# x y 
# 3 0

totaldeg(p)
# [1] 3

monomials(p)
# x^3
# x^2
# 2 x^2 y
# 2 x y
# x y^2
# y^2
```

## Polynomial arithmetic

Arithmetic is defined for both polynomials (`+`, `-`, `*` and `^`)…

``` r
p1 <- mp("x + y")

p2 <- mp("x - y")

p1 + p2
# 2 x

p1 - p2
# 2 y

p1 * p2
# x^2  -  y^2

p1^2
# x^2  +  2 x y  +  y^2
```

… and vectors of polynomials:

``` r
(ps1 <- mp(c("x", "y")))
# x
# y

(ps2 <- mp(c("2 x", "y + z")))
# 2 x
# y  +  z

ps1 + ps2
# 3 x
# 2 y  +  z

ps1 - ps2
# -1 x
# -1 z

ps1 * ps2 
# 2 x^2
# y^2  +  y z
```

## Some calculus

You can compute derivatives easily:

``` r
p <- mp("x + x y + x y^2")

deriv(p, "y")
# x  +  2 x y

gradient(p)
# y^2  +  y  +  1
# 2 y x  +  x
```

## Function coercion

You can turn polynomials and vectors of polynomials into functions you
can evaluate with `as.function()`. Here’s a basic example using a single
multivariate polynomial:

``` r
f <- as.function(mp("x + 2 y")) # makes a function with a vector argument
# f(.) with . = (x, y)

f(c(1,1))
# [1] 3

f <- as.function(mp("x + 2 y"), vector = FALSE) # makes a function with all arguments
# f(x, y)

f(1, 1)
# [1] 3
```

Here’s a basic example with a vector of multivariate polynomials:

``` r
(p <- mp(c("x", "2 y")))
# x
# 2 y

f <- as.function(p) 
# f(.) with . = (x, y)

f(c(1,1))
# [1] 1 2

f <- as.function(p, vector = FALSE) 
# f(x, y)

f(1, 1)
# [1] 1 2
```

Whether you’re working with a single multivariate polynomial or a vector
of them (`mpolyList`), if it/they are actually univariate polynomial(s),
the resulting function is vectorized. Here’s an example with a single
univariate polynomial.

``` r
f <- as.function(mp("x^2"))
# f(.) with . = x

f(1:3)
# [1] 1 4 9

(mat <- matrix(1:4, 2))
#      [,1] [,2]
# [1,]    1    3
# [2,]    2    4

f(mat) # it's vectorized properly over arrays
#      [,1] [,2]
# [1,]    1    9
# [2,]    4   16
```

Here’s an example with a vector of univariate polynomials:

``` r
(p <- mp(c("t", "t^2")))
# t
# t^2

f <- as.function(p)
f(1)
# [1] 1 1

f(1:3)
#      [,1] [,2]
# [1,]    1    1
# [2,]    2    4
# [3,]    3    9
```

You can use this to visualize a univariate polynomials like this:

``` r
library("tidyverse"); theme_set(theme_classic())
```

``` r
f <- as.function(mp("(x-2) x (x+2)"))
# f(.) with . = x
x <- seq(-2.5, 2.5, .1)

qplot(x, f(x), geom = "line")
# Warning: `qplot()` was deprecated in ggplot2 3.4.0.
# This warning is displayed once every 8 hours.
# Call `lifecycle::last_lifecycle_warnings()` to see where this warning was
# generated.
```

![](tools/README-as-function-1.png)

For multivariate polynomials, it’s a little more complicated:

``` r
f <- as.function(mp("x^2 - y^2")) 
# f(.) with . = (x, y)
s <- seq(-2.5, 2.5, .1)
df <- expand.grid(x = s, y = s)
df$f <- apply(df, 1, f)
qplot(x, y, data = df, geom = "raster", fill = f)
```

![](tools/README-as-function-multi-1.png)

Using [tidyverse-style coding](https://www.tidyverse.org) (install
tidyverse packages with `install.packages("tidyverse")`), this looks a
bit cleaner:

``` r
f <- as.function(mp("x^2 - y^2"), vector = FALSE)
# f(x, y)
seq(-2.5, 2.5, .1) %>% 
  list("x" = ., "y" = .) %>% 
  cross_df() %>% 
  mutate(f = f(x, y)) %>% 
  ggplot(aes(x, y, fill = f)) + 
    geom_raster()
# Warning: `cross_df()` was deprecated in purrr 1.0.0.
# ℹ Please use `tidyr::expand_grid()` instead.
# ℹ See <https://github.com/tidyverse/purrr/issues/768>.
# This warning is displayed once every 8 hours.
# Call `lifecycle::last_lifecycle_warnings()` to see where this warning was
# generated.
```

![](tools/README-as-function-multi-tidy-1.png)

## Algebraic geometry

**Grobner bases are no longer implemented in mpoly; they’re now in
[m2r](https://github.com/coneill-math/m2r).**

``` r
# polys <- mp(c("t^4 - x", "t^3 - y", "t^2 - z"))
# grobner(polys)
```

Homogenization and dehomogenization:

``` r
(p <- mp("x + 2 x y + y - z^3"))
# x  +  2 x y  +  y  -  z^3

(hp <- homogenize(p))
# x t^2  +  2 x y t  +  y t^2  -  z^3

dehomogenize(hp, "t")
# x  +  2 x y  +  y  -  z^3

homogeneous_components(p)
# x  +  y
# 2 x y
# -1 z^3
```

## Special polynomials

**mpoly** can make use of many pieces of the **polynom** and
**orthopolynom** packages with `as.mpoly()` methods. In particular, many
special polynomials are available.

#### [Chebyshev polynomials](https://en.wikipedia.org/wiki/Chebyshev_polynomials)

You can construct [Chebyshev
polynomials](https://en.wikipedia.org/wiki/Chebyshev_polynomials) as
follows:

``` r
chebyshev(1)
# x

chebyshev(2)
# -1  +  2 x^2

chebyshev(0:5)
# 1
# x
# 2 x^2  -  1
# 4 x^3  -  3 x
# 8 x^4  -  8 x^2  +  1
# 16 x^5  -  20 x^3  +  5 x
```

And you can visualize them:

``` r
s <- seq(-1, 1, length.out = 201); N <- 5
(chebPolys <- chebyshev(0:N))
# 1
# x
# 2 x^2  -  1
# 4 x^3  -  3 x
# 8 x^4  -  8 x^2  +  1
# 16 x^5  -  20 x^3  +  5 x

df <- as.function(chebPolys)(s) %>% cbind(s, .) %>% as.data.frame()
names(df) <- c("x", paste0("T_", 0:N))
mdf <- df %>% gather(degree, value, -x)
qplot(x, value, data = mdf, geom = "path", color = degree)
```

![](tools/README-chebyshev-1.png)

#### [Jacobi polynomials](https://en.wikipedia.org/wiki/Jacobi_polynomials)

``` r
s <- seq(-1, 1, length.out = 201); N <- 5
(jacPolys <- jacobi(0:N, 2, 2))
# 1
# 5 x
# 17.5 x^2  -  2.5
# 52.5 x^3  -  17.5 x
# 144.375 x^4  -  78.75 x^2  +  4.375
# 375.375 x^5  -  288.75 x^3  +  39.375 x
 
df <- as.function(jacPolys)(s) %>% cbind(s, .) %>% as.data.frame
names(df) <- c("x", paste0("P_", 0:N))
mdf <- df %>% gather(degree, value, -x)
qplot(x, value, data = mdf, geom = "path", color = degree) +
  coord_cartesian(ylim = c(-25, 25))
```

![](tools/README-jacobi-1.png)

#### [Legendre polynomials](https://en.wikipedia.org/wiki/Legendre_polynomials)

``` r
s <- seq(-1, 1, length.out = 201); N <- 5
(legPolys <- legendre(0:N))
# 1
# x
# 1.5 x^2  -  0.5
# 2.5 x^3  -  1.5 x
# 4.375 x^4  -  3.75 x^2  +  0.375
# 7.875 x^5  -  8.75 x^3  +  1.875 x
 
df <- as.function(legPolys)(s) %>% cbind(s, .) %>% as.data.frame
names(df) <- c("x", paste0("P_", 0:N))
mdf <- df %>% gather(degree, value, -x)
qplot(x, value, data = mdf, geom = "path", color = degree)
```

![](tools/README-legendre-1.png)

#### [Hermite polynomials](https://en.wikipedia.org/wiki/Hermite_polynomials)

``` r
s <- seq(-3, 3, length.out = 201); N <- 5
(hermPolys <- hermite(0:N))
# 1
# x
# x^2  -  1
# x^3  -  3 x
# x^4  -  6 x^2  +  3
# x^5  -  10 x^3  +  15 x

df <- as.function(hermPolys)(s) %>% cbind(s, .) %>% as.data.frame
names(df) <- c("x", paste0("He_", 0:N))
mdf <- df %>% gather(degree, value, -x)
qplot(x, value, data = mdf, geom = "path", color = degree)
```

![](tools/README-hermite-1.png)

#### [(Generalized) Laguerre polynomials](https://en.wikipedia.org/wiki/Laguerre_polynomials)

``` r
s <- seq(-5, 20, length.out = 201); N <- 5
(lagPolys <- laguerre(0:N))
# 1
# -1 x  +  1
# 0.5 x^2  -  2 x  +  1
# -0.1666667 x^3  +  1.5 x^2  -  3 x  +  1
# 0.04166667 x^4  -  0.6666667 x^3  +  3 x^2  -  4 x  +  1
# -0.008333333 x^5  +  0.2083333 x^4  -  1.666667 x^3  +  5 x^2  -  5 x  +  1

df <- as.function(lagPolys)(s) %>% cbind(s, .) %>% as.data.frame
names(df) <- c("x", paste0("L_", 0:N))
mdf <- df %>% gather(degree, value, -x)
qplot(x, value, data = mdf, geom = "path", color = degree) +
  coord_cartesian(ylim = c(-25, 25))
```

![](tools/README-laguerre-1.png)

#### [Bernstein polynomials](https://en.wikipedia.org/wiki/Bernstein_polynomial)

[Bernstein
polynomials](https://en.wikipedia.org/wiki/Bernstein_polynomial) are not
in **polynom** or **orthopolynom** but are available in **mpoly** with
`bernstein()`:

``` r
bernstein(0:4, 4)
# x^4  -  4 x^3  +  6 x^2  -  4 x  +  1
# -4 x^4  +  12 x^3  -  12 x^2  +  4 x
# 6 x^4  -  12 x^3  +  6 x^2
# -4 x^4  +  4 x^3
# x^4

s <- seq(0, 1, length.out = 101)
N <- 5 # number of bernstein polynomials to plot
(bernPolys <- bernstein(0:N, N))
# -1 x^5  +  5 x^4  -  10 x^3  +  10 x^2  -  5 x  +  1
# 5 x^5  -  20 x^4  +  30 x^3  -  20 x^2  +  5 x
# -10 x^5  +  30 x^4  -  30 x^3  +  10 x^2
# 10 x^5  -  20 x^4  +  10 x^3
# -5 x^5  +  5 x^4
# x^5

df <- as.function(bernPolys)(s) %>% cbind(s, .) %>% as.data.frame
names(df) <- c("x", paste0("B_", 0:N))
mdf <- df %>% gather(degree, value, -x)
qplot(x, value, data = mdf, geom = "path", color = degree)
```

![](tools/README-bernstein-1.png)

You can use the `bernstein_approx()` function to compute the Bernstein
polynomial approximation to a function. Here’s an approximation to the
standard normal density:

``` r
p <- bernstein_approx(dnorm, 15, -1.25, 1.25)
round(p, 4)
# -0.1624 x^2  +  0.0262 x^4  -  0.002 x^6  +  0.0001 x^8  +  0.3796

x <- seq(-3, 3, length.out = 101)
df <- data.frame(
  x = rep(x, 2),
  y = c(dnorm(x), as.function(p)(x)),
  which = rep(c("actual", "approx"), each = 101)
)
# f(.) with . = x
qplot(x, y, data = df, geom = "path", color = which)
```

![](tools/README-bernstein-approx-1.png)

## [Bezier polynomials and curves](https://en.wikipedia.org/wiki/Bézier_curve)

You can construct [Bezier
polynomials](https://en.wikipedia.org/wiki/Bézier_curve) for a given
collection of points with `bezier()`:

``` r
points <- data.frame(x = c(-1,-2,2,1), y = c(0,1,1,0))
(bezPolys <- bezier(points))
# -10 t^3  +  15 t^2  -  3 t  -  1
# -3 t^2  +  3 t
```

And viewing them is just as easy:

``` r
df <- as.function(bezPolys)(s) %>% as.data.frame

ggplot(aes(x = x, y = y), data = df) + 
  geom_point(data = points, color = "red", size = 4) +
  geom_path(data = points, color = "red", linetype = 2) +
  geom_path(size = 2)
# Warning: Using `size` aesthetic for lines was deprecated in ggplot2 3.4.0.
# ℹ Please use `linewidth` instead.
# This warning is displayed once every 8 hours.
# Call `lifecycle::last_lifecycle_warnings()` to see where this warning was
# generated.
```

![](tools/README-bezier-plot-1.png)

Weighting is available also:

``` r
points <- data.frame(x = c(1,-2,2,-1), y = c(0,1,1,0))
(bezPolys <- bezier(points))
# -14 t^3  +  21 t^2  -  9 t  +  1
# -3 t^2  +  3 t
df <- as.function(bezPolys, weights = c(1,5,5,1))(s) %>% as.data.frame

ggplot(aes(x = x, y = y), data = df) + 
  geom_point(data = points, color = "red", size = 4) +
  geom_path(data = points, color = "red", linetype = 2) +
  geom_path(size = 2)
```

![](tools/README-bezier-weighting-1.png)

To make the evaluation of the Bezier polynomials stable, `as.function()`
has a special method for Bezier polynomials that makes use of [de
Casteljau’s
algorithm](https://en.wikipedia.org/wiki/De_Casteljau%27s_algorithm).
This allows `bezier()` to be used as a smoother:

``` r
s <- seq(0, 1, length.out = 201) 
df <- as.function(bezier(cars))(s) %>% as.data.frame
qplot(speed, dist, data = cars) +
  geom_path(data = df, color = "red")
```

![](tools/README-bezier-smooth-1.png)

## Other stuff

I’m starting to put in methods for some other R functions:

``` r
set.seed(1)
n <- 101
df <- data.frame(x = seq(-5, 5, length.out = n))
df$y <- with(df, -x^2 + 2*x - 3 + rnorm(n, 0, 2))

mod <- lm(y ~ x + I(x^2), data = df)
(p <- mod %>% as.mpoly %>% round)
# 1.983 x  -  1.01 x^2  -  2.709
qplot(x, y, data = df) +
  stat_function(fun = as.function(p), colour = 'red')
# f(.) with . = x
```

![](tools/README-lm-1.png)

``` r
s <- seq(-5, 5, length.out = n)
df <- expand.grid(x = s, y = s) %>% 
  mutate(z = x^2 - y^2 + 3*x*y + rnorm(n^2, 0, 3))

(mod <- lm(z ~ poly(x, y, degree = 2, raw = TRUE), data = df))
# 
# Call:
# lm(formula = z ~ poly(x, y, degree = 2, raw = TRUE), data = df)
# 
# Coefficients:
#                           (Intercept)  poly(x, y, degree = 2, raw = TRUE)1.0  
#                             -0.070512                              -0.004841  
# poly(x, y, degree = 2, raw = TRUE)2.0  poly(x, y, degree = 2, raw = TRUE)0.1  
#                              1.005307                               0.001334  
# poly(x, y, degree = 2, raw = TRUE)1.1  poly(x, y, degree = 2, raw = TRUE)0.2  
#                              3.003755                              -0.999536
as.mpoly(mod)
# -0.004840798 x  +  1.005307 x^2  +  0.001334122 y  +  3.003755 x y  -  0.9995356 y^2  -  0.07051218
```

## Installation

-   From CRAN: `install.packages("mpoly")`

-   From Github (dev version):

``` r
# install.packages("devtools")
devtools::install_github("dkahle/mpoly")
```

## Acknowledgements

This material is based upon work partially supported by the National
Science Foundation under Grant
No. [1622449](https://www.nsf.gov/awardsearch/showAward?AWD_ID=1622449).

#' Pretty printing of multivariate polynomials.
#'
#' This is the major function used to view multivariate polynomials.
#'
#' @param x an object of class mpoly
#' @param varorder the order of the variables
#' @param order a total order used to order the monomials in the printing
#' @param stars print the multivariate polynomial in the more computer-friendly
#'   asterisk notation (default FALSE)
#' @param silent logical; if TRUE, suppresses output
#' @param ... additional parameters to go to [base::cat()]
#' @param plus_pad number of spaces to the left and right of plus sign
#' @param times_pad number of spaces to the left and right of times sign
#' @usage \method{print}{mpoly}(x, varorder, order, stars = FALSE, silent =
#'   FALSE, ..., plus_pad = 2L, times_pad = 1L)
#' @return Invisible string of the printed object.
#' @export
#' @examples
#' 
#' mp("-x^5 - 3 y^2 + x y^3 - 1")
#' 
#' 
#' (p <- mp("2 x^5  -  3 y^2  +  x y^3"))
#' print(p) # same
#' print(p, silent = TRUE)
#' s <- print(p, silent = TRUE)
#' s
#'
#' print(p, order = "lex") # -> 2 x^5  +  x y^3  -  3 y^2
#' print(p, order = "lex", varorder = c("y","x")) # -> y^3 x  -  3 y^2  +  2 x^5
#' print(p, varorder = c("y","x")) # -> y^3 x  -  3 y^2  +  2 x^5
#'
#' # this is mostly used internally
#' print(p, stars = TRUE)
#' print(p, stars = TRUE, times_pad = 0L)
#' print(p, stars = TRUE, times_pad = 0L, plus_pad = 1L)
#' print(p, stars = TRUE, times_pad = 0L, plus_pad = 0L)
#' print(p, plus_pad = 1L)
#' 
print.mpoly <- function(x, varorder, order, stars = FALSE, silent = FALSE, ..., plus_pad = 2L, times_pad = 1L) {
	
  # argument checking and basic variable setting
  stopifnot(is.mpoly(x))  
  stopifnot(is.logical(stars)) 
  vars <- vars(x) 
  
  
  # deal with constant mpolys
  if (length(vars) == 0) {
    if (!silent) cat(x[[1]], ...)
  	return(invisible(as.character(x[[1]])))
  }  
  
  
  # check variable order
  if (!missing(varorder)) {
    stopifnot(is.character(varorder))
    if(!all(vars %in% varorder)) {
      stop(
        "if specified, varorder must contain all computed vars - ", 
        stri_c(vars, collapse = ", "),
        call. = FALSE
      )
    }
  } 
  
  if (!missing(order)) match.arg(order, c("lex", "glex", "grlex"))    
  
  
  # reorder
  if (!missing(order) && !missing(varorder)) {
    x <- reorder(x, varorder = varorder, order = order)
  } else if(!missing(varorder)){
    x <- reorder(x, varorder = varorder)
  } else if(!missing(order)){
    x <- reorder(x, order = order)	
  }  
 
  
  # set delimiters
  if (!stars) { 
    
    times <- " "
    expo  <- "^"
  
  } else {
  	
    times_pad <- stringi::stri_dup(" ", times_pad)
    times <- stri_c(times_pad, "*", times_pad)
    expo  <- "**"
      	
  } 
  
  
  # now ready to start constructing the string. we don't do this by growing
  # the entire mpoly because growing in R is slow, so we print each of the terms,
  # merge them together, and take care of edge cases
  # one of the challenges here is to figure out where edge cases should be handled:
  # in the term printing or after the merging. we try to take a practical approach.
  # some things are fixed in term printing (e.g. ^1 or 1 x) and others 
  # (e.g. ?) are done in tidying the final result.
  
  
  # print terms
  terms <- vapply(x, print_term, character(1L), times = times, exp = expo) 


  # merge terms with +'s and -'s
  plus_pad <- stringi::stri_dup(" ", plus_pad)
  plus <- stri_c(plus_pad, "+", plus_pad)
  s <- stri_c(terms, collapse = plus)
  s <- stri_replace_all_fixed(s, stri_c("+", plus_pad, "-"), stri_c("-", plus_pad)) # fix subtractions 
  
  
  # final cleaning - clean "2 x  -  1 y^2"
  s <- stri_replace_all_fixed(s, stri_c("-", plus_pad, "1", times), stri_c("-", plus_pad))  
  
  
  # print
  if(!silent) cat(s, "\n", ...)
  
  
  ## return
  invisible(s)
  
}









print_term <- function(term, times = " ", expo = "^") {
  
  # if a constant term
  if (length(term) == 1L) return(format(term[["coef"]], scientific=FALSE))
  
  # split vars + exps from coef
  coef_ndx <- length(term)
  coef <- term[[ coef_ndx]]
  mono <- term [-coef_ndx]
  
  # build term
  out <- format(coef, scientific = FALSE)
  for (k in seq_along(mono)) {
    if (mono[k] == 1L) {
      out <- stri_c(out, times, names(mono)[k])
    } else {
      out <- stri_c(out, times, stri_c(names(mono)[k], expo, mono[k]))
    }  
  }

  # clean 1 from front
  if (abs(term[["coef"]] - 1) < sqrt(.Machine$double.eps) ) out <- stri_sub(out, stri_length(times) + 2L)
  
  # return
  out
  
}

context("print_term()")



test_that("print_term() prints constants properly", {
  
  expect_equal(print_term(c("coef" = 5)), "5")
  expect_equal(print_term(c("coef" = -5)), "-5")
  expect_equal(print_term(c("coef" = 1)), "1")
  expect_equal(print_term(c("coef" = -1)), "-1")
  expect_equal(print_term(c("coef" = 0)), "0")
  
  expect_equal(print_term(c("coef" = .1)), "0.1")
  expect_equal(print_term(c("coef" = -.1)), "-0.1")
  
  expect_equal(print_term(c("coef" = 0.1)), "0.1")
  expect_equal(print_term(c("coef" = -0.1)), "-0.1")
  
  expect_equal(print_term(c("coef" = 1.2)), "1.2")
  expect_equal(print_term(c("coef" = -1.2)), "-1.2")
  
  # scientific notation
  expect_equal(print_term(c("coef" = 1e2)), "100")
  expect_equal(print_term(c("coef" = -1e2)), "-100")
  
  expect_equal(print_term(c("coef" = 1e-2)), "0.01")
  expect_equal(print_term(c("coef" = -1e-2)), "-0.01")
  
  # longer digit examples - rounding to 7 digits
  expect_equal(print_term(c("coef" = 12.3456789)), "12.34568")
  expect_equal(print_term(c("coef" = -12.3456789)), "-12.34568")
  
})





test_that("print_term() prints univariate terms properly", {
  
  expect_equal(print_term(c("x" = 2L, "coef" =  5)), "5 x^2")
  expect_equal(print_term(c("x" = 2L, "coef" =  1)), "x^2")
  expect_equal(print_term(c("x" = 2L, "coef" = -5)), "-5 x^2")
  expect_equal(print_term(c("x" = 2L, "coef" = -1)), "-1 x^2")
  
  expect_equal(print_term(c("x" = 1L, "coef" =  5)),  "5 x")
  expect_equal(print_term(c("x" = 1L, "coef" =  1)),    "x")
  expect_equal(print_term(c("x" = 1L, "coef" = -5)), "-5 x")
  expect_equal(print_term(c("x" = 1L, "coef" = -1)), "-1 x")   
  
  
  expect_equal(print_term(c("x" = 2L, "coef" =  5), times = " * ", expo = "**"),  "5 * x**2")
  expect_equal(print_term(c("x" = 2L, "coef" =  1), times = " * ", expo = "**"),      "x**2")
  expect_equal(print_term(c("x" = 2L, "coef" = -5), times = " * ", expo = "**"), "-5 * x**2")
  expect_equal(print_term(c("x" = 2L, "coef" = -1), times = " * ", expo = "**"), "-1 * x**2")
  
  expect_equal(print_term(c("x" = 1L, "coef" =  5)),  "5 x")
  expect_equal(print_term(c("x" = 1L, "coef" =  1)),    "x")
  expect_equal(print_term(c("x" = 1L, "coef" = -5)), "-5 x")
  expect_equal(print_term(c("x" = 1L, "coef" = -1)), "-1 x")   
  
  
  # sci notation
  expect_equal(print_term(c("x" = 2L, "coef" =   1e2)),   "100 x^2")
  expect_equal(print_term(c("x" = 2L, "coef" =  -1e2)),  "-100 x^2")
  expect_equal(print_term(c("x" = 2L, "coef" =  1e-2)),  "0.01 x^2")
  expect_equal(print_term(c("x" = 2L, "coef" = -1e-2)), "-0.01 x^2")  
  
  expect_equal(print_term(c("x" = 2L, "coef" =   1e2), times = "*", expo = "**"),   "100*x**2")
  expect_equal(print_term(c("x" = 2L, "coef" =  -1e2), times = "*", expo = "**"),  "-100*x**2")
  expect_equal(print_term(c("x" = 2L, "coef" =  1e-2), times = "*", expo = "**"),  "0.01*x**2")
  expect_equal(print_term(c("x" = 2L, "coef" = -1e-2), times = "*", expo = "**"), "-0.01*x**2")  
  
})




test_that("print_term() prints multivariate terms properly", {
  
  expect_equal(print_term(c("x" = 2L, "y" = 3L, "coef" =  5)),  "5 x^2 y^3")
  expect_equal(print_term(c("x" = 2L, "y" = 3L, "coef" = -5)), "-5 x^2 y^3")
  expect_equal(print_term(c("x" = 2L, "y" = 3L, "coef" =  1)),    "x^2 y^3")
  expect_equal(print_term(c("x" = 2L, "y" = 3L, "coef" = -1)), "-1 x^2 y^3")
  
  expect_equal(print_term(c("x" = 1L, "y" = 3L, "coef" =  5)),  "5 x y^3")
  expect_equal(print_term(c("x" = 1L, "y" = 3L, "coef" = -5)), "-5 x y^3")
  expect_equal(print_term(c("x" = 1L, "y" = 3L, "coef" =  1)),    "x y^3")
  expect_equal(print_term(c("x" = 1L, "y" = 3L, "coef" = -1)), "-1 x y^3")
  
  expect_equal(print_term(c("x" = 2L, "y" = 1L, "coef" =  5)),  "5 x^2 y")
  expect_equal(print_term(c("x" = 2L, "y" = 1L, "coef" = -5)), "-5 x^2 y")
  expect_equal(print_term(c("x" = 2L, "y" = 1L, "coef" =  1)),    "x^2 y")
  expect_equal(print_term(c("x" = 2L, "y" = 1L, "coef" = -1)), "-1 x^2 y")
  
  
  
  expect_equal(print_term(c("x" = 2L, "y" = 3L, "coef" =  5), times = " * ", expo = "**"),  "5 * x**2 * y**3")
  expect_equal(print_term(c("x" = 2L, "y" = 3L, "coef" = -5), times = " * ", expo = "**"), "-5 * x**2 * y**3")
  expect_equal(print_term(c("x" = 2L, "y" = 3L, "coef" =  1), times = " * ", expo = "**"),      "x**2 * y**3")
  expect_equal(print_term(c("x" = 2L, "y" = 3L, "coef" = -1), times = " * ", expo = "**"), "-1 * x**2 * y**3")
  
  expect_equal(print_term(c("x" = 1L, "y" = 3L, "coef" =  5), times = " * ", expo = "**"),  "5 * x * y**3")
  expect_equal(print_term(c("x" = 1L, "y" = 3L, "coef" = -5), times = " * ", expo = "**"), "-5 * x * y**3")
  expect_equal(print_term(c("x" = 1L, "y" = 3L, "coef" =  1), times = " * ", expo = "**"),      "x * y**3")
  expect_equal(print_term(c("x" = 1L, "y" = 3L, "coef" = -1), times = " * ", expo = "**"), "-1 * x * y**3")
  
  expect_equal(print_term(c("x" = 2L, "y" = 1L, "coef" =  5), times = " * ", expo = "**"),  "5 * x**2 * y")
  expect_equal(print_term(c("x" = 2L, "y" = 1L, "coef" = -5), times = " * ", expo = "**"), "-5 * x**2 * y")
  expect_equal(print_term(c("x" = 2L, "y" = 1L, "coef" =  1), times = " * ", expo = "**"),      "x**2 * y")
  expect_equal(print_term(c("x" = 2L, "y" = 1L, "coef" = -1), times = " * ", expo = "**"), "-1 * x**2 * y")
  
})













context("print.mpoly()")


test_that("constants", {
  
  test_coef <- function(number) {
    p <- structure(list(c("coef" = number)), class = "mpoly")
    expect_equal(
      print.mpoly(p, silent = TRUE),
      as.character(number)
    )
  }
  
  test_coef(0)
  test_coef(1)
  test_coef(-1)
  test_coef(1.1)
  test_coef(-1.1) 
  test_coef(1e-3)
  test_coef(-1e-3)
  
})





test_that("sums: no coefs", {
  
  p <- structure(
    list(
      c("x" = 1, "coef" = 1),
      c("y" = 2, "coef" = 1)
    ), 
    class = "mpoly"
  )
  
  expect_equal(
    print.mpoly(p, silent = TRUE),
    "x  +  y^2"
  )
  
})



test_that("sums: + coefs", {
  
  p <- structure(
    list(
      c("x" = 2, "coef" = 2),
      c("y" = 3, "coef" = 3)
    ), 
    class = "mpoly"
  )
  
  expect_equal(
    print.mpoly(p, silent = TRUE),
    "2 x^2  +  3 y^3"
  )
  
})


test_that("subtraction: ordinary", {
  
  p <- structure(
    list(
      c("x" = 1, "coef" = 2),
      c("y" = 3, "coef" = -3)
    ), 
    class = "mpoly"
  )
  
  expect_equal(
    print.mpoly(p, silent = TRUE),
    "2 x  -  3 y^3"
  )
  
})


test_that("subtraction: 2 x - y^2", {
  
  p <- structure(
    list(
      c("x" = 1, "coef" = 2),
      c("y" = 2, "coef" = -1)
    ), 
    class = "mpoly"
  )
  
  expect_equal(
    print.mpoly(p, silent = TRUE),
    "2 x  -  y^2"
  )
  
})





test_that("errors", {
  
  expect_error(
    print.mpoly(
      structure(list(c("x" = 1, "y" = 2, "coef" = 1)), class = "mpoly"),
      varorder = "x",
      silent = TRUE
    ),
    "if specified, varorder must contain all computed vars - x, y",
    fixed = TRUE
  )
  
})

#' Define a multivariate polynomial.
#'
#' mp is a smart function which attempts to create a formal mpoly object from a
#' character string containing the usual representation  of a multivariate
#' polynomial.
#'
#' @param string a character string containing a polynomial, see examples
#' @param varorder (optional) order of variables in string
#' @param stars_only if you format your multiplications using asterisks, setting
#'   this to \code{TRUE} will reduce preprocessing time
#' @param vars a character vector of indeterminates
#' @return An object of class mpoly.
#' @author David Kahle \email{david@@kahle.io}
#' @seealso [mpoly()]
#' @name mp
#' @examples
#'
#' ( m <- mp("x + y + x y") )
#' is.mpoly( m )
#' unclass(m)
#' 
#' mp("1 + x")
#'
#'
#' mp("x + 2 y + x^2 y + x y z")
#' mp("x + 2 y + x^2 y + x y z", varorder = c("y", "z", "x"))
#'
#' ( ms <- mp(c("x + y", "2 x")) )
#' is.mpolyList(ms)
#'
#'
#' gradient( mp("x + 2 y + x^2 y + x y z") )
#' gradient( mp("(x + y)^10") )
#'
#' # mp and the print methods are kinds of inverses of each other
#' ( polys <- mp(c("x + y", "x - y")) )
#' strings <- print(polys, silent = TRUE)
#' strings
#' mp(strings)
#' 






#' @export
#' @rdname mp
make_indeterminate_list <- function (vars) {
  
  make_indeterminate <- function(var, ...) {
    v <- c(1, 1)
    names(v) <- c(var, "coef")
    structure(list(v), class = "mpoly")
  }

  uvars <- unique(vars)
  l <- lapply(uvars, make_indeterminate)
  names(l) <- uvars
  l
  
}
# make_indeterminate_list("a")
# make_indeterminate_list(letters) 



#' @export
#' @rdname mp
mp <- function (string, varorder, stars_only = FALSE) {
  
  # deal with mpolyLists
  if (length(string) > 1) {
    
    # do basic mpoly parsing
    ps <- structure(
      lapply(string, mp, stars_only = stars_only), 
      class = "mpolyList"
    )

    # enforce varorder if present
    if (!missing(varorder)) {
      for (k in seq_along(ps)) {
        ps[[k]] <- structure(
          lapply(ps[[k]], function(term) {
            vars_in_term <- intersect(varorder, names(term))
            term[c(vars_in_term, "coef")]
          }),
          class = "mpoly"
        )
      }
    }
    
    # return early
    return(ps)
  } 
 
  
  # clean spaces if needed
  if(!stars_only)  {
    # put *s in for spaces, twice for situations like "x y z"
    while (str_detect(string, "([\\w\\^.*\\(\\)]+) +([\\w\\^.*\\(\\)]+)")) {
      string <- str_replace_all(string, "([\\w\\^.*\\(\\)]+) +([\\w\\^.*\\(\\)]+)", "\\1*\\2")
    }
    
    # fix )('s and situations like 2(x+1)
    # string <- str_replace_all(string, pattern = "\\)\\(", replacement = ")*(")
    # string <- str_replace_all(string, "([\\w\\^.*\\(\\)]+)(\\(\\))", "\\1*\\2")
    
    # fix things like "-x"
    string <- str_replace_all(string, "^-([\\w\\^.*\\(]+)", "-1*\\1")
  }
  
  # parse using R's parser and mpoly arithmetic
  expr <- parse(text = string)[[1]]
  vars <- stringr::str_extract_all( deparse(expr),  "(?<!\\d)[a-zA-Z]\\w*" )
  vars <- unique(unlist(vars))
  p <- eval(expr, envir = make_indeterminate_list(vars))
  
  # if constant, make an mpoly
  if (is.numeric(p)) return( structure(list(c(coef = p)), class = "mpoly") )
  
  # reorder if needed and return
  if (!missing(varorder)) {
    p <- structure(
      lapply(p, function(term) {
        vars_in_term <- intersect(varorder, names(term))
        term[c(vars_in_term, "coef")]
      }),
      class = "mpoly"
    )
  }
  
  # return
  p
}

#' Define a collection of multivariate polynomials.
#'
#' Combine a series of mpoly objects into a mpolyList.
#'
#' @param ... a series of mpoly objects.
#' @return An object of class mpolyList.
#' @export
#' @examples
#' ( p1 <- mp("t^4 - x") )
#' ( p2 <- mp("t^3 - y") )
#' ( p3 <- mp("t^2 - z") )
#' ( ms <- mpolyList(p1, p2, p3) )
#' is.mpolyList( ms )
#'
#' mpolyList(mp("x + 1"))
#' p <- mp("x + 1")
#' mpolyList(p)
#'
#' ps <- mp(c("x + 1", "y + 2"))
#' is.mpolyList(ps)
#'
#'
#' f <- function(){
#'   a <- mp("1")
#'   mpolyList(a)
#' }
#' f()
#'
#' 
mpolyList <- function(...){
	
  arguments <- as.list(match.call()[-1])  

  out <- lapply(arguments, eval, parent.frame(1))

  # if(is.mpoly(out)) out <- list(out)
  
  if(!all(vapply(out, is.mpoly, logical(1)))){
  	stop("Each argument must be of class mpoly.", call. = FALSE)
  }
  
  class(out) <- "mpolyList"
  out
}

context("mpolyList()")

test_that("basic", {
  
  expect_equal(
    mpolyList(mp("x"), mp("x^2"), mp("x^3 t")),
    structure(
      list(
        structure(list(c(x = 1, coef = 1)), class = "mpoly"), 
        structure(list(c(x = 2, coef = 1)), class = "mpoly"), 
        structure(list(c(x = 3, t = 1, coef = 1)), class = "mpoly")
      ), 
      class = "mpolyList"
    )
  )
  
  expect_error(
    mpolyList(
      mp("x"),
      "y"
    ),
    "Each argument must be of class mpoly.",
    fixed = TRUE
  )
  
  
})

#' Chebyshev polynomials
#'
#' Chebyshev polynomials as computed by orthopolynom.
#'
#' @param degree degree of polynomial
#' @param kind \code{"t"} or \code{"u"} (Chebyshev polynomials of the first and
#'   second kinds), or \code{"c"} or \code{"s"}
#' @param indeterminate indeterminate
#' @param normalized provide normalized coefficients
#' @param k,n the k'th root of the n'th chebyshev polynomial
#' @return a mpoly object or mpolyList object
#' @author David Kahle calling code from the orthopolynom package
#' @seealso [orthopolynom::chebyshev.t.polynomials()],
#'   [orthopolynom::chebyshev.u.polynomials()],
#'   [orthopolynom::chebyshev.c.polynomials()],
#'   [orthopolynom::chebyshev.s.polynomials()],
#'   \url{https://en.wikipedia.org/wiki/Chebyshev_polynomials}
#' @name chebyshev
#' @examples
#'
#' chebyshev(0)
#' chebyshev(1)
#' chebyshev(2)
#' chebyshev(3)
#' chebyshev(4)
#' chebyshev(5)
#' chebyshev(6)
#' chebyshev(10)
#'
#' chebyshev(0:5)
#' chebyshev(0:5, normalized = TRUE)
#' chebyshev(0:5, kind = "u")
#' chebyshev(0:5, kind = "c")
#' chebyshev(0:5, kind = "s")
#' chebyshev(0:5, indeterminate = "t")
#'
#'
#'
#' # visualize the chebyshev polynomials
#'
#' library(ggplot2); theme_set(theme_classic())
#' library(tidyr)
#'
#' s <- seq(-1, 1, length.out = 201)
#' N <- 5 # number of chebyshev polynomials to plot
#' (cheb_polys <- chebyshev(0:N))
#'
#' # see ?bernstein for a better understanding of
#' # how the code below works
#'
#' df <- data.frame(s, as.function(cheb_polys)(s))
#' names(df) <- c("x", paste0("T_", 0:N))
#' mdf <- gather(df, degree, value, -x)
#' qplot(x, value, data = mdf, geom = "line", color = degree)
#'
#' 
#' 
#' # roots of chebyshev polynomials
#' N <- 5
#' cheb_roots <- chebyshev_roots(1:N, N)
#' cheb_fun <- as.function(chebyshev(N))
#' cheb_fun(cheb_roots)
#' 
#' 
#' 
#' # chebyshev polynomials are orthogonal in two ways:
#' T2 <- as.function(chebyshev(2))
#' T3 <- as.function(chebyshev(3))
#' T4 <- as.function(chebyshev(4))
#' 
#' w <- function(x) 1 / sqrt(1 - x^2)
#' integrate(function(x) T2(x) * T3(x) * w(x), lower = -1, upper = 1)
#' integrate(function(x) T2(x) * T4(x) * w(x), lower = -1, upper = 1)
#' integrate(function(x) T3(x) * T4(x) * w(x), lower = -1, upper = 1)
#' 
#' (cheb_roots <- chebyshev_roots(1:4, 4))
#' sum(T2(cheb_roots) * T3(cheb_roots) * w(cheb_roots))
#' sum(T2(cheb_roots) * T4(cheb_roots) * w(cheb_roots))
#' sum(T3(cheb_roots) * T4(cheb_roots) * w(cheb_roots))
#' 
#' sum(T2(cheb_roots) * T3(cheb_roots))
#' sum(T2(cheb_roots) * T4(cheb_roots))
#' sum(T3(cheb_roots) * T4(cheb_roots))
#' 
#'




#' @rdname chebyshev
#' @export
chebyshev <- function(degree, kind = "t", indeterminate = "x", normalized = FALSE){
  
  stopifnot(all(is.wholenumber(degree)))
  stopifnot(all(degree >= 0))
  
  
  ## deal with kind
  stopifnot(kind %in% c("t","u","c","s"))
  
  ## make coefs
  if(kind == "t") coefs <- chebyshev.t.polynomials(max(degree), normalized)
  if(kind == "u") coefs <- chebyshev.u.polynomials(max(degree), normalized)
  if(kind == "c") coefs <- chebyshev.c.polynomials(max(degree), normalized)
  if(kind == "s") coefs <- chebyshev.s.polynomials(max(degree), normalized)
  
  ## if only one degree is wanted, return that
  if(length(degree) == 1){
    coefs <- rev.default(coefs)[[1]]
    p <- as.mpoly.polynomial(coefs, indeterminate)
    class(p) <- c("chebyshev", "mpoly")
    attr(p, "chebyshev") <- list(
      "degree" = length(polynomial)-1, 
      "kind" = kind, 
      "indeterminate" = indeterminate,
      "normalized" = normalized
    )
    return(p)
  }
  
  ## if several are wanted, return them
  coefs <- coefs[degree+1]
  ps <- lapply(coefs, function(polynomial){
    p <- as.mpoly.polynomial(polynomial, indeterminate)
    class(p) <- c("chebyshev", "mpoly")
    attr(p, "chebyshev") <- list(
      "degree" = length(polynomial)-1, 
      "kind" = kind, 
      "indeterminate" = indeterminate,
      "normalized" = normalized
    )
    p
  })
  class(ps) <- "mpolyList"
  ps
  
}




#' @rdname chebyshev
#' @export
chebyshev_roots <- function(k, n) {
  stopifnot( is.wholenumber(k) )
  stopifnot( is.wholenumber(n) )
  stopifnot( round(k) %in% 1:n )
  cos((k - .5) * pi / n)
}

context("swap()")


test_that("swap() works for univariate polynomials", {
  
  expect_equal(
    swap(mp("(x + y)^2"), "x", "t"),
    mp("(t + y)^2")
  )
  
  expect_equal(
    swap(mp("(x + y)^2"), "x", "t_1"),
    mp("(t_1 + y)^2")
  )
  
})



test_that("swap() works for bernstein polynomials", {
  
  expect_equal(
    swap(bernstein(3, 5), "x", "t"),
    bernstein(3, 5, indeterminate = "t")
  )
  
})



test_that("swap() works for chebyshev polynomials", {
  
  expect_equal(
    swap(chebyshev(4), "x", "t"),
    chebyshev(4, indeterminate = "t")
  )
  
})


test_that("swap() correctly errors", {
  
  expect_error(swap(mp("x^2 + 1"), "x", "x^2"))
  
  expect_error(swap(mp("x^2 + 1"), "x", "x 2"))
  
  expect_error(swap(mp("x + y"), "x", "y"))
  
})

context("chebyshev()")


test_that("basic", {
  
  expect_equal(
    chebyshev(3),
    structure(
      list(
        c("x" = 1, "coef" = -3),
        c("x" = 3, "coef" = 4)
      ), 
      "class" = c("chebyshev", "mpoly"),
      "chebyshev" = list(
        "degree" = 0, 
        "kind" = "t", 
        "indeterminate" = "x",
        "normalized" = FALSE
      )
    )
  )
  
  
  expect_equal(
    chebyshev_roots(1:3, 3),
    cos((1:3 - .5) * pi / 3)
  )
  
})







test_that("different kinds", {
  
  expect_equal(
    chebyshev(3, kind = "u"),
    structure(
      list(
        c("x" = 1, "coef" = -4),
        c("x" = 3, "coef" = 8)
      ), 
      "class" = c("chebyshev", "mpoly"),
      "chebyshev" = list(
        "degree" = 0, 
        "kind" = "u", 
        "indeterminate" = "x",
        "normalized" = FALSE
      )
    )
  )
  
  
  expect_equal(
    chebyshev(3, kind = "c"),
    structure(
      list(
        c("x" = 1, "coef" = -1.5),
        c("x" = 3, "coef" = .5)
      ), 
      "class" = c("chebyshev", "mpoly"),
      "chebyshev" = list(
        "degree" = 0, 
        "kind" = "c", 
        "indeterminate" = "x",
        "normalized" = FALSE
      )
    )
  )
  
  
  expect_equal(
    chebyshev(3, kind = "s"),
    structure(
      list(
        c("x" = 1, "coef" = -2),
        c("x" = 3, "coef" = 1)
      ), 
      "class" = c("chebyshev", "mpoly"),
      "chebyshev" = list(
        "degree" = 0, 
        "kind" = "s", 
        "indeterminate" = "x",
        "normalized" = FALSE
      )
    )
  )
  
})






test_that("normalized", {
  
  expect_equal(
    chebyshev(3, normalized = TRUE),
    structure(
      list(
        c("x" = 1, "coef" = -2.393654),
        c("x" = 3, "coef" = 3.191538)
      ), 
      "class" = c("chebyshev", "mpoly"),
      "chebyshev" = list(
        "degree" = 0, 
        "kind" = "t", 
        "indeterminate" = "x",
        "normalized" = TRUE
      )
    ),
    tolerance = 1e-5
  )
  
})






test_that("vectorized", {
  
  expect_equal(
    chebyshev(0:2),
    structure(
      list(
        structure(
          list(
            c("coef" = 1)
          ), 
          "class" = c("chebyshev", "mpoly"),
          "chebyshev" = list(
            "degree" = 0, 
            "kind" = "t", 
            "indeterminate" = "x",
            "normalized" = FALSE
          )
        ),
        structure(
          list(
            c("x" = 1, "coef" = 1)
          ), 
          "class" = c("chebyshev", "mpoly"),
          "chebyshev" = list(
            "degree" = 1, 
            "kind" = "t", 
            "indeterminate" = "x",
            "normalized" = FALSE
          )
        ),
        structure(
          list(
            c("coef" = -1),
            c("x" = 2, "coef" = 2)
          ), 
          "class" = c("chebyshev", "mpoly"),
          "chebyshev" = list(
            "degree" = 2, 
            "kind" = "t", 
            "indeterminate" = "x",
            "normalized" = FALSE
          )
        )
      ),
      "class" = "mpolyList"
    )
  )
  
})

#' Generalized Laguerre polynomials
#'
#' Generalized Laguerre polynomials as computed by orthopolynom.
#'
#' @param degree degree of polynomial
#' @param alpha generalization constant
#' @param indeterminate indeterminate
#' @param normalized provide normalized coefficients
#' @return a mpoly object or mpolyList object
#' @author David Kahle calling code from the orthopolynom package
#' @seealso [orthopolynom::glaguerre.polynomials()],
#'   \url{https://en.wikipedia.org/wiki/Laguerre_polynomials}
#' @export
#' @examples
#'
#' laguerre(0)
#' laguerre(1)
#' laguerre(2)
#' laguerre(3)
#' laguerre(4)
#' laguerre(5)
#' laguerre(6)
#'
#' laguerre(2)
#' laguerre(2, normalized = TRUE)
#'
#' laguerre(0:5)
#' laguerre(0:5, normalized = TRUE)
#' laguerre(0:5, indeterminate = "t")
#'
#'
#'
#' # visualize the laguerre polynomials
#'
#' library(ggplot2); theme_set(theme_classic())
#' library(tidyr)
#'
#' s <- seq(-5, 20, length.out = 201)
#' N <- 5 # number of laguerre polynomials to plot
#' (lagPolys <- laguerre(0:N))
#'
#' # see ?bernstein for a better understanding of
#' # how the code below works
#'
#' df <- data.frame(s, as.function(lagPolys)(s))
#' names(df) <- c("x", paste0("L_", 0:N))
#' mdf <- gather(df, degree, value, -x)
#' qplot(x, value, data = mdf, geom = "line", color = degree)
#'
#' qplot(x, value, data = mdf, geom = "line", color = degree) +
#'   coord_cartesian(ylim = c(-25, 25))
#'
#'
#' # laguerre polynomials are orthogonal with respect to the exponential kernel:
#' L2 <- as.function(laguerre(2))
#' L3 <- as.function(laguerre(3))
#' L4 <- as.function(laguerre(4))
#'
#' w <- dexp
#' integrate(function(x) L2(x) * L3(x) * w(x), lower = 0, upper = Inf)
#' integrate(function(x) L2(x) * L4(x) * w(x), lower = 0, upper = Inf)
#' integrate(function(x) L3(x) * L4(x) * w(x), lower = 0, upper = Inf)
#'
#' 
laguerre <- function(degree, alpha = 0, indeterminate = "x", normalized = FALSE){
  
  stopifnot(all(is.wholenumber(degree)))
  stopifnot(all(degree >= 0))
  
  ## make coefs
  coefs <- glaguerre.polynomials(max(degree), alpha, normalized = normalized)
  
  ## if only one degree is wanted, return that
  if(length(degree) == 1){
    coefs <- rev.default(coefs)[[1]]
    p <- as.mpoly.polynomial(coefs, indeterminate)
    class(p) <- c("laguerre", "mpoly")
    attr(p, "laguerre") <- list(
      "degree" = length(coefs)-1, 
      "alpha" = alpha, 
      "indeterminate" = indeterminate, 
      "normalized" = normalized
    )
    return(p)
  }
  
  ## if several are wanted, return them
  coefs <- coefs[degree+1]
  ps <- lapply(coefs, function(polynomial){
    p <- as.mpoly.polynomial(polynomial, indeterminate)
    class(p) <- c("laguerre", "mpoly")
    attr(p, "laguerre") <- list(
      "degree" = length(coefs)-1, 
      "alpha" = alpha, 
      "indeterminate" = indeterminate, 
      "normalized" = normalized
    )
    p
  })
  class(ps) <- "mpolyList"
  ps
  
}

context("laguerre()")


test_that("basic", {
  
  expect_equal(
    laguerre(2),
    structure(
      list(
        c("coef" = 1),
        c("x" = 1, "coef" = -2),
        c("x" = 2, "coef" = .5)
      ), 
      "class" = c("laguerre", "mpoly"),
      "laguerre" = list(
        "degree" = 2, 
        "alpha" = 0, 
        "indeterminate" = "x",
        "normalized" = FALSE
      )
    )
  )
  
})



test_that("indeterminate", {
  
  expect_equal(
    laguerre(2, indeterminate = "t"),
    structure(
      list(
        c("coef" = 1),
        c("t" = 1, "coef" = -2),
        c("t" = 2, "coef" = .5)
      ), 
      "class" = c("laguerre", "mpoly"),
      "laguerre" = list(
        "degree" = 2, 
        "alpha" = 0, 
        "indeterminate" = "t",
        "normalized" = FALSE
      )
    )
  )
  
})




test_that("alpha", {
  
  expect_equal(
    laguerre(2, alpha = 3),
    structure(
      list(
        c("coef" = 10),
        c("x" = 1, "coef" = -5),
        c("x" = 2, "coef" = .5)
      ), 
      "class" = c("laguerre", "mpoly"),
      "laguerre" = list(
        "degree" = 2, 
        "alpha" = 3, 
        "indeterminate" = "x",
        "normalized" = FALSE
      )
    )
  )
  
})




test_that("normalized", {
  
  expect_equal(
    laguerre(2, alpha = 3, normalized = TRUE),
    structure(
      list(
        c("coef" = 1.290994),
        c("x" = 1, "coef" = -0.6454972),
        c("x" = 2, "coef" = 0.06454972)
      ), 
      "class" = c("laguerre", "mpoly"),
      "laguerre" = list(
        "degree" = 2, 
        "alpha" = 3, 
        "indeterminate" = "x",
        "normalized" = TRUE
      )
    ),
    tolerance = 1e-5
  )
  
})





test_that("vectorized", {
  
  expect_equal(
    laguerre(0:2),
    structure(
      list(
        structure(
          list(
            c("coef" = 1)
          ), 
          "class" = c("laguerre", "mpoly"),
          "laguerre" = list(
            "degree" = 2, 
            "alpha" = 0, 
            "indeterminate" = "x",
            "normalized" = FALSE
          )
        ),
        structure(
          list(
            c("coef" = 1),
            c("x" = 1, "coef" = -1)
          ), 
          "class" = c("laguerre", "mpoly"),
          "laguerre" = list(
            "degree" = 2, 
            "alpha" = 0, 
            "indeterminate" = "x",
            "normalized" = FALSE
          )
        ),
        structure(
          list(
            c("coef" = 1),
            c("x" = 1, "coef" = -2),
            c("x" = 2, "coef" = .5)
          ), 
          "class" = c("laguerre", "mpoly"),
          "laguerre" = list(
            "degree" = 2, 
            "alpha" = 0, 
            "indeterminate" = "x",
            "normalized" = FALSE
          )
        )
      ),
      "class" = "mpolyList"
    )
  )
  
})

#' Legendre polynomials
#'
#' Legendre polynomials as computed by orthopolynom.
#'
#' @param degree degree of polynomial
#' @param indeterminate indeterminate
#' @param normalized provide normalized coefficients
#' @return a mpoly object or mpolyList object
#' @author David Kahle calling code from the orthopolynom package
#' @seealso [orthopolynom::legendre.polynomials()],
#'   \url{https://en.wikipedia.org/wiki/Legendre_polynomials}
#' @export
#' @examples
#'
#' legendre(0)
#' legendre(1)
#' legendre(2)
#' legendre(3)
#' legendre(4)
#' legendre(5)
#' legendre(6)
#'
#' legendre(2)
#' legendre(2, normalized = TRUE)
#'
#' legendre(0:5)
#' legendre(0:5, normalized = TRUE)
#' legendre(0:5, indeterminate = "t")
#'
#'
#'
#' # visualize the legendre polynomials
#'
#' library(ggplot2); theme_set(theme_classic())
#' library(tidyr)
#'
#' s <- seq(-1, 1, length.out = 201)
#' N <- 5 # number of legendre polynomials to plot
#' (legPolys <- legendre(0:N))
#'
#' # see ?bernstein for a better understanding of
#' # how the code below works
#'
#' df <- data.frame(s, as.function(legPolys)(s))
#' names(df) <- c("x", paste0("P_", 0:N))
#' mdf <- gather(df, degree, value, -x)
#' qplot(x, value, data = mdf, geom = "line", color = degree)
#'
#'
#' # legendre polynomials and the QR decomposition
#' n <- 201
#' x <- seq(-1, 1, length.out = n)
#' mat <- cbind(1, x, x^2, x^3, x^4, x^5)
#' Q <- qr.Q(qr(mat))
#' df <- as.data.frame(cbind(x, Q))
#' names(df) <- c("x", 0:5)
#' mdf <- gather(df, degree, value, -x)
#' qplot(x, value, data = mdf, geom = "line", color = degree)
#' 
#' Q <- apply(Q, 2, function(x) x / x[n])
#' df <- as.data.frame(cbind(x, Q))
#' names(df) <- c("x", paste0("P_", 0:5))
#' mdf <- gather(df, degree, value, -x)
#' qplot(x, value, data = mdf, geom = "line", color = degree)
#' 
#' 
#' # chebyshev polynomials are orthogonal in two ways:
#' P2 <- as.function(legendre(2))
#' P3 <- as.function(legendre(3))
#' P4 <- as.function(legendre(4))
#' 
#' integrate(function(x) P2(x) * P3(x), lower = -1, upper = 1)
#' integrate(function(x) P2(x) * P4(x), lower = -1, upper = 1)
#' integrate(function(x) P3(x) * P4(x), lower = -1, upper = 1)
#' 
#' n <- 10000L
#' u <- runif(n, -1, 1)
#' 2 * mean(P2(u) * P3(u))
#' 2 * mean(P2(u) * P4(u))
#' 2 * mean(P3(u) * P4(u))
#' 
#' (2/n) * sum(P2(u) * P3(u))
#' (2/n) * sum(P2(u) * P4(u))
#' (2/n) * sum(P3(u) * P4(u))
#' 
legendre <- function(degree, indeterminate = "x", normalized = FALSE){
  
  stopifnot(all(is.wholenumber(degree)))
  stopifnot(all(degree >= 0))
  
  ## make coefs
  coefs <- legendre.polynomials(max(degree), normalized = normalized)
  
  ## if only one degree is wanted, return that
  if(length(degree) == 1){
    coefs <- rev.default(coefs)[[1]]
    p <- as.mpoly.polynomial(coefs, indeterminate)
    class(p) <- c("legendre", "mpoly")
    attr(p, "legendre") <- list(degree = length(coefs)-1, indeterminate = indeterminate, normalized = normalized)
    return(p)
  }
  
  ## if several are wanted, return them
  coefs <- coefs[degree+1]
  ps <- lapply(coefs, function(polynomial){
    p <- as.mpoly.polynomial(polynomial, indeterminate)
    class(p) <- c("legendre", "mpoly")
    attr(p, "legendre") <- list(degree = length(polynomial)-1, indeterminate = indeterminate, normalized = normalized)
    p
  })
  class(ps) <- "mpolyList"
  ps
  
}

context("legendre()")


test_that("basic", {
  
  expect_equal(
    legendre(3),
    structure(
      list(
        c("x" = 1, "coef" = -1.5),
        c("x" = 3, "coef" = 2.5)
      ), 
      "class" = c("legendre", "mpoly"),
      "legendre" = list(
        "degree" = 3, 
        "indeterminate" = "x",
        "normalized" = FALSE
      )
    )
  )
  
})






test_that("basic", {
  
  expect_equal(
    legendre(3, indeterminate = "t"),
    structure(
      list(
        c("t" = 1, "coef" = -1.5),
        c("t" = 3, "coef" = 2.5)
      ), 
      "class" = c("legendre", "mpoly"),
      "legendre" = list(
        "degree" = 3, 
        "indeterminate" = "t",
        "normalized" = FALSE
      )
    )
  )
  
})







test_that("normalized", {
  
  expect_equal(
    legendre(3, normalized = TRUE),
    structure(
      list(
        c("x" = 1, "coef" = -2.806243),
        c("x" = 3, "coef" = 4.677072)
      ), 
      "class" = c("legendre", "mpoly"),
      "legendre" = list(
        "degree" = 3, 
        "indeterminate" = "x",
        "normalized" = TRUE
      )
    ),
    tolerance = 1e-5
  )
  
})





test_that("vectorized", {
  
  expect_equal(
    legendre(0:2),
    structure(
      list(
        structure(
          list(
            c("coef" = 1)
          ), 
          "class" = c("legendre", "mpoly"),
          "legendre" = list(
            "degree" = 0, 
            "indeterminate" = "x",
            "normalized" = FALSE
          )
        ),
        structure(
          list(
            c("x" = 1, "coef" = 1)
          ), 
          "class" = c("legendre", "mpoly"),
          "legendre" = list(
            "degree" = 1, 
            "indeterminate" = "x",
            "normalized" = FALSE
          )
        ),
        structure(
          list(
            c("coef" = -.5),
            c("x" = 2, "coef" = 1.5)
          ), 
          "class" = c("legendre", "mpoly"),
          "legendre" = list(
            "degree" = 2, 
            "indeterminate" = "x",
            "normalized" = FALSE
          )
        )
      ),
      class = "mpolyList"
    )
  )
  
})

#' Round the coefficients of a polynomial
#'
#' Round the coefficients of an mpoly object.
#'
#' @param x an mpoly object
#' @param digits number of digits to round to
#' @return the rounded mpoly object
#' @author David Kahle \email{david@@kahle.io}
#' @seealso [mp()]
#' @export
#' @examples
#'
#' p <- mp("x + 3.14159265")^4
#' p
#' round(p)
#' round(p, 0)
#'
#' \dontrun{
#' library(plyr)
#' library(ggplot2)
#' library(stringr)
#'
#' n <- 101
#' s <- seq(-5, 5, length.out = n)
#'
#' # one dimensional case
#' df <- data.frame(x = s)
#' df <- mutate(df, y = -x^2 + 2*x - 3 + rnorm(n, 0, 2))
#' qplot(x, y, data = df)
#' mod <- lm(y ~ x + I(x^2), data = df)
#' p <- as.mpoly(mod)
#' qplot(x, y, data = df) +
#'   stat_function(fun = as.function(p), colour = 'red')
#'
#' p
#' round(p, 1)
#' qplot(x, y, data = df) +
#'   stat_function(fun = as.function(p), colour = 'red') +
#'   stat_function(fun = as.function(round(p,1)), colour = 'blue')
#'
#'
#' }
#' 
round.mpoly <- function(x, digits = 3){
  
  ## round coefficients
  p <- lapply(x, function(term){
    term["coef"] <- round(term["coef"], digits = digits)
    term
  })
  
  ## drop zero terms
  p <- Filter(function(v) v[["coef"]] != 0, p)
  
  ## class
  class(p) <- "mpoly"
  
  ## out
  p
}

#' Solve a univariate mpoly with polyroot
#'
#' Solve a univariate mpoly with polyroot
#'
#' @param mpoly an mpoly
#' @param real_only return only real solutions?
#' @export
#' @examples
#'
#' solve_unipoly(mp("x^2 - 1")) # check x = -1, 1
#' solve_unipoly(mp("x^2 - 1"), real_only = TRUE)
#' 
solve_unipoly <- function(mpoly, real_only = FALSE) {
  mpoly <- reorder(mpoly)
  degs <- vapply(mpoly, function(term) {
    if(length(term) == 1) return(0L)
    term[[1]]
  }, double(1))
  coefs <- sapply(mpoly, `[[`, "coef") # vapply errors?
  coef_vec <- rep(0L, degs[1]+1)
  names(coef_vec) <- 0:degs[1]
  for(k in 1:length(mpoly)) coef_vec[as.character(degs[k])] <- coefs[k]
  solns <- polyroot(coef_vec)
  if (real_only) {
    is.real <- function(x) Im(x) == 0
    solns <- Re(Filter(is.real, solns))
  }
  solns
}

context("solve_unipoly()")


test_that("solve_unipoly() works with real roots", {
  
  expect_equal(
    solve_unipoly(mp("x^2 - 1"), real_only = TRUE),
    c(1, -1)
  )
  
})


test_that("solve_unipoly() works generally", {
  
  expect_equal(
    solve_unipoly(mp("x^2 - 1")),
    c(1+0i, -1+0i)
  )
  
  expect_equal(
    solve_unipoly(mp("x^2 - 2")),
    c(1.41421356237309-0i, -1.41421356237309+0i)
  )
  
  expect_equal(
    solve_unipoly(mp("x^3 - 1")),
    c(1-0i, -0.5+0.866025403784439i, -0.5-0.866025403784439i)
  )
  
  expect_equal(
    solve_unipoly(mp("x^4 - 1")),
    c(0+1i, -1-0i, 0-1i, 1+0i)
  )
  
})

#' Bernstein polynomials
#'
#' Bernstein polynomials
#'
#' @param k Bernstein polynomial k
#' @param n Bernstein polynomial degree
#' @param indeterminate indeterminate
#' @return a mpoly object
#' @author David Kahle
#' @name bernstein
#' @examples
#'
#' bernstein(0, 0)
#'
#' bernstein(0, 1)
#' bernstein(1, 1)
#'
#' bernstein(0, 1, "t")
#'
#' bernstein(0:2, 2)
#' bernstein(0:3, 3)
#' bernstein(0:3, 3, "t")
#'
#'
#' bernstein(0:4, 4)
#' bernstein(0:10, 10)
#' bernstein(0:10, 10, "t")
#' bernstein(0:20, 20, "t")
#'
#' \dontrun{  # visualize the bernstein polynomials
#'
#' library(ggplot2); theme_set(theme_classic())
#' library(tidyr)
#'
#' s <- seq(0, 1, length.out = 101)
#' N <- 10 # number of bernstein polynomials to plot
#' (bernPolys <- bernstein(0:N, N))
#'
#' df <- data.frame(s, as.function(bernPolys)(s))
#' names(df) <- c("x", paste0("B_", 0:N))
#' head(df)
#'
#' mdf <- gather(df, degree, value, -x)
#' head(mdf)
#'
#' qplot(x, value, data = mdf, geom = "line", color = degree)
#'
#' }
#'
#' 







#' @rdname bernstein
#' @export
bernstein <- function(k, n, indeterminate = "x"){  
  
  stopifnot(k <= n)
  
  ## make it possible for vector k args
  if(length(k) > 1){
    listOPolys <- lapply(k, function(.) bernstein(., n, indeterminate))
    class(listOPolys) <- "mpolyList"
    return(listOPolys)
  }
  
  ## construct coefficients and degrees of terms
  m <- n - k  
  coefs <- choose(n, k) * (-1)^(0:m) * choose(m, 0:m)
  degs  <- k:n
  
  ## construct polynomial as list
  p <- Map(function(deg, coef) c(x = deg, coef = coef), degs, coefs)
  
  ## wipe out zeros
  p <- lapply(p, function(v) v[v != 0])
  
  ## class list
  class(p) <- c("bernstein", "mpoly")
  attr(p, "bernstein") <- list(k = k, n = n, indeterminate = indeterminate)

  ## swap and return
  swap(p, "x", indeterminate)
}

context("as.function.mpoly()")



test_that("univariate", {
  
  f <- as.function(mp("(3 - x)^2"), silent = TRUE)
  
  x <- 2
  expect_equal(f(x), (3 - x)^2)
  
  x <- 5
  expect_equal(f(x), (3 - x)^2)
  
  f <- as.function(mp("(3 - x)^2"), silent = TRUE)
  
  x <- 2:10
  expect_equal(f(x), (3 - x)^2)

})



test_that("vector = TRUE", {
  
  f <- as.function(mp("3 x + y^2"), vector = TRUE, silent = TRUE)
  
  expect_equal(f(c(1,2)), 3*(1) + (2)^2)
  
})


test_that("vector = FALSE", {
  
  f <- as.function(mp("3 x + y^2"), vector = FALSE, silent = TRUE)
  
  expect_equal(f(1, 2), 3*(1) + (2)^2)
  
})


test_that("constant mpoly", {
  
  f <- as.function(mp("3"), silent = TRUE)
  
  expect_equal(f(1), 3)
  expect_equal(f(1:4), rep(3, 4))
  
})


test_that("bernstein", {
  
  s <- seq(0, 1, .01)
  
  expect_equal(
    as.function(bernstein(1, 2))(s),
    2*s - 2*s^2
  )
  
  s <- seq(0, 1, .01)
  
  expect_equal(
    as.function(bernstein(1, 2))(-3:3),
    2*(-3:3) - 2*(-3:3)^2
  )
  
})

context("bernstein()")


#' bernstein(0, 0)
#'
#' bernstein(0, 1)
#' bernstein(1, 1)

test_that("bernstein() works", {
  
  expect_equal(
    bernstein(0, 0),
    structure(
      list(c("coef" = 1)), 
      class = c("bernstein", "mpoly"),
      bernstein = list("k" = 0, "n" = 0, "indeterminate" = "x")
    )
  )
  
  
  expect_equal(
    bernstein(0, 1),
    structure(
      list(c("coef" = 1), c("x" = 1, "coef" = -1)), 
      class = c("bernstein", "mpoly"),
      bernstein = list("k" = 0, "n" = 1, "indeterminate" = "x")
    )
  )
  
  
  expect_equal(
    bernstein(1, 1),
    structure(
      list(c("x" = 1, "coef" = 1)), 
      class = c("bernstein", "mpoly"),
      bernstein = list("k" = 1, "n" = 1, "indeterminate" = "x")
    )
  )
  
  
})






test_that("bernstein() chooses variables correctly", {
  
  
  expect_equal(
    bernstein(0, 1, "t"),
    structure(
      list(c("coef" = 1), c("t" = 1, "coef" = -1)), 
      class = c("bernstein", "mpoly"),
      bernstein = list("k" = 0, "n" = 1, "indeterminate" = "t")
    )
  )
  
  
})

#' Lissajous polynomials
#'
#' The Lissajous polynomials are the implicit (variety) descriptions of the
#' image of the parametric map x = cos(m t + p), y = sin(n t + q).
#'
#' @param m,n,p,q Trigonometric coefficients, see examples for description
#' @param digits The number of digits to round coefficients to, see
#'   [round.mpoly()]. This is useful for cleaning terms that are numerically
#'   nonzero, but should be.
#' @return a mpoly object
#' @seealso [chebyshev()], Merino, J. C (2003). Lissajous figures and Chebyshev
#'   polynomials. The College Mathematics Journal, 34(2), pp. 122-127.
#' @export
#' @examples
#'
#' lissajous(3, 2,  -pi/2, 0)
#' lissajous(4, 3,  -pi/2, 0)
#' 
lissajous <- function(m, n, p, q, digits = 3) {
  
  delta <- det(matrix(c(m, n, p, q), nrow = 2))
  Tnx <- chebyshev(n, indeterminate = "x")
  Tmy <- chebyshev(m, indeterminate = "y")
  is.even <- function(n) (n %% 2L) == 0L
  is.zero <- function(x, tol = 1e-5) abs(x) <= tol
  
  if (is.even(m)) {
    
    if (is.zero(sin(delta))) {
      p <- cos(delta)*Tnx - (-1)^(m/2)*Tmy
    } else {
      p <- Tnx^2 + Tmy^2 - 2*(-1)^(m/2)*Tnx*Tmy*cos(delta) - sin(delta)^2
    }
    
  } else { # m odd
    
    if (is.zero(cos(delta))) {
      p <- sin(delta)*Tnx - (-1)^((m-1)/2)*Tmy
    } else {
      p <- Tnx^2 + Tmy^2 - 2*(-1)^((m-1)/2)*Tnx*Tmy*sin(delta) - cos(delta)^2
    }  
    
  }
  
  round(p, digits)
  
}

#' Arithmetic with multivariate polynomials
#'
#' Arithmetic with multivariate polynomials
#'
#' @param e1 an object of class mpoly
#' @param e2 an object of class mpoly
#' @return object of class mpoly
#' @name mpolyArithmetic
#' @examples
#'
#' p <- mp("x + y")
#' p + p
#' p - p
#' p * p
#' p^2
#' p^10
#'
#'
#' mp("(x+1)^10")
#' p + 1
#' 2*p
#'
#'
#' 
NULL








#' @rdname mpolyArithmetic
#' @export
`+.mpoly` <- function(e1, e2){
  
  # strip to lists
  e1 <- unclass(e1)
  e2 <- unclass(e2)
  
  # if either is constant, mpoly it
  if(!is.list(e1)){
    stopifnot(is.numeric(e1) && length(e1) == 1L)
    e1 <- list(c(coef = e1))
  }
  
  if(!is.list(e2)){
    stopifnot(is.numeric(e2) && length(e2) == 1L)
    e2 <- list(c(coef = e2))
  }

  # let mpoly do the heavy lifting
  mpoly(c(e1, e2)) 
}







#' @rdname mpolyArithmetic
#' @export
`-.mpoly` <- function(e1, e2){
  
  # deal with constants
  if(is.numeric(e2)) {
    e2 <- mpoly(list(c("coef" = e2)))
  }
  
  # flip coefficients of each term
  e2 <- lapply(e2, function(v){
    v["coef"] <- -v["coef"]
    v
  })
  class(e2) <- "mpoly"
  
  # add
  e1 + e2	
}






#' @rdname mpolyArithmetic
#' @export
`*.mpoly` <- function(e1, e2){
  
  # allow for multiplication by a constant
  if(is.numeric(e1) && length(e1) == 1) e1 <- mpoly(list(c(coef = e1)))
  if(is.numeric(e2) && length(e2) == 1) e2 <- mpoly(list(c(coef = e2)))
  
  # argument check
  stopifnot(is.mpoly(e1))
  stopifnot(is.mpoly(e2))
    
  # multiply
  list <- lapply(e1, function(v){
    lapply(e2, function(z){
      c(v, z)  
    })	
  })
    
  # return
  mpoly( unlist(list, recursive = FALSE) )
}	











#' @rdname mpolyArithmetic
#' @export
`^.mpoly` <- function(e1, e2){
  
  if(!is.mpoly(e1)) stop('e1 must be of class mpoly.', call. = FALSE)
  
  if(!is.wholenumber(e2) || e2 < 0) stop('exponent must be a positive integer.', call. = FALSE)
  
  out <- mpoly(list(c(coef = 1)))
  
  if(e2 == 0) return(out)
    
  for(k in 1:e2) out <- out * e1
  
  out
}

#' Compute partial derivatives of a multivariate polynomial.
#'
#' This is a deriv method for mpoly objects.  It does not call the
#' [stats::deriv()].
#'
#' @param expr an object of class mpoly
#' @param var character - the partial derivative desired
#' @param ... any additional arguments
#' @param bring_power_down if `FALSE`, x^n -> x^(n-1), not n x^(n-1)
#' @return An object of class mpoly or mpolyList.
#' @export
#' @examples
#' p <- mp("x y + y z + z^2")
#' deriv(p, "x")
#' deriv(p, "y")
#' deriv(p, "z")
#' deriv(p, "t")
#' deriv(p, c("x","y","z"))
#' 
#' is.mpoly(deriv(p, "x"))
#' is.mpolyList( deriv(p, c("x","y","z")) )
#' 
#' p <- mp("x^5")
#' deriv(p, "x")
#' deriv(p, "x", bring_power_down = FALSE)
#' 
deriv.mpoly <- function(expr, var, ..., bring_power_down = TRUE){

  # argument checks	
  if (missing(var)) stop("var must be specified, see ?deriv.mpoly", call. = FALSE)
  stopifnot(is.character(var))
  
  
  # if many vars provided
  if(length(var) > 1){
    mpolyList <- lapply(as.list(var), function(var){
      deriv(expr, var = var, ...)
    })
    class(mpolyList) <- "mpolyList"  
    return(mpolyList)    
  }
  
	
  # if the variable is not in the polynomial, return zero
  if(!(var %in% vars(expr))){
    return( mpoly(list(c(coef = 0))) )
  }
	
  expr <- unclass(expr)
  
  # take derivative
  expr <- lapply(expr, function(v){
  	if(length(v) == 1) return(c(coef = 0))
    p <- length(v)
    if(!(var %in% names(v[1:p]))) return(c(coef = 0))
    if (bring_power_down) v["coef"] <- unname(v[var]) * v["coef"]
    v[var] <- v[var] - 1
    v
  })
  
  # return
  mpoly(expr)

}


#' Compute gradient of a multivariate polynomial.
#'
#' This is a wrapper for deriv.mpoly.
#'
#' @param mpoly an object of class mpoly
#' @seealso [deriv.mpoly()]
#' @return An object of class mpoly or mpolyList.
#' @export
#' @examples
#' m <- mp("x y + y z + z^2")
#' gradient(m)
#' 
#' 
#' # gradient descent illustration using the symbolically
#' # computed gradient of the rosenbrock function (shifted)
#' rosenbrock <- mp("(1 - x)^2 + 100 (y - x^2)^2")
#' fn <- as.function(rosenbrock)
#' (rosenbrock_gradient <- gradient(rosenbrock))
#' gr <- as.function(rosenbrock_gradient)
#' 
#' # visualize the function
#' library(ggplot2)
#' s <- seq(-5, 5, .05)
#' df <- expand.grid(x = s, y = s)
#' df$z <- apply(df, 1, fn)
#' ggplot(df, aes(x = x, y = y)) +
#'   geom_raster(aes(fill = z + 1e-10)) +
#'   scale_fill_continuous(trans = "log10")
#'   
#' # run the gradient descent algorithm using line-search
#' # step sizes computed with optimize()
#' current <- steps <- c(-3,-4)
#' change <- 1
#' tol <- 1e-5
#' while(change > tol){
#'   last  <- current
#'   delta <- optimize(
#'     function(delta) fn(current - delta*gr(current)),
#'     interval = c(1e-10, .1)
#'   )$minimum
#'   current <- current - delta*gr(current)
#'   steps   <- unname(rbind(steps, current))
#'   change  <- abs(fn(current) - fn(last))
#' }
#' steps <- as.data.frame(steps)
#' names(steps) <- c("x", "y")
#'   
#' # visualize steps, note the optim at c(1,1)
#' # the routine took 5748 steps
#' ggplot(df, aes(x = x, y = y)) +
#'   geom_raster(aes(fill = z + 1e-10)) +
#'   geom_path(data = steps, color = "red") +
#'   geom_point(data = steps, color = "red", size = .5) +
#'   scale_fill_continuous(trans = "log10")
#' 
#' # it gets to the general region of space quickly
#' # but once it gets on the right arc, it's terrible
#' # here's what the end game looks like
#' last_steps <- tail(steps, 100)
#' rngx <- range(last_steps$x); sx <- seq(rngx[1], rngx[2], length.out = 201)
#' rngy <- range(last_steps$y); sy <- seq(rngy[1], rngy[2], length.out = 201)
#' df <- expand.grid(x = sx, y = sy)
#' df$z <- apply(df, 1, fn) 
#' ggplot(df, aes(x = x, y = y)) +
#'   geom_raster(aes(fill = z)) +
#'   geom_path(data = last_steps, color = "red", size = .25) +
#'   geom_point(data = last_steps, color = "red", size = 1) +
#'   scale_fill_continuous(trans = "log10") 
#'   
#' 
gradient <- function(mpoly) deriv(mpoly, var = vars(mpoly))

context("deriv()")


test_that("basic", {
  
  expect_equal(
    deriv(mp("x y + y z + z^2"), "x"),
    mp("y")
  )
  
  expect_equal(
    deriv(mp("x y + y z + z^2"), "y"),
    mp("x + z")
  )
  
  expect_equal(
    deriv(mp("x y + y z + z^2"), "z"),
    mp("y + 2 z")
  )
  
  expect_equal(
    deriv(mp("x y + y z + z^2"), "t"),
    mp("0")
  )
  
  expect_error(
    deriv(mp("x y + y z + z^2")),
    "var must be specified, see ?deriv.mpoly",
    fixed = TRUE
  )
  
})





test_that("vector indeterminates", {
  
  expect_equal(
    deriv(mp("x y + y z + z^2"), c("x", "y")),
    mp(c("y", "x + z"))
  )
  
})






test_that("bring_down_power works", {
  
  p <- mp("x^5")
  
  expect_equal(
    mp("x^4"),
    deriv(p, "x", bring_power_down = FALSE)
  )
  
})



test_that("gradient", {
  
  expect_equal(
    gradient(mp("x y + y z + z^2")),
    mp(c("y", "x + z", "y + 2 z"))
  )
  
  
})

#' Bernstein polynomial approximation
#'
#' Bernstein polynomial approximation
#'
#' @param f the function to approximate
#' @param n Bernstein polynomial degree
#' @param lower lower bound for approximation
#' @param upper upper bound for approximation
#' @param indeterminate indeterminate
#' @param ... ...
#' @return a mpoly object
#' @author David Kahle
#' @name bernstein-approx
#' @examples
#'
#'
#'
#'
#'
#' \dontrun{  # visualize the bernstein polynomials
#'
#' library(ggplot2); theme_set(theme_bw())
#' library(reshape2)
#'
#'
#'
#'
#' f <- function(x) sin(2*pi*x)
#' p <- bernstein_approx(f, 20)
#' round(p, 3)
#'
#' x <- seq(0, 1, length.out = 101)
#' df <- data.frame(
#'   x = rep(x, 2),
#'   y = c(f(x), as.function(p)(x)),
#'   which = rep(c("actual", "approx"), each = 101)
#' )
#' qplot(x, y, data = df, geom = "line", color = which)
#'
#'
#'
#'
#'
#'
#' p <- bernstein_approx(sin, 20, pi/2, 1.5*pi)
#' round(p, 4)
#'
#' x <- seq(0, 2*pi, length.out = 101)
#' df <- data.frame(
#'   x = rep(x, 2),
#'   y = c(sin(x), as.function(p)(x)),
#'   which = rep(c("actual", "approx"), each = 101)
#' )
#' qplot(x, y, data = df, geom = "line", color = which)
#'
#'
#'
#'
#'
#'
#'
#'
#' p <- bernstein_approx(dnorm, 15, -1.25, 1.25)
#' round(p, 4)
#'
#' x <- seq(-3, 3, length.out = 101)
#' df <- data.frame(
#'   x = rep(x, 2),
#'   y = c(dnorm(x), as.function(p)(x)),
#'   which = rep(c("actual", "approx"), each = 101)
#' )
#' qplot(x, y, data = df, geom = "line", color = which)
#'
#'
#'
#'
#'
#'
#' }
#' 


#' @rdname bernstein-approx
#' @export
bernstein_approx <- function(f, n, lower = 0, upper = 1, indeterminate = "x"){  
  
  ## compute support and determine weights
  s <- (0:n)/n
  fscaled <- function(.) f( (upper-lower)*. + lower )
  weights <- as.list(fscaled(s))
  
  ## convert weights to mpolyList
  weights <- lapply(weights, function(x) mpoly(list(c(coef = x))))
  class(weights) <- "mpolyList"
  
  ## multiply weights by basis
  approxPoly <- Reduce(`+`, weights * bernstein(0:n, n, "temp"))  
  
  ## compute plugin and plug in
  pluginPoly <- (upper-lower)^-1 * (mp(indeterminate) + -1*lower)
  plug(approxPoly, "temp", pluginPoly)
  
}


#' @rdname bernstein-approx
#' @export
bernsteinApprox <- function(...) {
  .Deprecated("bernstein_approx")
  bernstein_approx(...)
}

context("bernstein_approx()")


test_that("bernstein_approx() works", {
  
  expect_equal(
    bernstein_approx(function(x) x^4, 2L),
    mp(".125 x + .875 x^2")
  )
  
  expect_equal(
    suppressWarnings(bernsteinApprox(function(x) x^4, 2L)),
    mp(".125 x + .875 x^2")
  )
  
})

#' Multivariate polynomials in R.
#'
#' A package for symbolic computation and more with multivariate polynomials
#'
#' @importFrom stringr str_c str_detect str_sub str_replace_all str_replace
#'   str_count str_dup str_extract str_extract_all str_locate str_locate_all
#'   str_split str_sub<- str_trim fixed str_remove str_remove_all
#' @importFrom stringi stri_c stri_replace_all_fixed stri_sub stri_length
#'   stri_replace_all_regex stri_sub
#' @importFrom partitions parts
#' @importFrom plyr ddply
#' @importFrom stats deriv reorder coef
#' @importFrom ggplot2 ggplot aes
#' @importFrom polynom polynomial
#' @importFrom orthopolynom chebyshev.c.polynomials chebyshev.s.polynomials
#'   chebyshev.t.polynomials chebyshev.u.polynomials glaguerre.polynomials
#'   hermite.h.polynomials hermite.he.polynomials jacobi.g.polynomials
#'   jacobi.p.polynomials legendre.polynomials
#' @importFrom graphics contour par
#' @importFrom tidyr gather
#' @name mpoly
#' @aliases mpoly mpoly-package
#' @keywords internal
"_PACKAGE"

#' Determine all n-tuples using the elements of a set.
#'
#' Determine all n-tuples using the elements of a set.  This is really just a
#' simple wrapper for expand.grid, so it is not optimized.
#'
#' @param set a set
#' @param n length of each tuple
#' @param repeats if set contains duplicates, should the result?
#' @param list tuples as list?
#' @return a matrix whose rows are the n-tuples
#' @export
#' @examples
#'
#' tuples(1:2, 3)
#' tuples(1:2, 3, list = TRUE)
#'
#' apply(tuples(c("x","y","z"), 3), 1, paste, collapse = "")
#'
#' # multinomial coefficients
#' r <- 2 # number of variables, e.g. x, y
#' n <- 2 # power, e.g. (x+y)^2
#' apply(burst(n,r), 1, function(v) factorial(n)/ prod(factorial(v))) # x, y, xy
#' mp("x + y")^n
#'
#' r <- 2 # number of variables, e.g. x, y
#' n <- 3 # power, e.g. (x+y)^3
#' apply(burst(n,r), 1, function(v) factorial(n)/ prod(factorial(v)))
#' mp("x + y")^n
#'
#' r <- 3 # number of variables, e.g. x, y, z
#' n <- 2 # power, e.g. (x+y+z)^2
#' apply(burst(n,r), 1, function(v) factorial(n)/ prod(factorial(v))) # x, y, z, xy, xz, yz
#' mp("x + y + z")^n
#'
#' 
tuples <- function(set, n = length(set), repeats = FALSE, list = FALSE){

  ## determine how big the output will be
  r <- length(set)	
  nCombos <- r^n
  
  ## expand.grid really does all the work in this function, so
  ## setup the interior part of expand.grid.  
  ## sets4call looks like "set2, set2, set2" with n = 3
  out <- do.call(expand.grid, replicate(n, set, simplify = FALSE))
  out <- unname(as.matrix(out))
  out <- out[,n:1]
  
  ## delete duplicates
  if(!repeats) out <- unique(out)

  ## do list
  if(list) out <- unname(split(out, rep(1:nCombos, n)))
  
  ## return
  out
}

context("tuples()")


test_that("basic tuples() functionality works", {
  
  expect_equal(
    tuples(1:2, 3),
    structure(c(1L, 1L, 1L, 1L, 2L, 2L, 2L, 2L, 1L, 1L, 2L, 2L, 1L, 
      1L, 2L, 2L, 1L, 2L, 1L, 2L, 1L, 2L, 1L, 2L), .Dim = c(8L, 3L))
  )
  
})




test_that("tuples() list = TRUEs properly", {
  
  expect_equal(
    tuples(1:2, 3, list = TRUE),
    list(
      c(1L, 1L, 1L), c(1L, 1L, 2L), c(1L, 2L, 1L), c(1L, 2L, 2L), 
      c(2L, 1L, 1L), c(2L, 1L, 2L), c(2L, 2L, 1L), c(2L, 2L, 2L)
    )
  )
  
})




test_that("tuples() repeats argument works properly", {
  
  expect_equal(
    tuples(c(1, 2, 2), 2, repeats = TRUE),
    structure(c(1, 1, 1, 2, 2, 2, 2, 2, 2, 1, 2, 2, 1, 2, 2, 1, 2, 2), .Dim = c(9L, 2L))
  )
  
  expect_equal(
    tuples(c(1, 2, 2), 2, repeats = FALSE),
    structure(c(1, 1, 2, 2, 1, 2, 1, 2), .Dim = c(4L, 2L))
  )
  
})

#' Define an mpoly object.
#'
#' mpoly is the most basic function used to create objects of class mpoly.
#' However, it is not a general purpose function; for that see [mp()].
#'
#' @param list a list from which to construct an mpoly object
#' @param varorder (optional) a character vector setting the intrinsic variable
#'   order of the polynomial
#' @return Object of class mpoly.
#' @author David Kahle \email{david@@kahle.io}
#' @seealso [mp()]
#' @export
#' @examples
#' 
#' list <- list(
#'   c(coef = 1),
#'   c(x = 1, coef = 1)
#' )
#' mpoly(list)
#' 
#' 
#' list <- list(
#'   c(x = 1, coef = 1, y = 0),
#'   c(x = 0, y = 1, coef = 2),
#'   c(y = 1, coef = -6),
#'   c(z = 1, coef = -3, x = 2),
#'   c(x = 1, coef = 0, x = 3),
#'   c(t = 1, coef = 4, t = 2, y = 4),
#'   c(x = 1),
#'   c(x = 1),
#'   c(coef = 5),
#'   c(coef = 5),
#'   c(coef = -5)
#' )
#'
#' mpoly(list) # 3 x  -  4 y  -  3 x^2 z  +  4 y^4 t^3  +  5
#' mpoly(list, varorder = c("y", "z", "t", "x"))
#'
#' list <- list(  c(x = 5, x = 2, coef = 5, coef = 6, y = 0) )
#' mpoly(list)
#'
#' 
mpoly <- function(list, varorder){

  ## argument checking
  if(!is.list(list)) stop("input to mpoly must be a list.", call. = FALSE)
  
  if(!all(vapply(list, is.numeric, logical(1)))){
    stop("each element of list must be of type numeric.", call. = FALSE)  
  }
  
  if(any( unlist(lapply(list, function(v) nchar(names(v))), use.names = FALSE) == 0 )){
    stop("each element of list must be named for every element.", call. = FALSE)  
  }    
  
  flat_list <- unlist(list)
  flat_list <- flat_list[names(flat_list) != "coef"]
  if (!all(is.wholenumber(flat_list)) || any(flat_list < 0)){
    stop("degrees must be nonnegative integers.", call. = FALSE)
  }  
  
  
  # give terms without a coefficient a coef of 1
  add_coef_if_missing <- function(v){
    if (any("coef" == names(v))) return(v)
    c(v, "coef" = 1)
  }
  list <- lapply(list, add_coef_if_missing)
  
  
  
  ## organize 
    
  # remove terms with coef 0
  list <- filter_out_zero_terms(list)
  
  
  # remove 0 degrees, combine like degrees, single coef as rightmost element
  list <- lapply(list, function(v){  	
    
    # separate vardegs from coefs
    coef_ndx <- which(names(v) == "coef")
    coefs <- v[coef_ndx]    
    v     <- v[-coef_ndx]
  	
  	# combine like degrees (sum)
  	if(length(names(v)) != length(unique(names(v)))) v <- fast_named_vec_tapply(v, sum)   
  	
  	# combine like coefficients (product)
  	coefs <- c(coef = prod(coefs))
  	
  	# remove zero degree elements, combine and return  
    c(v[v != 0], coefs)
    
  })
  
  
  
  ## set intrinsic varorder - done again after 0 degrees are removed
  vars <- unique(names(flat_list))
  
  
  # deal with varorder argument
  if (!missing(varorder)) {  	
    if( !setequal(vars, varorder) ){
      stop("If specified varorder must be a permutation of ", stri_c(vars, collapse = ", "), call. = FALSE)	
    }    
    vars <- varorder
  }

  
  # sort variables in terms
  list <- lapply(list, function(v){
    p <- length(v) - 1L
    if(p == 0L) return(v)
    c( (v[1:p])[intersect(vars, names(v[1:p]))], v["coef"] )
  })
  
   
   
  ## prepare to check if like terms are present
  monomials <- vapply(list, function(v){
  	p <- length(v) - 1 # remove coef on monomials
    paste(names(v[1:p]), v[1:p],  sep = "", collapse = "")
  }, character(1))  
  # e.g. c("x1", "y1", "y1", "x2z1", "y4t3", "x1", "x1", "coef5")
  
  unique_monomials <- unique(monomials)
  
  
  
  ## check if like terms are present and, if so, correct  
  if (length(monomials) != length(unique_monomials)) {

    matched_monomials <- match(monomials, unique_monomials)
    matched_monomials <- factor(matched_monomials, levels = 1:max(matched_monomials))
    ndcs2combine      <- split.default(1:length(list), matched_monomials)
    
    list <- lapply(ndcs2combine, function(v){
      if(length(v) == 1) return(list[[v]])      
      coef <- sum(vapply(list[v], `[`, double(1), length(list[[v[1]]])))
      v <- list[[v[1]]]
      v["coef"] <- coef
      v
    })
    
    names(list) <- NULL # i.e. list <- unname(list)
  }
  
  
  
  ## combine constant terms
  ## mpoly(list(c(x = 1, coef = 1), c(coef = 1), c(coef = 2)))    
  non_constant_terms <- fast_filter(is_not_length_one, list)
  constant_terms     <- fast_filter(is_length_one, list)
  
  if (length(constant_terms) > 0) {
    list <- c(
      non_constant_terms, 
      list(Reduce(`+`, constant_terms))
    )  
  }
  
  
  
  ## re-organize after like-terms combined
  list <- filter_out_zero_terms(list)

  
  
  ## return classed list
  class(list) <- "mpoly"
  list  
}











filter_out_zero_terms <- function(list){
  
  # a term is zero if any of its coefficients are 0
  has_non_zero_coef <- function(v) all(v["coef"] != 0)
  
  # remove terms with coef 0
  list <- fast_filter(has_non_zero_coef, list)
  
  # if all terms have been eliminated, recreate it with a 0 coef
  if(length(list) == 0) list <- list(c(coef = 0))  
  
  # return
  list  
}







is_length_one    <- function(x) length(x) == 1L
is_not_length_one <- function(x) length(x) > 1L
fast_filter     <- function(f, x) x[vapply(x, f, logical(1))]







fast_named_vec_tapply <- function(x, f, type = double(1)){
  uniqueNames   <- unique(names(x))
  matched_names <- match(names(x), uniqueNames) # indices
  matched_names <- factor(matched_names, levels = 1:max(matched_names))
  groupIndices  <- split.default(1:length(x), matched_names)
  out <- vapply(groupIndices, function(ndcs) f(x[ndcs]), type)
  names(out) <- uniqueNames
  out
}
# x <- 1:10
# names(x) <- sample(letters[1:3], 10, replace = TRUE)
# fast_named_vec_tapply(x, sum)
# tapply(x, names(x), sum)

context("mpoly()")

test_that("mpoly() flags non-list arguments",{
  expect_error(mpoly(1:5), "input to mpoly must be a list.")
})



context("mp() is working properly")


test_that("mp() splices *'s properly", {
  
  expect_equal(
    mp("x y"),
    mpoly(list(c(x = 1, y = 1, coef = 1)))
  )
  
  expect_equal(
    mp("x y z"),
    mpoly(list(c(x = 1, y = 1, z = 1, coef = 1)))
  )
  
  expect_equal(
    mp("a b c d e"),
    mpoly(list(c(a = 1, b = 1, c = 1, d = 1, e = 1, coef = 1)))
  )
  
  expect_equal(
    mp("x1 x2"),
    mpoly(list(c(x1 = 1, x2 = 1, coef = 1)))
  )
  
  expect_equal(
    mp("x1a x2a"),
    mpoly(list(c(x1a = 1, x2a = 1, coef = 1)))
  )
  
  expect_equal(
    mp("x1a^2 x2a^3"),
    mpoly(list(c(x1a = 2, x2a = 3, coef = 1)))
  )
  
})










test_that("mp() parses character vectors properly", {
  
  expect_equal(
    mp(c("2 x y", "-3 x y z")),
    structure(
      list(
        structure(list(c(x = 1, y = 1, coef = 2)), class = "mpoly"), 
        structure(list(c(x = 1, y = 1, z = 1, coef = -3)), class = "mpoly")
      ),
      class = "mpolyList"
    )
  )
  
  expect_equal(
    mp(c("x y", "x y z", "a^3 b c^2 d")),
    structure(
      list(
        structure(list(c(x = 1, y = 1, coef = 1)), class = "mpoly"), 
        structure(list(c(x = 1, y = 1, z = 1, coef = 1)), class = "mpoly"),
        structure(list(c(a = 3, b = 1, c = 2, d = 1, coef = 1)), class = "mpoly")
      ),
      class = "mpolyList"
    )
  )
  
  
  
})









test_that("mp() obeys varorder", {
  
  expect_equal(
    mp("3 y^2 x^3", varorder = c("x", "y")),
    structure(list(c(x = 3, y = 2, coef = 3)), class = "mpoly")
  )
  
})












# 
# 
# 
# 
# 
# 
# 
# test_that("mpoly only allows named whole number exponents",{
#   
#   expect_error(
#     mpoly(list(c(x = "a", coef = 1))), 
#     "each element of list must be of type numeric."
#   )
#   
#   expect_error(
#     mpoly(list(c(x = 1,     1, coef = 2))), 
#     "each element of list must be named for every element."
#   )
#   
#   expect_error(
#     mpoly(list(c(x = 1, y = 1.5, coef = 2))), 
#     "degrees must be nonnegative integers."
#   )
#   
#   expect_error(
#     mpoly(list(c(x = 1, y = -2, coef = 2))), 
#     "degrees must be nonnegative integers."
#   )
#   
#   expect_error(
#     mpoly(list(c(x = 1, y = 2, coef = 2)), varorder = "x"), 
#     "if specified varorder must be a permutation of"
#   )
#   
# })
# 
# 
# 
# 
# 
# 
# 
# 
# 
# 
# 
# 
# 
# 
# 
# 
# 
# 
# context("Quality of polynomial parsing")
# 
# test_that("parse_nonparenthetical_term works", {
#   
#   f <- parse_nonparenthetical_term
#   m <- mpoly
#   
#   expect_equal(f("0"), m(list(c(coef = 0))))
#   expect_equal(f("1"), m(list(c(coef = 1))))
#   expect_equal(f("+1"), m(list(c(coef = 1))))
#   expect_equal(f("-1"), m(list(c(coef = -1))))
#   
#   expect_equal(f("1.5"), m(list(c(coef = 1.5))))
#   expect_equal(f("+1.5"), m(list(c(coef = 1.5))))
#   expect_equal(f("-1.5"), m(list(c(coef = -1.5))))
#   
#   expect_equal(f(".5"), m(list(c(coef = .5))))
#   expect_equal(f("+.5"), m(list(c(coef = .5))))
#   expect_equal(f("-.5"), m(list(c(coef = -.5))))
#   
#   expect_equal(f("2 2"), m(list(c(coef = 4))))
#   expect_equal(f("-2 2"), m(list(c(coef = -4))))
#   expect_error(f("-2 -2"), "Negative signs are only allowed at the beginning of terms.")
#   
#   expect_equal(f(".5 .5"), m(list(c(coef = .25))))
#   expect_equal(f(".5 2"), m(list(c(coef = 1))))  
#   expect_equal(f("-.5 .5"), m(list(c(coef = -.25))))
#   expect_equal(f("-.5 2"), m(list(c(coef = -1))))  
#   expect_equal(f("-2 .5"), m(list(c(coef = -1))))  
# 
#   expect_equal(f("2^2"), m(list(c(coef = 4))))
#   expect_equal(f("-2^2"), m(list(c(coef = -4))))
#   expect_equal(f(".5^2"), m(list(c(coef = .25))))
#   expect_equal(f("-.5^2"), m(list(c(coef = -.25))))
#  
#   expect_equal(f("0x"), m(list(c(coef = 0))))
#   expect_equal(f("-0x"), m(list(c(coef = 0))))
#   
#   expect_equal(f("0 x"), m(list(c(coef = 0))))
#   expect_equal(f("-0 x"), m(list(c(coef = 0))))
#   
#   expect_equal(f("x"), m(list(c(coef = 1, x = 1))))
#   # expect_equal(f("-x"), m(list(c(coef = -1, x = 1))))
#   
#   # expect_equal(f("- x"), m(list(c(coef = -1, x = 1))))
#   
#   expect_equal(f("2x"), m(list(c(coef = 2, x = 1))))
#   expect_equal(f("-2x"), m(list(c(coef = -2, x = 1))))  
#   expect_equal(f("- 2x"), m(list(c(coef = -2, x = 1))))  
#   
#   expect_equal(f(".2x"), m(list(c(coef = .2, x = 1))))
#   expect_equal(f("-.2x"), m(list(c(coef = -.2, x = 1))))  
#   expect_equal(f("- .2x"), m(list(c(coef = -.2, x = 1))))  
#   
#   expect_equal(f("2 x"), m(list(c(coef = 2, x = 1))))
#   expect_equal(f("-2 x"), m(list(c(coef = -2, x = 1))))  
#   expect_equal(f("- 2 x"), m(list(c(coef = -2, x = 1))))    
#   
#   expect_equal(f(".2 x"), m(list(c(coef = .2, x = 1))))
#   expect_equal(f("-.2 x"), m(list(c(coef = -.2, x = 1))))  
#   expect_equal(f("- .2 x"), m(list(c(coef = -.2, x = 1))))    
#   
#   expect_equal(f("x[1]^2"), m(list(c(coef = 1, "x[1]" = 2))))    
#   expect_equal(f("x[1] ^ 2"), m(list(c(coef = 1, "x[1]" = 2))))    
#   expect_equal(f("2 x[1] ^ 2"), m(list(c(coef = 2, "x[1]" = 2))))    
#   expect_equal(f("-2 x[1] ^ 2"), m(list(c(coef = -2, "x[1]" = 2))))    
#   
#   expect_equal(f("x1^2"), m(list(c(coef = 1, x1 = 2))))    
#   expect_equal(f("x1 ^ 2"), m(list(c(coef = 1, x1 = 2))))    
#   expect_equal(f("2 x1 ^ 2"), m(list(c(coef = 2, x1 = 2))))    
#   expect_equal(f("-2 x1 ^ 2"), m(list(c(coef = -2, x1 = 2))))    
#   
#   expect_equal(f("x^2"), m(list(c(coef = 1, x = 2))))
#   expect_equal(f("x ^ 2"), m(list(c(coef = 1, x = 2))))
#   expect_equal(f("x ^2"), m(list(c(coef = 1, x = 2))))
#   expect_equal(f("x^ 2"), m(list(c(coef = 1, x = 2))))
#   
#   expect_equal(f("x^0"), m(list(c(coef = 1))))
#   # expect_equal(f("-x^0"), m(list(c(coef = -1))))
#   
#   expect_equal(f("2x^2"), m(list(c(coef = 2, x = 2))))  
#   expect_equal(f("-2x^2"), m(list(c(coef = -2, x = 2))))  
#   
#   expect_equal(f("5^2x"), m(list(c(coef = 25, x = 1))))  
#   expect_equal(f("-5^2x"), m(list(c(coef = -25, x = 1))))  
#   expect_equal(f("5^2x^2"), m(list(c(coef = 25, x = 2))))  
#   expect_equal(f("-5^2x^2"), m(list(c(coef = -25, x = 2))))  
#   expect_equal(f("5^2x^2"), m(list(c(coef = 25, x = 2))))  
#   expect_equal(f(".5^2x^2"), m(list(c(coef = .25, x = 2))))  
#   
#   expect_equal(f("12xx 2 y 2x"), m(list(c(coef = 48, xx = 1, y = 1, x = 1))))
#   expect_equal(f("2 x y^2 3 2           3^2"), m(list(c(coef = 108, x = 1, y = 2))))  
#   expect_equal(f("x y .2"), m(list(c(coef = .2, x = 1, y = 1))))
# })
# 
# 
# 
# 
# 
# 
# 
# 
# 
# 
# 
# 
# 
# 
# 
# 
# 
# 
# 
# 
# 
# 
# 
# 
# test_that("parse_nonparenthetical_polynomial works", {
#   
#   f <- parse_nonparenthetical_polynomial
#   m <- mpoly
#   
#   ## the ones from before parse_nonparenthetical_term
#   expect_equal(f("0"), m(list(c(coef = 0))))
#   expect_equal(f("1"), m(list(c(coef = 1))))
#   expect_equal(f("-1"), m(list(c(coef = -1))))
#   
#   expect_equal(f("1.5"), m(list(c(coef = 1.5))))
#   expect_equal(f("-1.5"), m(list(c(coef = -1.5))))
#   
#   expect_equal(f(".5"), m(list(c(coef = .5))))
#   expect_equal(f("-.5"), m(list(c(coef = -.5))))
#   
#   expect_equal(f("2 2"), m(list(c(coef = 4))))
#   expect_equal(f("-2 2"), m(list(c(coef = -4))))
#   # expect_error(f("-2 -2"), "negative signs only allowed at the beginning of terms")
#   
#   expect_equal(f(".5 .5"), m(list(c(coef = .25))))
#   expect_equal(f(".5 2"), m(list(c(coef = 1))))  
#   expect_equal(f("-.5 .5"), m(list(c(coef = -.25))))
#   expect_equal(f("-.5 2"), m(list(c(coef = -1))))  
#   expect_equal(f("-2 .5"), m(list(c(coef = -1))))  
#   
#   expect_equal(f("2^2"), m(list(c(coef = 4))))
#   expect_equal(f("-2^2"), m(list(c(coef = -4))))
#   expect_equal(f(".5^2"), m(list(c(coef = .25))))
#   expect_equal(f("-.5^2"), m(list(c(coef = -.25))))
#   
#   expect_equal(f("0x"), m(list(c(coef = 0))))
#   expect_equal(f("-0x"), m(list(c(coef = 0))))
#   
#   expect_equal(f("0 x"), m(list(c(coef = 0))))
#   expect_equal(f("-0 x"), m(list(c(coef = 0))))
#   
#   expect_equal(f("x"), m(list(c(coef = 1, x = 1))))
#   # expect_equal(f("-x"), m(list(c(coef = -1, x = 1))))
#   
#   # expect_equal(f("- x"), m(list(c(coef = -1, x = 1))))
#   
#   expect_equal(f("2x"), m(list(c(coef = 2, x = 1))))
#   expect_equal(f("-2x"), m(list(c(coef = -2, x = 1))))  
#   expect_equal(f("- 2x"), m(list(c(coef = -2, x = 1))))  
#   
#   expect_equal(f(".2x"), m(list(c(coef = .2, x = 1))))
#   expect_equal(f("-.2x"), m(list(c(coef = -.2, x = 1))))  
#   expect_equal(f("- .2x"), m(list(c(coef = -.2, x = 1))))  
#   
#   expect_equal(f("2 x"), m(list(c(coef = 2, x = 1))))
#   expect_equal(f("-2 x"), m(list(c(coef = -2, x = 1))))  
#   expect_equal(f("- 2 x"), m(list(c(coef = -2, x = 1))))    
#   
#   expect_equal(f(".2 x"), m(list(c(coef = .2, x = 1))))
#   expect_equal(f("-.2 x"), m(list(c(coef = -.2, x = 1))))  
#   expect_equal(f("- .2 x"), m(list(c(coef = -.2, x = 1))))    
#     
#   expect_equal(f("x[1]^2"), m(list(c(coef = 1, "x[1]" = 2))))    
#   expect_equal(f("x[1] ^ 2"), m(list(c(coef = 1, "x[1]" = 2))))    
#   expect_equal(f("2 x[1] ^ 2"), m(list(c(coef = 2, "x[1]" = 2))))    
#   expect_equal(f("-2 x[1] ^ 2"), m(list(c(coef = -2, "x[1]" = 2))))      
#   
#   expect_equal(f("x1^2"), m(list(c(coef = 1, x1 = 2))))    
#   expect_equal(f("x1 ^ 2"), m(list(c(coef = 1, x1 = 2))))    
#   expect_equal(f("2 x1 ^ 2"), m(list(c(coef = 2, x1 = 2))))    
#   expect_equal(f("-2 x1 ^ 2"), m(list(c(coef = -2, x1 = 2))))    
#   
#   expect_equal(f("x^2"), m(list(c(coef = 1, x = 2))))
#   expect_equal(f("x ^ 2"), m(list(c(coef = 1, x = 2))))
#   expect_equal(f("x ^2"), m(list(c(coef = 1, x = 2))))
#   expect_equal(f("x^ 2"), m(list(c(coef = 1, x = 2))))
#   
#   expect_equal(f("x^0"), m(list(c(coef = 1))))
#   # expect_equal(f("-x^0"), m(list(c(coef = -1))))
#   
#   expect_equal(f("2x^2"), m(list(c(coef = 2, x = 2))))  
#   expect_equal(f("-2x^2"), m(list(c(coef = -2, x = 2))))  
#   
#   expect_equal(f("5^2x"), m(list(c(coef = 25, x = 1))))  
#   expect_equal(f("-5^2x"), m(list(c(coef = -25, x = 1))))  
#   expect_equal(f("5^2x^2"), m(list(c(coef = 25, x = 2))))  
#   expect_equal(f("-5^2x^2"), m(list(c(coef = -25, x = 2))))  
#   expect_equal(f("5^2x^2"), m(list(c(coef = 25, x = 2))))  
#   expect_equal(f(".5^2x^2"), m(list(c(coef = .25, x = 2))))  
#   
#   expect_equal(f("12xx 2 y 2x"), m(list(c(coef = 48, xx = 1, y = 1, x = 1))))
#   expect_equal(f("2 x y^2 3 2           3^2"), m(list(c(coef = 108, x = 1, y = 2))))  
#   expect_equal(f("x y .2"), m(list(c(coef = .2, x = 1, y = 1))))
#   
#   
#   
#   ## the new ones
#   expect_equal(f("1+1"), m(list(c(coef = 2))))
#   expect_equal(f("1 + 1"), m(list(c(coef = 2))))
#   expect_equal(f("1+ 1"), m(list(c(coef = 2))))
#   expect_equal(f("1 +1"), m(list(c(coef = 2))))
#   
#   expect_equal(f("1-1"), m(list(c(coef = 0))))
#   expect_equal(f("1 - 1"), m(list(c(coef = 0))))
#   expect_equal(f("1- 1"), m(list(c(coef = 0))))
#   # expect_equal(f("1 -1"), m(list(c(coef = 0))))
#   
#   expect_equal(f("1--1"), m(list(c(coef = 2))))
#   expect_equal(f("1-- 1"), m(list(c(coef = 2))))
#   expect_equal(f("1- -1"), m(list(c(coef = 2))))
#   expect_equal(f("1 --1"), m(list(c(coef = 2))))
#   expect_equal(f("1 -- 1"), m(list(c(coef = 2))))
#   expect_equal(f("1--  1"), m(list(c(coef = 2))))
#   expect_equal(f("1- - 1"), m(list(c(coef = 2))))
#   expect_equal(f("1-  -1"), m(list(c(coef = 2))))
#   expect_equal(f("1 -- 1"), m(list(c(coef = 2))))
#   expect_equal(f("1 - -1"), m(list(c(coef = 2))))
#   expect_equal(f("1  --1"), m(list(c(coef = 2))))
#   
#   expect_equal(f("1+-1"), m(list(c(coef = 0))))
#   expect_equal(f("1+- 1"), m(list(c(coef = 0))))
#   expect_equal(f("1+ -1"), m(list(c(coef = 0))))
#   expect_equal(f("1 +-1"), m(list(c(coef = 0))))
#   expect_equal(f("1 +- 1"), m(list(c(coef = 0))))
#   expect_equal(f("1+-  1"), m(list(c(coef = 0))))
#   expect_equal(f("1+ - 1"), m(list(c(coef = 0))))
#   expect_equal(f("1+  -1"), m(list(c(coef = 0))))
#   expect_equal(f("1 +- 1"), m(list(c(coef = 0))))
#   expect_equal(f("1 + -1"), m(list(c(coef = 0))))
#   expect_equal(f("1  +-1"), m(list(c(coef = 0))))
#   
#   expect_equal(f("-1+1"), m(list(c(coef = 0))))
#   expect_equal(f("- 1+1"), m(list(c(coef = 0))))
#   
#   
#   expect_equal(f("x+1"), m(list(c(coef = 1, x = 1), c(coef = 1))))
#   expect_equal(f("x + 1"), m(list(c(coef = 1, x = 1), c(coef = 1))))
#   expect_equal(f("x +1"), m(list(c(coef = 1, x = 1), c(coef = 1))))
#   expect_equal(f("x+ 1"), m(list(c(coef = 1, x = 1), c(coef = 1))))
#   
#   expect_equal(f("x-1"), m(list(c(coef = 1, x = 1), c(coef = -1))))
#   expect_equal(f("x - 1"), m(list(c(coef = 1, x = 1), c(coef = -1))))
#   expect_equal(f("x -1"), m(list(c(coef = 1, x = 1), c(coef = -1))))
#   expect_equal(f("x- 1"), m(list(c(coef = 1, x = 1), c(coef = -1))))
#   
#   expect_equal(f("-x+1"), m(list(c(coef = -1, x = 1), c(coef = 1))))
#   expect_equal(f("-x + 1"), m(list(c(coef = -1, x = 1), c(coef = 1))))
#   expect_equal(f("-x +1"), m(list(c(coef = -1, x = 1), c(coef = 1))))
#   expect_equal(f("-x+ 1"), m(list(c(coef = -1, x = 1), c(coef = 1))))
#   
#   expect_equal(f("-x-1"), m(list(c(coef = -1, x = 1), c(coef = -1))))
#   expect_equal(f("-x - 1"), m(list(c(coef = -1, x = 1), c(coef = -1))))
#   expect_equal(f("-x -1"), m(list(c(coef = -1, x = 1), c(coef = -1))))
#   expect_equal(f("-x- 1"), m(list(c(coef = -1, x = 1), c(coef = -1))))  
#   
#   expect_equal(f("-1 -1 -1"), m(list(c(coef = -3))))  
#   
#   expect_equal(f("-x+1"), m(list(c(coef = -1, x = 1), c(coef = 1))))
#   expect_equal(f("-x-1"), m(list(c(coef = -1, x = 1), c(coef = -1))))
#   expect_equal(f("- x - 1"), m(list(c(coef = -1, x = 1), c(coef = -1))))
#   
#   expect_equal(f("- x^2 - 1^2"), m(list(c(coef = -1, x = 2), c(coef = -1))))
#   expect_equal(f("x^2 + x^2 y"), m(list(c(coef = 1, x = 2), c(coef = 1, x = 2, y = 1))))
#   expect_equal(f("x-x"), m(list(c(coef = 0))))
#   expect_equal(f("x - x"), m(list(c(coef = 0))))
#   expect_equal(f("x - 4 x"), m(list(c(coef = -3, x = 1))))
#   expect_equal(f("x y^2 - y^2 x"), m(list(c(coef = 0))))
#   
#   expect_equal(f("2^2 x + 5^2 + 3^2"), m(list(c(coef = 4, x = 1), c(coef = 34))))
#   expect_equal(f("1  3 5^2 + 2 3^4 x"), m(list(c(coef = 75), c(coef = 162, x = 1))))
#   expect_equal(f("x - 2 x -3"), m(list(c(coef = -1, x = 1), c(coef = -3))))
#   
#   
#   
#   
#   
#   expect_equal(f("5-2x"), m(list(c(coef = 5), c(coef = -2, x = 1))))
#   expect_equal(f("5 + -2x"), m(list(c(coef = 5), c(coef = -2, x = 1))))
#   expect_equal(f("5^2-x"), m(list(c(coef = 25), c(coef = -1, x = 1))))
#   expect_equal(f("1+-x-x"), m(list(c(coef = 1), c(coef = -2, x = 1))))
#   
#   expect_equal(f("-x + 2y - 4x - -4"), m(list(
#     c(coef = -5, x = 1),
#     c(coef = 2, y = 1),
#     c(coef = 4)
#   )))
#   
#   expect_equal(f("x + 2 y + x^2 y + x y z"), m(list(
#     c(coef = 1, x = 1),
#     c(coef = 2, y = 1),
#     c(coef = 1, x = 2, y = 1),
#     c(coef = 1, x = 1, y = 1, z = 1)
#   )))  
#   
#   expect_equal(f("10 x + 2 y 3 + x^2 5 y"), m(list(
#     c(coef = 10, x = 1), 
#     c(coef = 6, y = 1), 
#     c(coef = 5, x = 2, y = 1)))
#   )
#   
#   expect_equal(f("-4 + 2+2 x +   1 x y^4 -3 prq^3 -y - 3 x 2 - 3 y -2"), m(list(
#     c(coef = -4),
#     c(coef = -4, x = 1),
#     c(coef = 1, x = 1, y = 4),
#     c(coef = -3, prq = 3),
#     c(coef = -4, y = 1)
#   )))  
#   
# })
# 
# 
# 
# test_that("Scientific notation is parsed properly", {
#   
#   m <- mpoly
#   
#   expect_equal(mp("1e+2 x"), m(list(c(coef = 100, x = 1))))
#   expect_equal(mp("1e-2 x"), m(list(c(coef = .01, x = 1))))
#   expect_equal(mp("e-a"), m(list(c(coef = 1, e = 1), c(coef = -1, a = 1))))
#   expect_equal(mp("e+a"), m(list(c(coef = 1, e = 1), c(coef =  1, a = 1))))
#   expect_equal(mp("a-e"), m(list(c(coef = 1, a = 1), c(coef = -1, e = 1))))
#   expect_equal(mp("a+e"), m(list(c(coef = 1, a = 1), c(coef =  1, e = 1))))
# })
# 
# 
# 














# mp("(x)")
# mp("((((x))))")
# mp("(x + 0)")
# mp("(x + 1)")
# mp("(x - 1)")
# mp("(-1 x - 1)")
# mp("2 (x + 1)")
# mp("-1 (x + 1)")
# 
# mp("-2 x + -1 x (3 x - (7 - 2 x)) 7 (x + 1) -3")
# 
# mp("(x-.25)^2")
# mp("(x + 1) (x - 1)")
# mp("(x + 1) (x + 2)")
# mp("(x + 1)^5")
# mp("(x+1)^5")
# mp("3 (x + 1) (x + 2)^2")
# mp("(x + y)^10")
#
# mp("(x + y) (x - y)")
# mp("((x + y) (x - y))^2")
# mp("((x + y) (x - y)^2)")
# mp("((x + y) (x - y)^2 3)^2")
#
# mp(c("x","x + y"))

#' Determine whether two multivariate polynomials are equal.
#'
#' Determine whether two multivariate polynomials are equal.
#'
#' @param e1 an object of class mpoly
#' @param e2 an object of class mpoly
#' @method == mpoly
#' @aliases ==.mpoly ==
#' @return A logical value.
#' @name mpoly-equal
#' @examples
#'
#' p1 <- mp("x + y + 2 z")
#' p1 == p1
#'
#' p2 <- reorder(p1, order = "lex", varorder = c("z","y","x"))
#' p1 == p2
#' p2 <- reorder(p1, order = "lex", varorder = c("z","w","y","x"))
#' p1 == p2
#' p1 == ( 2 * p2 )
#'
#' p1 <- mp("x + 1")
#' p2 <- mp("x + 1")
#' identical(p1, p2)
#' p1 == p2
#'
#' mp("x + 1") == mp("y + 1")
#' mp("2") == mp("1")
#' mp("1") == mp("1")
#' mp("0") == mp("-0")
#' 
#' 


#' @rdname mpoly-equal
#' @export
`==.mpoly` <- function(e1, e2) {
	
  if(!is.mpoly(e1)  || !is.mpoly(e2)) stop("e1 and e2 must be of class mpoly.", call. = FALSE)
  
  diff <- e1 - e2
  
  if(
    length(diff) == 1 && 
    length(diff[[1]]) == 1 && 
    diff[[1]] == 0
  ){
    return(TRUE)
  } else {
    return(FALSE)	
  }
  
}


#' @rdname mpoly-equal
#' @export
`!=.mpoly` <- function(e1, e2) !(e1 == e2)

#' Element-wise arithmetic with vectors of multivariate polynomials.
#'
#' Element-wise arithmetic with vectors of multivariate polynomials.
#'
#' @param e1 an object of class mpolyList
#' @param e2 an object of class mpolyList
#' @return An object of class mpolyList.
#' @name mpolyListArithmetic
#' @examples
#'
#' ( ms1 <- mp( c("x", 'y') ) )
#' ( ms2 <- mp( c("y", '2 x^2') ) )
#' ms1 + ms2
#' ms1 - ms2
#' ms1 * ms2
#' ms1^3
#' 
NULL







#' @rdname mpolyListArithmetic
#' @export
`+.mpolyList` <- function(e1, e2){
  
  # argument check
  if (is.numeric(e1)) {
    e1 <- structure(
      lapply(e1, function(.) structure(list(c("coef" = .)), class = "mpoly")),
      class = "mpolyList"
    )
  }
  # if (is.mpoly(e1)) e1 <- mpolyList(e1)
  
  if (is.numeric(e2)) {
    e2 <- structure(
      lapply(e2, function(.) structure(list(c("coef" = .)), class = "mpoly")),
      class = "mpolyList"
    )
  }
  # if (is.mpoly(e2)) e2 <- mpolyList(e2)
  
  stopifnot(is.mpolyList(e1))
  stopifnot(is.mpolyList(e2))
  
  # fix lengths
  if (length(e1) == 1L && length(e2) != 1L) {
    e1 <- structure(
      replicate(length(e2), e1[[1L]], simplify = FALSE),
      class = "mpolyList"
    )
  }
  
  if (length(e1) != 1L && length(e2) == 1L) {
    e2 <- structure(
      replicate(length(e1), e2[[1L]], simplify = FALSE),
      class = "mpolyList"
    )
  }
  
  if(length(e1) != length(e2)) stop("e1 and e2 must have equal length.", call. = FALSE)

  
  # template outcome
  out <- vector(length = length(e1), mode = "list")
  
  # compute sums
  for (k in seq_along(out)) out[[k]] <- e1[[k]] + e2[[k]]
  
  # return
  structure(out, class = "mpolyList")
}	












#' @rdname mpolyListArithmetic
#' @export
`-.mpolyList` <- function(e1, e2) e1 + -1*e2











#' @rdname mpolyListArithmetic
#' @export
`*.mpolyList` <- function(e1, e2){
  
  # argument check
  if (is.numeric(e1)) {
    e1 <- structure(
      lapply(e1, function(.) structure(list(c("coef" = .)), class = "mpoly")),
      class = "mpolyList"
    )
  }
  # if (is.mpoly(e1)) e1 <- mpolyList(e1)
  
  if (is.numeric(e2)) {
    e2 <- structure(
      lapply(e2, function(.) structure(list(c("coef" = .)), class = "mpoly")),
      class = "mpolyList"
    )
  }
  # if (is.mpoly(e2)) e2 <- mpolyList(e2)
  
  stopifnot(is.mpolyList(e1))
  stopifnot(is.mpolyList(e2))
  
  # fix lengths
  if (length(e1) == 1L && length(e2) != 1L) {
    e1 <- structure(
      replicate(length(e2), e1[[1L]], simplify = FALSE),
      class = "mpolyList"
    )
  }
  
  if (length(e1) != 1L && length(e2) == 1L) {
    e2 <- structure(
      replicate(length(e1), e2[[1L]], simplify = FALSE),
      class = "mpolyList"
    )
  }
  
  if(length(e1) != length(e2)) stop("e1 and e2 must have equal length.", call. = FALSE)
  
  # template outcome
  out <- vector(length = length(e1), mode = "list")
  
  # compute products
  for (k in seq_along(out)) out[[k]] <- e1[[k]] * e2[[k]]
  
  # return
  structure(out, class = "mpolyList")
}	









#' @rdname mpolyArithmetic
#' @export
`^.mpolyList` <- function(e1, e2){
  
  stopifnot(is.mpolyList(e1))
  
  if(!is.wholenumber(e2) || e2 < 0) stop("exponent must be a positive integer.", call. = FALSE)
  
  out <- structure(
    replicate(
      length(e1),
      structure(list(c("coef" = 1L)), class = "mpoly"),
      simplify = FALSE
    ),
    class = "mpolyList"
  )
  
  if(e2 == 0) return(out)
  
  for(k in 1:e2) out <- out * e1
  
  out
}

#' Enumerate basis monomials
#'
#' Enumerate basis monomials in the standard basis up to a given degree.
#'
#' @param indeterminates a character vector
#' @param d the highest total degree
#' @return a [mpolyList()] object of monomials
#' @export
#' @examples
#'
#' basis_monomials(c("x", "y"), 2)
#' basis_monomials(c("x", "y"), 3)
#' basis_monomials(c("x", "y", "z"), 2)
#' basis_monomials(c("x", "y", "z"), 3)
#' 
basis_monomials <- function(indeterminates, d) {
  
  # to get the x-y example working cleanly
  indeterminates <- rev(indeterminates)
  
  # compute powers as rows of a matrix
  p <- length(indeterminates)
  pows <- lapply(0:d, function(.) burst(n = ., r = p))
  pows <- do.call("rbind", pows)
  
  # make function to convert a power vector to an mpoly
  pow_to_mpoly <- function(v) {
    v <- c(v, 1)
    names(v) <- c(indeterminates, "coef")
    structure(list(v[v != 0]), class = "mpoly")
  }
    
  # split rows across list, add a coef, convert to mpoly
  lapply(
    split(pows, 1:nrow(pows)),
    pow_to_mpoly
  ) ->.;
    unname(.) ->.;
    structure(., class = "mpolyList")
  
}

#' Swap polynomial indeterminates
#'
#' Swap polynomial indeterminates
#'
#' @param p polynomial
#' @param variable the variable in the polynomial to replace
#' @param replacement the replacement variable
#' @return a mpoly object
#' @author David Kahle
#' @export
#' @examples
#'
#' (p <- mp("(x + y)^2"))
#' swap(p, "x", "t")
#'
#' ## the meta data is retained
#' (p <- bernstein(3, 5))
#' (p2 <- swap(p, "x", "t"))
#' is.bernstein(p2)
#'
#' (p <- chebyshev(3))
#' (p2 <- swap(p, "x", "t"))
#' is.chebyshev(p2)
#'
#' 
swap <- function(p, variable, replacement){


  # arg check
  stopifnot(is.mpoly(p))
  if (stringi::stri_detect_regex(replacement, "[^\\w.]")) stop("replacement must be a single symbol.", call. = FALSE)

  # determine variables
  vars <- vars(p)

  # if constant, return
  if(is.constant(p)) return(p)
  

  # don't allow replacement of one variable in the presence of many when
  # the variable is already a part of the polynomial, since
  # mpoly won't be run again to determine if a variable is duplicated
  stopifnot(variable %in% vars)
  if(replacement %in% vars && length(vars) > 1){
    stop("the replacement value cannot be a variable in the polynomial, try plug.", call. = FALSE)
  }
  
  
  # swapping in polynomial
  p_out <- unclass(p)
  p_out <- lapply(p, function(v) {
    names(v)[names(v) == variable] <- replacement
    v
  })
  class(p_out) <- "mpoly"
  
  # custom for special polynomials
  if (is.bernstein(p)) {
    class(p_out) <- c("bernstein", "mpoly")
    attr(p_out, "bernstein") <- attr(p, "bernstein")
    attr(p_out, "bernstein")["indeterminate"] <- replacement
  }
  
  if (is.chebyshev(p)) {
    class(p_out) <- c("chebyshev", "mpoly")
    attr(p_out, "chebyshev") <- attr(p, "chebyshev")
    attr(p_out, "chebyshev")["indeterminate"] <- replacement
  }
  
  # return
  p_out
}

#' Jacobi polynomials
#'
#' Jacobi polynomials as computed by orthopolynom.
#'
#' @param degree degree of polynomial
#' @param alpha the first parameter, also called p
#' @param beta the second parameter, also called q
#' @param kind "g" or "p"
#' @param indeterminate indeterminate
#' @param normalized provide normalized coefficients
#' @return a mpoly object or mpolyList object
#' @author David Kahle calling code from the orthopolynom package
#' @seealso [orthopolynom::jacobi.g.polynomials()],
#'   [orthopolynom::jacobi.p.polynomials()],
#'   \url{https://en.wikipedia.org/wiki/Jacobi_polynomials}
#' @export
#' @examples
#'
#' jacobi(0)
#' jacobi(1)
#' jacobi(2)
#' jacobi(3)
#' jacobi(4)
#' jacobi(5)
#' jacobi(6)
#' jacobi(10, 2, 2, normalized = TRUE)
#'
#' jacobi(0:5)
#' jacobi(0:5, normalized = TRUE)
#' jacobi(0:5, kind = "g")
#' jacobi(0:5, indeterminate = "t")
#'
#'
#'
#' # visualize the jacobi polynomials
#'
#' library(ggplot2); theme_set(theme_classic())
#' library(tidyr)
#'
#' s <- seq(-1, 1, length.out = 201)
#' N <- 5 # number of jacobi polynomials to plot
#' (jacPolys <- jacobi(0:N, 2, 2))
#'
#' df <- data.frame(s, as.function(jacPolys)(s))
#' names(df) <- c("x", paste0("P_", 0:N))
#' mdf <- gather(df, degree, value, -x)
#' qplot(x, value, data = mdf, geom = "line", color = degree)
#'
#' qplot(x, value, data = mdf, geom = "line", color = degree) +
#'   coord_cartesian(ylim = c(-30, 30))
#'
#'
#' 
jacobi <- function(degree, alpha = 1, beta = 1, kind = "p", indeterminate = "x", normalized = FALSE){
  
  stopifnot(all(is.wholenumber(degree)))
  stopifnot(all(degree >= 0))
  
  
  ## deal with kind
  stopifnot(kind %in% c("g","p"))
  
  ## make coefs
  if(kind == "g") coefs <- jacobi.g.polynomials(max(degree), p = alpha, q = beta, normalized)
  if(kind == "p") coefs <- jacobi.p.polynomials(max(degree), alpha = alpha, beta = beta, normalized)
  
  ## if only one degree is wanted, return that
  if(length(degree) == 1){
    coefs <- rev.default(coefs)[[1]]
    p <- as.mpoly.polynomial(coefs, indeterminate)
    class(p) <- c("jacobi", "mpoly")
    attr(p, "jacobi") <- list(
      degree = length(polynomial)-1, 
      kind = kind, 
      indeterminate = indeterminate,
      normalized = normalized,
      alpha = alpha, beta = beta
    )
    return(p)
  }
  
  ## if several are wanted, return them
  coefs <- coefs[degree+1]
  ps <- lapply(coefs, function(polynomial){
    p <- as.mpoly.polynomial(polynomial, indeterminate)
    class(p) <- c("jacobi", "mpoly")
    attr(p, "jacobi") <- list(
      degree = length(polynomial)-1, 
      kind = kind, 
      indeterminate = indeterminate,
      normalized = normalized,
      alpha = alpha, beta = beta
    )
    p
  })
  class(ps) <- "mpolyList"
  ps
  
}

context("jacobi()")


test_that("basic", {
  
  expect_equal(
    jacobi(3),
    structure(
      list(
        c("x" = 1, "coef" = -7.5),
        c("x" = 3, "coef" = 17.5)
      ), 
      "class" = c("jacobi", "mpoly"),
      "jacobi" = list(
        "degree" = 0, 
        "kind" = "p", 
        "indeterminate" = "x",
        "normalized" = FALSE,
        "alpha" = 1,
        "beta" = 1
      )
    )
  )
    
})




test_that("normalized", {
  
  expect_equal(
    jacobi(3, normalized = TRUE),
    structure(
      list(
        c("x" = 1, "coef" = -3.557562),
        c("x" = 3, "coef" = 8.300979)
      ), 
      "class" = c("jacobi", "mpoly"),
      "jacobi" = list(
        "degree" = 0, 
        "kind" = "p", 
        "indeterminate" = "x",
        "normalized" = TRUE,
        "alpha" = 1,
        "beta" = 1
      )
    ),
    tolerance = 1e-5
  )
  
})



test_that("alpha and beta", {
  
  expect_equal(
    jacobi(2, alpha = 2, beta = 3),
    structure(
      list(
        c("coef" = -1),
        c("x" = 1, "coef" = -2),
        c("x" = 2, "coef" = 9)
      ), 
      "class" = c("jacobi", "mpoly"),
      "jacobi" = list(
        "degree" = 0, 
        "kind" = "p", 
        "indeterminate" = "x",
        "normalized" = FALSE,
        "alpha" = 2,
        "beta" = 3
      )
    )
  )
  
})



test_that("kind = g", {
  
  expect_equal(
    jacobi(2, kind = "g"),
    structure(
      list(
        c("coef" = 1/6),
        c("x" = 1, "coef" = -1),
        c("x" = 2, "coef" = 1)
      ), 
      "class" = c("jacobi", "mpoly"),
      "jacobi" = list(
        "degree" = 0, 
        "kind" = "g", 
        "indeterminate" = "x",
        "normalized" = FALSE,
        "alpha" = 1,
        "beta" = 1
      )
    )
  )
  
})



test_that("degree is vectorized", {
  
  expect_equal(
    jacobi(0:1),
    structure(
      list(
        structure(
          list(
            c("coef" = 1)
          ), 
          "class" = c("jacobi", "mpoly"),
          "jacobi" = list(
            "degree" = 0, 
            "kind" = "p", 
            "indeterminate" = "x",
            "normalized" = FALSE,
            "alpha" = 1,
            "beta" = 1
          )
        ),
        structure(
          list(
            c("x" = 1, "coef" = 3)
          ), 
          "class" = c("jacobi", "mpoly"),
          "jacobi" = list(
            "degree" = 1, 
            "kind" = "p", 
            "indeterminate" = "x",
            "normalized" = FALSE,
            "alpha" = 1,
            "beta" = 1
          )
        )
      ),
      class = "mpolyList"
    )
  )
  
})

#' Convert an equation to a polynomial
#'
#' Convert characters of the form \code{"p1 = p2"} (or similar) to an mpoly
#' object representing \code{p1 - p2}.
#'
#' @param string a character string containing a polynomial, see examples
#' @param ... arguments to pass to [mpoly()]
#' @return An object of class mpoly or mpolyList.
#' @author David Kahle \email{david@@kahle.io}
#' @seealso [mpoly()]
#' @export
#' @examples
#'
#' eq_mp(c("y = x", "y ==  (x + 2)"))
#'
#'
#' 

eq_mp <- function (string, ...) {
  
  # "y = x" -> "(y) - (x)"
  strings <- vapply(
    stringi::stri_split_regex(string, "\\s*==?\\s*"),
    function(x) stringi::stri_c("(", x[1], ") - (", x[2], ")"),
    character(1)
  )
  
  # parse and output
  mp(strings, ...)
  
}

context("eq_mp()")


test_that("eq_mp() properly deals with spaces", {
  expect_equal(eq_mp("y = x"), mp("y - x"))
  expect_equal(eq_mp("y   =x"), mp("y - x"))
})


test_that("eq_mp() works with = or ==", {
  expect_equal(eq_mp("y = x"), eq_mp("y == x"))
})


test_that("eq_mp() is properly vectorized", {
  expect_equal(
    eq_mp(c("y = x", "x^2 + y^2 = 1")), 
    mp(c("y - x", "x^2 + y^2 - 1"))
  )
})

#' Defunct mpoly functions
#'
#' This is a list of past functions of the mpoly package.
#'
#' The following are defunct mpoly functions:
#'
#' \itemize{
#'
#' \item grobner
#'
#' }
#'
#' @name mpoly-defunct
NULL

#' Convert an object to an mpoly
#'
#' mpoly is the most basic function used to create objects of class mpoly.
#'
#' @param x an object
#' @param ... additional arguments to pass to methods
#' @return the object formated as a mpoly object.
#' @author David Kahle \email{david@@kahle.io}
#' @seealso [mp()]
#' @export
#' @examples
#'
#' library(ggplot2); theme_set(theme_classic())
#' library(dplyr)
#'
#' n <- 101
#' s <- seq(-5, 5, length.out = n)
#'
#'
#'
#' # one dimensional case
#'
#' df <- data.frame(x = seq(-5, 5, length.out = n)) %>%
#'   mutate(y = -x^2 + 2*x - 3 + rnorm(n, 0, 2))
#'
#' (mod <- lm(y ~ x + I(x^2), data = df))
#' (p <- as.mpoly(mod))
#' qplot(x, y, data = df) +
#'   stat_function(fun = as.function(p), colour = "red", size = 1)
#'
#' (mod <- lm(y ~ poly(x, 2, raw = TRUE), data = df))
#' (p <- as.mpoly(mod))
#' qplot(x, y, data = df) +
#'   stat_function(fun = as.function(p), colour = "red", size = 1)
#'
#' (mod <- lm(y ~ poly(x, 1, raw = TRUE), data = df))
#' (p <- as.mpoly(mod))
#' qplot(x, y, data = df) +
#'   stat_function(fun = as.function(p), colour = "red", size = 1)
#'
#'
#'
#' # two dimensional case with ggplot2
#'
#' df <- expand.grid(x = s, y = s) %>%
#'   mutate(z = x^2 - y^2 + 3*x*y + rnorm(n^2, 0, 3))
#' qplot(x, y, data = df, geom = "raster", fill = z)
#'
#' (mod <- lm(z ~ x + y + I(x^2) + I(y^2) + I(x*y), data = df))
#' (mod <- lm(z ~ poly(x, y, degree = 2, raw = TRUE), data = df))
#' (p <- as.mpoly(mod))
#' df$fit <- apply(df[,c("x","y")], 1, as.function(p))
#'
#' qplot(x, y, data = df, geom = "raster", fill = fit)
#'
#' qplot(x, y, data = df, geom = "raster", fill = z - fit) # residuals
#'
#'
#'
#'
#'
#'
#' 
as.mpoly <- function(x, ...) UseMethod("as.mpoly")



#' @export  
as.mpoly.default <- function(x, ...) {
  stop(sprintf("objects of class %s not supported by as.mpoly().", class(x)), call. = FALSE)
}






#' @export  
as.mpoly.alm <- function(x, ...) {
  
  names <- names(coef(x))
  if ("(Intercept)" %in% names) {
    names[names == "(Intercept)"] <- "1"
  }
  
  string <- paste(
    unname(coef(x)),
    names,
    sep = "*",
    collapse = " + "
  )
  
  mp(string, stars_only = TRUE)
  
}





#' @export  
as.mpoly.lm <- function(x, ...){
  
  coefs <- coef(x)
  coef_names <- names(coefs)
  coef_names[coef_names == "(Intercept)"] <- 1
  
  # check for "poly(x, 2)1" syntax
  if(any(str_detect(coef_names, "poly"))) {
    
    poly_ndcs <- which(str_detect(coef_names, "poly"))  
    coef_names[poly_ndcs] <- vapply(coef_names[poly_ndcs], parse_model_poly, character(1))
    mp_str <- paste(coefs, coef_names, sep = " ", collapse = " + ")
    
  } else { # I() notation
    
    I_ndcs <- which(str_detect(coef_names, "I([0-9a-zA-Z]*)"))  
    if(length(I_ndcs) > 0){
      coef_names[I_ndcs] <- vapply(
        coef_names[I_ndcs], 
        function(s) str_sub(s, 3, -2),
        character(1)
      )
    }
    coef_names  <- str_replace_all(coef_names, " \\* ", " ")
    mp_str <- paste(coefs, coef_names, sep = " ", collapse = " + ")
    
  }
  

  mp(mp_str)
}



# s <- "poly(x, y, degree = 2, raw = TRUE)2.0"
# parse_model_poly(s)
parse_model_poly <- function(s) {

  # grab vars; check for raw
  inside <- str_extract_all(s, "poly\\(.+\\)")[[1]]
  inside <- str_sub(inside, 6, -2)
  inside <- str_split(inside, ", ")[[1]]
  if(!any(inside == "raw = TRUE")) {
    stop("poly() statements currently must contain raw = TRUE.", call. = FALSE)
  }
  vars <- inside[!(str_detect(inside, "=") | !str_detect(inside, "[:alpha:]"))]
  
  # parse exponents
  if(str_sub(s, -1) == ")") {
    exponents <- 1
  } else {
    exponents <- str_sub(str_extract_all(s, "\\).+")[[1]], 2)
    if(length(exponents) == 0) exponent <- "1"
    exponents <- str_split(exponents, "\\.")[[1]]
  }
  
  # put together
  paste(paste(vars, exponents, sep = "^"), collapse = " ")
}





#' @export  
as.mpoly.polynomial <- function(x, indeterminate = "x", ...){
  as.mpoly.numeric(unclass(x), indeterminate)
}





#' @export  
as.mpoly.numeric <- function(x, indeterminate = "x", ...){
  n <- length(x)
  
  ## make list and populate
  p <- list()
  p[[1]] <- c(coef = x[1])
  if(n > 1) for(deg in 1:(n-1)){
    v <- c(deg, x[deg+1])
    names(v) <- c(indeterminate, "coef")
    p[[deg+1]] <- v
  }
  
  ## clean out zeros
  p <- Filter(function(v) v[["coef"]] != 0, p)
  
  ## clean in case everything is removed
  if(length(p) == 0) p <- list(c(coef = 0))
  
  ## class and out
  class(p) <- "mpoly"
  p
}

context("as.mpoly()")

test_that("numeric", {
  
  expect_equal(
    as.mpoly(3),
    structure(list(c("coef" = 3)), class = "mpoly")
  )
  
  expect_equal(
    as.mpoly(1:3),
    structure(
      list(
        c("coef" = 1),
        c("x" = 1, "coef" = 2),
        c("x" = 2, "coef" = 3)
      ), 
      class = "mpoly"
    )
  )
  
})



test_that("polynomial", {
  
  expect_equal(
    as.mpoly(hermite.h.polynomials(3)[[4]]),
    structure(
      list(
        c("x" = 1, "coef" = -12),
        c("x" = 3, "coef" = 8)
      ), 
      class = "mpoly"
    )
  )
  
})




test_that("lm", {
  
  expect_equal(
    as.mpoly(lm(y ~ x, data = data.frame("x" = c(0,1), "y" = c(1,3)))),
    structure(
      list(
        c("x" = 1, "coef" = 2),
        c("coef" = 1)
      ), 
      class = "mpoly"
    )
  )
  
  
  expect_equal(
    as.mpoly(
      lm(
        y ~ poly(x, 1, raw = TRUE), 
        data = data.frame("x" = c(0,1), "y" = c(1,3))
      )
    ),
    structure(
      list(
        c("x" = 1, "coef" = 2),
        c("coef" = 1)
      ), 
      class = "mpoly"
    )
  )
  
  
  expect_equal(
    as.mpoly(
      lm(
        y ~ poly(x, 2, raw = TRUE), 
        data = data.frame("x" = c(-1,0,1), "y" = c(0,-1,0))
      )
    ),
    structure(
      list(
        c("x" = 1, "coef" = 0),
        c("x" = 2, "coef" = 1),
        c("coef" = -1)
      ), 
      class = "mpoly"
    )
  )
  
  
  expect_equal(
    as.mpoly(
      lm(
        y ~ x + I(x^2), 
        data = data.frame("x" = c(-1,0,1), "y" = c(0,-1,0))
      )
    ),
    structure(
      list(
        c("x" = 1, "coef" = 0),
        c("x" = 2, "coef" = 1),
        c("coef" = -1)
      ), 
      class = "mpoly"
    )
  )
  
  
  expect_error(
    as.mpoly(
      lm(
        y ~ poly(x, 2, raw = FALSE), 
        data = data.frame("x" = c(-1,0,1), "y" = c(0,-1,0))
      )
    ),
    "poly() statements currently must contain raw = TRUE.",
    fixed = TRUE
  )
  
})




test_that("errors", {
  
  expect_error(
    as.mpoly("x"),
    "objects of class character not supported by as.mpoly().",
    fixed = TRUE
  )
  
})

#' Extract the terms of a multivariate polynomial.
#'
#' Compute the terms of an mpoly object as a mpolyList.
#'
#' @param x an object of class mpoly
#' @param ... additional parameters
#' @return An object of class mpolyList.
#' @usage \method{terms}{mpoly}(x, ...)
#' @export
#' @examples
#'
#' \dontrun{ .Deprecated issues a warning
#'
#' x <- mp("x^2 - y + x y z")
#' terms(x)
#' monomials(x)
#'
#' }
#' 
terms.mpoly <- function(x, ...){
  .Deprecated("monomials")
  # mpolyList <- lapply(x, function(v){
  #   l <- list(v)
  #   class(l) <- 'mpoly'
  #   l
  # })
  # class(mpolyList) <- 'mpolyList'
  # mpolyList
  monomials(x)
}

#' Pretty printing of a list of multivariate polynomials.
#'
#' This function iterates print.mpoly on an object of class mpolyList.
#'
#' @inheritParams print.mpoly
#' @return Invisible character vector of the printed objects.
#' @export
#' @examples
#'
#' mL <- mp(c("x + 1", "y - 1", "x y^2 z  +  x^2 z^2  +  z^2  +  x^3"))
#' mL
#' print(mL, order = "lex")
#' print(mL, order = "glex")
#' print(mL, order = "grlex")
#' print(mL, order = "glex", varorder = c("z","y","x"))
#' print(mL, order = "grlex", varorder = c("z","y","x"))
#' print(mL, varorder = c("z","y","x"))
#' 
#' (print(mL, varorder = c("z","y","x"), plus_pad = 1L, silent = TRUE))
#' 
#' (print(mL, silent = TRUE, stars = TRUE, plus_pad = 1L, times_pad = 0L))
#' 
print.mpolyList <- function(x, varorder = vars(x), stars = FALSE, order, silent = FALSE, ..., plus_pad = 2L, times_pad = 1L){
  
  stopifnot(is.mpolyList(x))
  n <- length(x)
  vars <- vars(x)
  
  if(!missing(varorder) && !all(vars %in% varorder)){
    stop(
      stri_c("If specified, varorder must contain all computed vars - ", stri_c(vars, collapse = ", ")), 
      call. = FALSE
    )
  }
  
  if(missing(varorder) && !missing(order)){
    message(stri_c('Using variable ordering - ', stri_c(vars, collapse = ', ')))
  }

  if(missing(order)){
    polys <- vapply(x, print, character(1), varorder = varorder,                stars = stars, silent = TRUE, ..., plus_pad = plus_pad, times_pad = times_pad)  
  } else {
    polys <- vapply(x, print, character(1), varorder = varorder, order = order, stars = stars, silent = TRUE, ..., plus_pad = plus_pad, times_pad = times_pad)    
  }

  if(!silent) cat(polys, sep = "\n")
  
  invisible(polys)
}

context("print.mpolyList()")


test_that("basic print.mpolyList() functionality", {
  
  expect_equal(
    print(mp(c("x + 1", "y^2 - 2")), silent = TRUE),
    c("x  +  1", "y^2  -  2")
  )
  
})




test_that("plus_pad works", {
  
  expect_equal(
    print(mp(c("x + 1", "y^2 - 2")), silent = TRUE, plus_pad = 1L),
    c("x + 1", "y^2 - 2")
  )
  
})



test_that("stars works", {
  
  expect_equal(
    print(mp(c("x z + 1", "y^2 - 2")), silent = TRUE, stars = TRUE),
    c("x * z  +  1", "y**2  -  2")
  )
  
})


test_that("times_pad works", {
  
  expect_equal(
    print(mp(c("x z + 1", "y^2 - 2")), silent = TRUE, stars = TRUE, times_pad = 0L),
    c("x*z  +  1", "y**2  -  2")
  )
  
})


test_that("errors correctly", {
  
  expect_error(
    print(mp(c("x z + 1", "y^2 - 2")), silent = TRUE, varorder = c("x", "y")),
    "If specified, varorder must contain all computed vars - x, z, y"
  )
  
})


test_that("messages correctly", {
  
  expect_message(
    print(mp(c("x z + 1", "y^2 - 2")), silent = TRUE, order = "lex"),
    "Using variable ordering - x, z, y"
  )
  
})


test_that("silent is coded correctly", {
  
  expect_equal(
    capture.output(print(mp(c("x z + 1", "y^2 - 2")), silent = FALSE)),
    c("x z  +  1", "y^2  -  2")
  )
  
})

#' Change polynomials into functions.
#'
#' Transform \code{mpoly} and \code{mpolyList} objects into function that can be
#' evaluated.
#'
#' Convert polynomials to functions mapping Rn to Rm that are vectorized in the
#' following way [as.function.mpoly()] governs this behavior:
#' 
#' \describe{
#'
#' \item{\[m = 1, n = 1, e.g. f(x) = x^2 + 2x\] }{Ordinary vectorized function
#' returned, so that if you input a numeric vector x, the returned function is
#' evaluated at each of the input values, and a numeric vector is returned.}
#'
#' \item{\[m = 1, n = 2+, e.g. f(x,y) = x^2 + 2x + 2xy\] }{A function of a
#' single vector argument f(v) = f(c(x,y)) is returned. If a N x n matrix is
#' input to the returned function, the function will be applied across the rows
#' and return a numeric vector of length N. If desired, setting `vector = FALSE`
#' changes this behavior so that an arity-n function is returned, i.e. the
#' function f(x,y) of two arguments.} In this case, the returned function will
#' accept equal-length numeric vectors and return a numeric vector, vectorizing
#' it.
#' 
#' }
#' 
#' And [as.function.mpolyList()] governs this behavior:
#' 
#' \describe{
#'
#' \item{\[m = 2+, n = 1, e.g. f(x) = (x, x^2)\] }{Ordinary vectorized function
#' returned, so that if you input a numeric vector x, the function is evaluated
#' at each of the input values, and a numeric matrix N x m, where N is the
#' length of the input vector.}
#' 
#' \item{\[m = 2+, n = 2+, e.g. f(x,y) = (1, x, y)\] }{When `vector = FALSE`
#' (the default), the created function accepts a numeric vector and returns a
#' numeric vector. The function will also accept an N x n matrix, in which case
#' the function is applied to its rows to return a N x m matrix.}
#'
#' }
#'
#' @inheritParams print.mpoly
#' @param vector whether the function should take a vector argument (TRUE) or a
#'   series of arguments (FALSE)
#' @param name should the returned object be named? only for `mpolyList` objects
#' @param ... any additional arguments
#' @param squeeze minify code in the created function
#' @name as-function
#' @seealso [plug()]
#' @examples
#' 
#' # basic usage. m = # polys/eqns, n = # vars
#' 
#' # m = 1, n = 1, `as.function.mpoly()`
#' p <- mp("x^2 + 1")
#' (f <- as.function(p))
#' f(2)
#' f(1:3) # vectorized
#' 
#' 
#' # m = 1, n = 2 , `as.function.mpoly()`
#' p <- mp("x y")
#' (f <- as.function(p))
#' f(1:2) 
#' (mat <- matrix(1:6, ncol = 2))
#' f(mat) # vectorized across rows of input matrix
#' 
#'  
#' # m = 2, n = 1, `as.function.mpolyList()`
#' p <- mp(c("x", "x^2"))
#' (f <- as.function(p))
#' f(2)
#' f(1:3) # vectorized
#' 
#' (f <- as.function(p, name = TRUE))
#' f(2)
#' f(1:3) # vectorized
#' 
#' 
#' # m = 3, n = 2, `as.function.mpolyList()`
#' p <- mp("(x + y)^2")
#' (p <- monomials(p))
#' 
#' (f <- as.function(p))
#' f(1:2) 
#' (mat <- cbind(x = 1:3, y = 4:6))
#' f(mat) # vectorized across rows of input matrix
#' 
#' (f <- as.function(p, name = TRUE))
#' f(1:2) 
#' f(mat)
#' 
#' 
#'
#' # setting vector = FALSE changes the function to a sequence of arguments
#' # this is only of interest if n = # of vars > 1
#' 
#' # m = 1, n = 2, `as.function.mpoly()`
#' p <- mp("x y")
#' (f <- as.function(p, vector = FALSE))
#' f(1, 2) 
#' (mat <- matrix(1:6, ncol = 2))
#' f(mat[,1], mat[,2]) # vectorized across input vectors
#' 
#' # m = 3, n = 2, `as.function.mpolyList()`
#' p <- mp(c("x", "y", "x y"))
#' (f <- as.function(p, vector = FALSE))
#' f(1, 2) 
#' (mat <- matrix(1:4, ncol = 2))
#' f(mat[,1], mat[,2]) # vectorized across rows of input matrix
#' (f <- as.function(p, vector = FALSE, name = TRUE))
#' f(1, 2) 
#' (mat <- matrix(1:4, ncol = 2))
#' f(mat[,1], mat[,2]) # vectorized across rows of input matrix
#' 
#' 
#' 
#' # it's almost always a good idea to use the varorder argument, 
#' # otherwise, mpoly has to guess at the order of the arguments
#' invisible( as.function(mp("y + x")) )
#' invisible( as.function(mp("x + y")) )
#' invisible( as.function(mp("y + x"), varorder = c("x","y")) )
#'
#'
#' # constant polynomials have some special rules
#' f <- as.function(mp("1"))
#' f(2)
#' f(1:10)
#' f(matrix(1:6, nrow = 2))
#' 
#'
#'
#' # you can use this to create a gradient function, useful for optim()
#' p <- mp("x + y^2 + y z")
#' (ps <- gradient(p))
#' (f <- as.function(ps, varorder = c("x","y","z")))
#' f(c(0,2,3)) # -> [1, 7, 2]
#'
#'
#'
#' # a m = 1, n = 2+ mpolyList creates a vectorized function
#' # whose rows are the evaluated quantities
#' (ps <- basis_monomials("x", 3))
#' (f <- as.function(ps))
#' s <- seq(-1, 1, length.out = 11)
#' f(s)
#'
#' # another example
#' (ps <- chebyshev(1:3))
#' f <- as.function(ps)
#' f(s)
#'
#' # the binomial pmf as an algebraic (polynomial) map
#' # from [0,1] to [0,1]^size
#' # p |-> {choose(size, x) p^x (1-p)^(size-x)}_{x = 0, ..., size}
#' abinom <- function(size, indet = "p"){
#'   chars4mp <- vapply(0:size, function(x){
#'     sprintf("%d %s^%d (1-%s)^%d", choose(size, x), indet, x, indet, size-x)
#'   }, character(1))
#'   mp(chars4mp)
#' }
#' (ps <- abinom(2, "p")) # = mp(c("(1-p)^2", "2 p (1-p)", "p^2"))
#' f <- as.function(ps)
#'
#' f(.50) # P[X = 0], P[X = 1], and P[X = 2] for X ~ Bin(2, .5)
#' dbinom(0:2, 2, .5)
#'
#' f(.75) # P[X = 0], P[X = 1], and P[X = 2] for X ~ Bin(2, .75)
#' dbinom(0:2, 2, .75)
#'
#' f(c(.50, .75)) # the above two as rows
#'
#' # as the degree gets larger, you'll need to be careful when evaluating
#' # the polynomial.  as.function() is not currently optimized for
#' # stable numerical evaluation of polynomials; it evaluates them in
#' # the naive way
#' all.equal(
#'   as.function(abinom(10))(.5),
#'   dbinom(0:10, 10, .5)
#' )
#'
#' all.equal(
#'   as.function(abinom(30))(.5),
#'   dbinom(0:30, 20, .5)
#' )
#'
#'
#' # the function produced is vectorized:
#' number_of_probs <- 11
#' probs <- seq(0, 1, length.out = number_of_probs)
#' (mat <- f(probs))
#' colnames(mat) <- sprintf("P[X = %d]", 0:2)
#' rownames(mat) <- sprintf("p = %.2f", probs)
#' mat
#' 









#' @usage \method{as.function}{mpoly}(x, varorder = vars(x), vector = TRUE,
#'   silent = FALSE, ..., plus_pad = 1L, times_pad = 1L, squeeze = TRUE)
#' @rdname as-function
#' @exportS3Method base::as.function
as.function.mpoly <- function(x, varorder = vars(x), vector = TRUE, silent = FALSE, ..., plus_pad = 1L, times_pad = 1L, squeeze = TRUE){
  
  # argument checking
  stopifnot(is.character(varorder))
  stopifnot(is.logical(vector))  	
  stopifnot(is.mpoly(x))
  if (!setequal(varorder, vars(x))) {
    stop("`varorder` must contain all of the input polynomial.", call. = FALSE)
  }
  
  
  # determine the number of variables
  n <- length(vars(x))
  
  
  # deal with constant polynomials
  if (is.constant(x)) return( function(.) 0*. + unlist(x)[["coef"]] )
  
  
  # print poly with stars
  mp_str <- print.mpoly(x, stars = TRUE, silent = TRUE, plus_pad = plus_pad, times_pad = times_pad)
  
  
  # univariate polynomial
  if (n == 1L) {
    mp_str <- stri_replace_all_fixed(mp_str, vars(x), ".")
    mp_str <- stri_replace_all_fixed(mp_str, "**", " ^")
    if (squeeze) mp_str <- stri_replace_all_fixed(mp_str, " ", "")
    if (!silent) message("f(.) with . = ", vars(x))
    return(eval(parse(text = paste0("function(.) ", mp_str))))
  }
  
  
  # general polynomials as a vector argument
  if (vector) { 
    mp_str <- stri_c(" ", mp_str, " ") # pad to make parsing easier 
    for (k in 1L:n) {
      mp_str <- stri_replace_all_fixed(mp_str, "**", " ^")
      mp_str <- stri_replace_all_fixed(mp_str, stri_c(" ", varorder[k], " "), stri_c(" .[", k, "] "))
      mp_str <- stri_replace_all_fixed(mp_str, " ^", "**")
    } 
    if (squeeze) mp_str <- stri_replace_all_fixed(mp_str, " ", "") 
    v <- stri_c("(", stri_c(varorder, collapse = ", "), ")") 
    if(!silent) message("f(.) with . = ", v) 
    f <- function(){}
    formals(f) <- alist(. = )
    body(f) <- as.call(c(
      as.name("{"),
      expression(if(is.matrix(.)) {
        if (ncol(.) != n) 
          stop("`ncol(mat)` not equal to number of variables.", call. = FALSE)
        return(apply(., 1, f))
      }),
      parse(text = mp_str)
    ))
    return(f)
  }
  
  
  # general polynomials as a bunch of arguments
  if (!vector) { 
    if(!silent) message("f(", stri_c(varorder, collapse = ", "), ")", sep = "") 
    mp_str <- stri_c(
      "function(", stri_c(varorder, collapse = ", "), ") {", 
      mp_str, 
      "}",
      sep = ""
    ) 
    return(eval(parse(text = mp_str)))
  }  
  
  
}








#' @rdname as-function
#' @exportS3Method base::as.function
as.function.bernstein <- function(x, ...){
  
  # grab bernstein values
  k <- attr(x, "bernstein")$k
  n <- attr(x, "bernstein")$n
  
  # return exp'd log function
  function(.) {
    
    out <- vector("numeric", length = length(.))
    
    non_pos_ndcs <- (. <= 0)
    sup_one_ndcs <- (. >= 1)
    if (any(non_pos_ndcs)) out[non_pos_ndcs] <- choose(n, k) * .[non_pos_ndcs]^k * (1 - .[non_pos_ndcs])^(n-k)
    if (any(sup_one_ndcs)) out[sup_one_ndcs] <- choose(n, k) * .[sup_one_ndcs]^k * (1 - .[sup_one_ndcs])^(n-k)
    if (!all(sup_one_ndcs | sup_one_ndcs)) out[0 < . & . < 1] <- exp(
      lchoose(n, k)  +  k*log(.[0 < . & . < 1])  +  (n-k)*log(1-.[0 < . & . < 1])
    )
    
    out
  }
  
}







# as.function.jacobi <- function(x, ...){
#   return(as.function.mpoly(x)) ## below is broken.
#   
#   # grab bernstein values
#   d <- attr(x, "jacobi")$degree
#   k <- attr(x, "jacobi")$kind
#   i <- attr(x, "jacobi")$indeterminate
#   n <- attr(x, "jacobi")$normalized
#   a <- attr(x, "jacobi")$alpha
#   b <- attr(x, "jacobi")$beta
#   
#   # return exp'd log function #
#   #https://en.wikipedia.org/wiki/Jacobi_polynomials function(.)
#   #pochhammer(a+1, d) / factorial(d) * hypergeo(-d, 1+a+b+d, a+1,
#   #(1-.)/2)
#   
# }











#' @usage \method{as.function}{mpolyList}(x, varorder = vars(x), vector = TRUE,
#'   silent = FALSE, name = FALSE, ..., plus_pad = 1L, times_pad = 1L, squeeze = TRUE)
#' @rdname as-function
#' @exportS3Method base::as.function
as.function.mpolyList <- function(x, varorder = vars(x), vector = TRUE, silent = FALSE, name = FALSE, ..., plus_pad = 1L, times_pad = 1L, squeeze = TRUE){
  
  # argument checking
  stopifnot(is.character(varorder))
  stopifnot(is.logical(vector))  	
  stopifnot(is.mpolyList(x))

  if (!missing(varorder) && !all( vars(x) %in% varorder )) stop("varorder must contain all of the variables.", call. = FALSE)
  
  
  # determine the number of variables
  n <- length(varorder)
    
  
  # print polys with stars
  mp_str <- print.mpolyList(x, stars = TRUE, silent = TRUE, plus_pad = plus_pad, times_pad = times_pad)
  if (name) printed_mps <- print.mpolyList(x, silent = TRUE)
  

  # univariate polynomial - vectorize
  if (n == 1) {
    mp_str <- stri_c(mp_str, collapse = " , ")
    mp_str <- stri_replace_all_fixed(mp_str, varorder, ".")
    if (squeeze) mp_str <- stri_replace_all_fixed(mp_str, " ", "") 
    f <- function(){}
    formals(f) <- alist(. = )
    body(f) <- as.call(c(
      as.name("{"),
      expression(if(length(.) > 1) return(t(sapply(., f)))),
      parse(text = paste0("out <- c(", stri_c(mp_str, collapse = ", "), ")")),
      expression(if (name) structure(out, names = printed_mps) else out)
    ))
    return(f)
  }
  
  # general polynomials as a vector argument
  if (vector) {
    mp_str <- stri_c(" ", mp_str, " ") # pad to make parsing easier 
    for (k in 1L:n) {
      mp_str <- stri_replace_all_fixed(mp_str, "**", " ^")
      mp_str <- stri_replace_all_fixed(mp_str, stri_c(" ", varorder[k], " "), stri_c(" .[", k, "] "))
      mp_str <- stri_replace_all_fixed(mp_str, " ^", "**")
    } 
    if (squeeze) mp_str <- stri_replace_all_fixed(mp_str, " ", "") 
    v <- stri_c("(", stri_c(varorder, collapse = ", "), ")")
    if(!silent && (missing(vector) || missing(varorder))) message("f(.) with . = ", v)
    f <- function(){}
    formals(f) <- alist(. = )
    body(f) <- as.call(c(
      as.name("{"),
      expression(if (!name) . <- unname(.)),
      expression(if(is.matrix(.)) {
        if (ncol(.) != n) 
          stop("`ncol(mat)` not equal to number of variables.", call. = FALSE)
        return(t(apply(., 1, f)))
      }),
      parse(text = paste0("out <- c(", stri_c(mp_str, collapse = ", "), ")")),
      expression(if (name) structure(out, names = printed_mps) else out)
    ))
    return(f)
    
  }

  # general polynomials as a bunch of arguments
  if (!vector) {
    if (squeeze) mp_str <- stri_replace_all_fixed(mp_str, " ", "") 
    mp_str <- stri_c(mp_str, collapse = ", ")
    if(!silent && (missing(vector) || missing(varorder))) {
      message("f(", paste(varorder, collapse = ", "), ")", sep = "")
    }
    mp_str <- stri_c(
      "function(", stri_c(varorder, collapse = ", "), ") {", 
      "if (length(", varorder[1], ") > 1) {",
      "out <- unname(cbind(", mp_str, ")); return(if (name) structure(out, dimnames = list(NULL, printed_mps)) else out) ",
      "};", 
      # "c(", mp_str, ")",
      paste0("out <- c(", mp_str, ");"),
      "if (name) structure(out, names = printed_mps) else out",
      "}"
    )
    return(eval(parse(text = mp_str)))
  }  
  
}

context("as.function.mpolyList()")

test_that("basic", {
  
  expect_equal(
    as.function(mp(c("2 x + 1", "x - z^2")), silent = TRUE)(c(1,2)),
    c(3L, -3L)
  )
  
})


test_that("vector", {
  
  expect_equal(
    as.function(mp(c("2 x + 1", "x - z^2")), silent = TRUE, vector = FALSE)(1, 2),
    c(3L, -3L)
  )
  
})


test_that("univariate", {
  
  expect_equal(
    as.function(mp(c("x", "x^2", "x^3")), silent = TRUE)(2),
    c(2L, 4L, 8L)
  )
  
  
  expect_equal(
    as.function(mp(c("x", "x^2", "x^3")), silent = TRUE)(1:3),
    structure(c(1, 2, 3, 1, 4, 9, 1, 8, 27), .Dim = c(3L, 3L))
  )
  
})


test_that("errors when should", {
  
  expect_error(
    as.function(mp(c("x", "y", "z")), silent = TRUE, varorder = c("x", "y")),
    "varorder must contain all of the variables."
  )
  
})

#' Polynomial components
#'
#' Compute quantities/expressions related to a multivariate polynomial.
#'
#' @param x an object of class mpoly
#' @param ndx a subsetting index
#' @param varorder the order of the variables
#' @param order a total order used to order the terms
#' @param reduced if TRUE, don't include zero degrees
#' @param unit for [monomials()], should the monomials have coefficient 1?
#' @param object mpoly object to pass to [coef()]
#' @param ... In [coef()], additional arguments passed to [print.mpoly()] for
#'   the names of the resulting vector
#' @param p an object of class mpoly or mpolyList
#' @param norm a norm (function) to normalize the coefficients of a polynomial, 
#'   defaults to the Euclidean norm
#' @return An object of class mpoly or mpolyList, depending on the context
#' @name components
#' @examples
#' (p <- mp("x y^2 + x (x+1) (x+2) x z + 3 x^10"))
#' p[2]
#' p[-2]
#' p[2:3]
#'
#' LT(p)
#' LC(p)
#' LM(p)
#'
#' multideg(p)
#' totaldeg(p)
#' monomials(p)
#' monomials(p, unit = TRUE)
#' coef(p)
#' 
#' p[1:2]
#' coef_lift(p[1:2])
#'
#' exponents(p)
#' exponents(p, reduce = TRUE)
#' lapply(exponents(p), is.integer)
#'
#' homogeneous_components(p)
#' 
#' (p <- mp("(x + y)^2"))
#' normalize_coefficients(p)
#' norm <- function(v) sqrt(sum(v^2))
#' norm(coef( normalize_coefficients(p) ))
#' 
#' abs_norm <- function(x) sum(abs(x))
#' normalize_coefficients(p, norm = abs_norm)
#' 
#' p <- mp(c("x", "2 y"))
#' normalize_coefficients(p)
#' 
#' # normalize_coefficients on the zero polynomial returns the zero polynomial
#' normalize_coefficients(mp("0"))
#' 
#' 





#' @rdname components
#' @export
`[.mpoly` <- function(x, ndx){
  x <- unclass(x)[ndx]
  class(x) <- "mpoly"
  x
}




#' @rdname components
#' @export
LT <- function(x, varorder = vars(x), order = "lex"){
  reorder.mpoly(x, varorder, order)[1]
}





#' @rdname components
#' @export
LC <- function(x, varorder = vars(x), order = "lex"){
  lt <- LT(x, varorder, order)
  unname(lt[[1]]["coef"])
}




#' @rdname components
#' @export
LM <- function(x, varorder = vars(x), order = "lex"){
  lt <- LT(x, varorder, order)
  lt[[1]]["coef"] <- 1
  lt
}





#' @rdname components
#' @export
multideg <- function(x, varorder = vars(x), order = "lex"){
  if(is.constant(x)) return(0L)
  lt <- LT(x, varorder, order)
  coef_ndx <- which(names(lt[[1]]) == "coef")
  sparse_multideg <- lt[[1]][-coef_ndx]
  lt_vars <- names(sparse_multideg)
  nvars <- length(varorder)
  mdeg <- rep(0L, length = nvars)
  names(mdeg) <- varorder
  for (k in 1:length(lt_vars)) { 
    mdeg[lt_vars[k]] <- mdeg[lt_vars[k]] + sparse_multideg[k]
  }
  mdeg
}





#' @rdname components
#' @export
totaldeg <- function(x){
  if(!is.mpoly(x) && length(x) > 1){
    return(vapply(x, totaldeg, integer(1)))
  }
  if(!is.mpoly(x)) stop("totaldeg requires an mpoly or mpolyList object.", call. = FALSE)
  max(vapply(exponents(x), sum, integer(1)))
}





#' @rdname components
#' @export
monomials <- function(x, unit = FALSE){
  if(!is.mpoly(x)) stop("monomials requires an mpoly or mpolyList object.", call. = FALSE)
  vec_to_mpoly <- function(.) {
    if (unit) .["coef"] <- 1
    structure(list(.), class = "mpoly")
  }
  structure(lapply(x, vec_to_mpoly), class = "mpolyList")
}






#' @rdname components
#' @export
exponents <- function(x, reduced = FALSE){

  l <- lapply(x, function(term){
    fixed_term <- as.integer(term[-which(names(term) == "coef")])
    names(fixed_term) <- names(term[-which(names(term) == "coef")])
    fixed_term
  })  
  
  if(reduced) return(l)
  
  v <- vars(x)
  p <- length(v)
  tmp <- rep.int(0L, p)
  names(tmp) <- v
  
  lapply(l, function(exp){
    tmp <- tmp + exp[v]
    tmp[is.na(tmp)] <- 0
    tmp <- as.integer(tmp)
    names(tmp) <- v
    tmp
  })
}



#' @rdname components
#' @exportS3Method 
coef.mpoly <- function(object, ...) {
  coef_vec <- vapply(object, `[[`, double(1), "coef")
  names(coef_vec) <- print.mpolyList(monomials(object, unit = TRUE), silent = TRUE, ...)
  coef_vec
}







#' @rdname components
#' @export
normalize_coefficients <- function(p, norm = function(x) sqrt(sum(x^2))) {
  if (is.mpolyList(p)) return( structure(lapply(p, normalize_coefficients), class = "mpolyList") )
  normalize <- function(x) x / norm(x)
  c <- coef(p)
  if (is.constant(p) && c == 0) return(structure(c(coef = 0), class = "mpoly"))
  c <- normalize(c)
  for (i in seq_along(p)) p[[i]]["coef"] <- c[i]
  p
}



#' @rdname components
#' @export
coef_lift <- function(p) {
  
  monos <- monomials(p, unit = TRUE)
  printed_monos <- print(monos, silent = TRUE)
  
  # remove carats and spaces, e.g. x^2 y -> x2y
  printed_monos <- gsub("\\^", "", printed_monos)
  printed_monos <- gsub(" ", "", printed_monos)
  
  # add b
  coefs_to_add <- paste0("b", printed_monos)
  
  # add coefs to mpoly
  for (i in seq_along(p)) {
    p[[i]]["coef"] <- 1
    p[[i]] <- structure(
      c(1, p[[i]]),
      names = c(coefs_to_add[i], names(p[[i]]))
    )
  }
  
  # return p
  p

}

context("Polynomial component extraction")


test_that("[.mpoly", {
  
  expect_equal(mp("x + y")[1], mp("x"))
  expect_equal(mp("x + y + z")[1:2], mp("x + y"))
  expect_equal(mp("x + y + z")[-1], mp("y + z"))
  
})


test_that("LT, LC, and LM work - lex", {
  
  expect_equal(
    LT(mp("x y^2 + x (x+1) (x+2) x z + 3 x^10")), 
    mp("3 x^10")
  )
  
  expect_equal(
    LC(mp("x y^2 + x (x+1) (x+2) x z + 3 x^10")), 
    3
  )
  
  expect_equal(
    LM(mp("x y^2 + x (x+1) (x+2) x z + 3 x^10")), 
    mp("x^10")
  )
  
})


test_that("multideg works - lex", {
  
  expect_equal(multideg(mp("1")), 0L)
  
  expect_equal(
    multideg(mp("x y^2 + x (x+1) (x+2) x z + 3 x^10")), 
    c(x = 10, y = 0, z = 0)
  )
  
})


test_that("totaldeg works - lex", {
  
  expect_equal(
    totaldeg(mp("x y^2 + x (x+1) (x+2) x z + 3 x^10")), 
    10
  )
  
  expect_equal(
    totaldeg(mp(c("x y", "y z^2", "0"))),
    c(2L, 3L, 0L)
  )
  
  expect_error(
    totaldeg("5 x"),
    "totaldeg requires an mpoly or mpolyList object.",
    fixed = TRUE
  )
  
})


test_that("monomials works", {
  
  expect_equal(
    monomials(mp("x y^2 + x (x+1) (x+2) x z + 3 x^10")), 
    mp(c("x y^2", "x^4 z", "3 x^3 z", "2 x^2 z", "3 x^10"))
  )
  
  expect_equal(
    monomials(mp("x y^2 + 3 y^3 x^2")), 
    mp(c("x y^2", "3 x^2 y^3"), varorder = c("x", "y"))
  )
  
  expect_error(
    monomials("5 x"),
    "monomials requires an mpoly or mpolyList object.",
    fixed = TRUE
  )
  
  expect_equal(
    suppressWarnings(terms(mp("x y^2 + 3 y^3 x^2"))), 
    mp(c("x y^2", "3 x^2 y^3"), varorder = c("x", "y"))
  )
  
  expect_warning(
    terms(mp("x y^2 + 3 y^3 x^2")),
    "'terms.mpoly' is deprecated.",
    fixed = TRUE
  )
  
})


test_that("exponents works", {
  
  expect_equal(
    exponents(mp("x y^2 + 3 y^3 x^2")), 
    list(structure(1:2, .Names = c("x", "y")), structure(2:3, .Names = c("x", "y")))
  )
  
  expect_equal(
    exponents(mp("x y^2 + 3 y^3 x^2 z"), reduced = TRUE),
    list(structure(1:2, .Names = c("x", "y")), c(x = 2L, y = 3L, z = 1L))
  )
  
})



test_that("coef works", {
  
  expect_equal(
    coef(mp("x y^2 + 3 y^3 x^2 + 5")), 
    c("x y^2" = 1, "x^2 y^3" = 3, "1" = 5)
  )
  
  expect_equal(
    coef(mp("x y^2 + 3 y^3 x^2 + 5"), stars = TRUE), 
    c("x * y**2" = 1, "x**2 * y**3" = 3, "1" = 5)
  )
  
  
})




context("normalize_coefficients()")

test_that("normalize_coefficients works", {
  
  p <- mp("(x + y)^2")
  
  expect_equal(
    normalize_coefficients(p),
    structure(
      list(
        c(x = 2, coef = 0.408248290463863), 
        c(x = 1, y = 1, coef = 0.816496580927726), 
        c(y = 2, coef = 0.408248290463863)
      ), 
      class = "mpoly"
    )
  )
  
  abs_norm <- function(x) sum(abs(x))
  expect_equal(
    normalize_coefficients(p, norm = abs_norm),
    structure(
      list(
        c(x = 2, coef = 0.25), 
        c(x = 1, y = 1, coef = 0.5), 
        c(y = 2, coef = 0.25)
      ), 
      class = "mpoly"
    )
  )
  
  expect_equal(
    sum(coef(normalize_coefficients(p))^2),
    1
  )
  
  expect_equal(
    normalize_coefficients(mp(c("x", "5 y"))),
    mp(c("x", "y"))
  )
  
  expect_equal(
    normalize_coefficients(mp("0")),
    structure(c(coef = 0), class = "mpoly")
  )
  
  
  
  
})





context("coef_lift()")

test_that("coef_lift() works", {
  
  expect_equal(
    coef_lift( mp("x + y") ), 
    mp("bx x + by y")
  )
  
  expect_equal(
    coef_lift( mp("2 x - y") ), 
    mp("bx x + by y")
  )
  
})

#' Reorder a multivariate polynomial.
#'
#' This function is used to set the intrinsic order of a multivariate
#' polynomial. It is used for both the in-term variables and the
#' terms.
#'
#' @param x an object of class mpoly
#' @param varorder the order of the variables
#' @param order a total order used to order the terms, \code{"lex"}, \code{"glex"}, or \code{"grlex"}
#' @param ... additional arguments
#' @usage \method{reorder}{mpoly}(x, varorder = vars(x), order, ...)
#' @return An object of class mpoly.
#' @export
#' @examples
#'	
#'	
#' p <- mp("x y^2 z  +  x^2 z^2  +  z^2  +  x^3")	
#' reorder(p) # -> x y^2 z  +  x^2 z^2  +  z^2  +  x^3
#' reorder(p, varorder = c("x","y","z"), order = "lex")
#'     # -> x^3  +  x^2 z^2  +  x y^2 z  +  z^2
#' reorder(p, varorder = c("x","y","z"), order = "glex")
#'     # -> x^2 z^2  +  x y^2 z  +  x^3  +  z^2
#' reorder(p, varorder = c("x","y","z"), order = "grlex")
#'     # -> x y^2 z  +  x^2 z^2  +  x^3  +  z^2
#' 
#' reorder(mp("x + 1"), varorder = c("y","x","z"), order = "lex")
#' reorder(mp("x + y"), varorder = c("y","x","z"), order = "lex")
#' reorder(mp("x y + y + 2 x y z^2"), varorder = c("y","x","z"))
#' reorder(mp("x^2 + y x + y"), order = "lex")
#'
#'
reorder.mpoly <- function(x, varorder = vars(x), order = "lex", ...){
  
  ## set basic quantities
  vars <- vars(x)
  p <- length(vars)
  n <- length(x)
  
  
  
  
  ## argument checking  
  # if(missing(varorder) && missing(order)) return(x)
  
  if(!missing(varorder)){
    if(!all(vars %in% varorder)){
      error <- stri_c(
        "If specified, varorder must contain all computed vars - ",
        paste(vars, collapse = ", ")
      )
      stop(error, call. = FALSE)
    }
    
    # reduce varorder to size of vars
    varorder <- varorder[varorder %in% vars]    
    
    ## reorder in-term variables
    x <- lapply(x, function(v){
      if(length(v) == 1) return(v) # constant case
      v <- v[intersect(c(varorder, "coef"), names(v))]
    })
    class(x) <- "mpoly"    
  }  
  
  if(missing(varorder) && !missing(order)){
    message <- stri_c(
      "Using variable ordering - ",
      paste(vars, collapse = ", ")
    )
    message(message)
  }
  

  
  
  ## argument check - order
  match.arg(order, c("lex","glex","grlex"))
  
  
  
  
  
  ## order == "lex"  
  if(order == "lex"){
  	
  	if(n == 1) return(x)  # single term polynomial
  	
  	# add zeros
    l <- lapply(x, function(v){
      z <- rep(0, p + 1)
      names(z) <- c(varorder, "coef")
      z[names(v)] <- v
      z
    })
    
    # construct matrix and sort appropriately
    m <- matrix(unname(unlist(l)), nrow = length(l), ncol = p + 1, byrow = TRUE)
    dimnames(m) <- list(1:nrow(m), c(varorder, "coef"))
    for(k in p:1) m <- m[order(m[,k], decreasing = TRUE),]	
    
    
    # split into list and add names
    list4mpoly <- unname(lapply(split(m, 1:n), function(v){
      names(v) <- c(varorder, "coef")
      v
    }))
    
    # mpoly it and return
    return( mpoly(list4mpoly, varorder = varorder) )  
  }
  
  
  
  
  
  ## order == "glex"  
  if(order == "glex"){
  	
  	if(n == 1) return(x)  # single term polynomial
  	
  	# add zeros
    l <- lapply(x, function(v){
      z <- rep(0, p + 1)
      names(z) <- c(varorder, "coef")
      z[names(v)] <- v
      z
    })
    
    # construct matrix and sort appropriately
    m <- matrix(unname(unlist(l)), nrow = length(l), ncol = p + 1, byrow = TRUE)
    dimnames(m) <- list(1:nrow(m), c(varorder, "coef"))
    for(k in p:1) m <- m[order(m[,k], decreasing = TRUE),]	
    
    m <- m[order(apply(m[, 1:p, drop = FALSE],1,sum), decreasing = TRUE),]    
    
    # split into list and add names
    list4mpoly <- unname(lapply(split(m, 1:n), function(v){
      names(v) <- c(varorder, "coef")
      v
    }))
    
    # mpoly it and return
    return( mpoly(list4mpoly, varorder = varorder) )
  }  
  
  
  
  
  
  ## order == "grlex"  
  if(order == "grlex"){
  	
  	if(n == 1) return(x)  # single term polynomial
  	
  	# add zeros
    l <- lapply(x, function(v){
      z <- rep(0, p + 1)
      names(z) <- c(varorder, "coef")
      z[names(v)] <- v
      z
    })
    
    # construct matrix and sort appropriately
    m <- matrix(unname(unlist(l)), nrow = length(l), ncol = p + 1, byrow = TRUE)
    dimnames(m) <- list(1:nrow(m), c(varorder, "coef"))
    for(k in 1:p) m <- m[order(m[,k]),]	
    m <- m[order(apply(m[, 1:p, drop = FALSE],1,sum), decreasing = TRUE),]    
    
    # split into list and add names
    list4mpoly <- unname(lapply(split(m, 1:n), function(v){
      names(v) <- c(varorder, "coef")
      v
    }))
    
    # mpoly it and return
    return( mpoly(list4mpoly, varorder = varorder) )    
  }    

}

context("reorder.mpoly()")


# examples from cox little and o'shea, 3e, p.59

test_that("lex works", {
  
  expect_equal(
    reorder.mpoly(mp("4 x y^2 z + 4 z^2 - 5 x^3 + 7 x^2 z^2"), order = "lex", varorder = c("x", "y", "z")),
    structure(
      list(
        c(x = 3, coef = -5), 
        c(x = 2, z = 2, coef = 7), 
        c(x = 1, y = 2, z = 1, coef = 4), 
        c(z = 2, coef = 4)
      ), 
      class = "mpoly"
    )
  )

})



test_that("glex works", {
  
  expect_equal(
    reorder.mpoly(mp("4 x y^2 z + 4 z^2 - 5 x^3 + 7 x^2 z^2"), order = "glex", varorder = c("x", "y", "z")),
    structure(
      list(
        c(x = 2, z = 2, coef = 7), 
        c(x = 1, y = 2, z = 1, coef = 4), 
        c(x = 3, coef = -5), 
        c(z = 2, coef = 4)
      ), 
      class = "mpoly"
    )
  )
  
})



test_that("grlex works", {
  
  expect_equal(
    reorder.mpoly(mp("4 x y^2 z + 4 z^2 - 5 x^3 + 7 x^2 z^2"), order = "grlex", varorder = c("x", "y", "z")),
    structure(
      list(
        c(x = 1, y = 2, z = 1, coef = 4), 
        c(x = 2, z = 2, coef = 7), 
        c(x = 3, coef = -5), 
        c(z = 2, coef = 4)
      ), 
      class = "mpoly"
    )
  )
  
})



test_that("message when no order specified but varorder not", {
  
  expect_message(
    reorder.mpoly(mp("x + y^2"), order = "glex"),
    "Using variable ordering - x, y",
    fixed = TRUE
  )
  
})



test_that("all vars in varorder", {
  
  expect_error(
    reorder.mpoly(mp("x + y^2"), varorder = "x"),
    "If specified, varorder must contain all computed vars - x, y",
    fixed = TRUE
  )
  
})

#' Plot the (real) variety of a polynomial
#'
#' The variety must have only 2 variables; it is plotted over a finite window;
#' and it will not discover zero-dimensional components.
#'
#' @param x an mpoly object
#' @param xlim,ylim numeric(2) vectors; x and y limits
#' @param varorder character(2); first element is x, second is y, defaults to
#'   \code{sort(vars(poly))}
#' @param add logical; should the plot be added to the current device?
#' @param n,nx,ny integer specifying number of points in the x and y dimensions
#' @param f argument to pass to [extendrange()]
#' @param col color of curve
#' @param ... arguments to pass to [contour()]
#' @usage \method{plot}{mpoly}(x, xlim, ylim, varorder, add = FALSE, n = 251, nx
#'   = n, ny = n, f = 0.05, col = "red", ...)
#' @exportS3Method base::plot
#' @return [NULL]
#' @examples
#'
#' p <- mp("x^2 + y^2 - 1")
#' plot(p, xlim = c(-1, 1), ylim = c(-1, 1))
#' plot(p, xlim = c(-1, 1), ylim = c(-1, 1), asp = 1)
#'
#' p <- mp("x^2 + 16 y^2 - 1")
#' plot(p, xlim = c(-1, 1), ylim = c(-1, 1))
#'
#' p <- mp("u^2 + 16 v^2 - 1")
#' plot(p, xlim = c(-1, 1), ylim = c(-1, 1))
#'
#' p <- mp("v^2 + 16 u^2 - 1")
#' plot(p, xlim = c(-1, 1), ylim = c(-1, 1))
#'
#' p <- mp("u^2 + 16 v^2 - 1")
#' plot(p, xlim = c(-1, 1), ylim = c(-1, 1), varorder = c("v","u"))
#'
#' p <- mp("y^2 - (x^3 + x^2)")
#' plot(p, xlim = c(-1.5, 1.5), ylim = c(-1.5, 1.5))
#'
#' plot(lissajous(3, 3, 0, 0), xlim = c(-1, 1), ylim = c(-1, 1), asp = 1)
#' plot(lissajous(5, 5, 0, 0), col = "steelblue", add = TRUE)
#' 
#' # how it works - inefficient
#' p <- lissajous(5, 5, 0, 0)
#' df <- expand.grid(
#'   x = seq(-1, 1, length.out = 26),
#'   y = seq(-1, 1, length.out = 26)
#' )
#' pf <- as.function(p)
#' df$z <- structure(pf(df), .Names = "p")
#' head(df)
#' with(df, plot(x, y, cex = .75, pch = 16, col = c("firebrick", "steelblue")[(z >= 0) + 1]))
#' plot(lissajous(5, 5, 0, 0), col = "black", add = TRUE)
#'
#' 
plot.mpoly <- function(x, xlim, ylim, varorder, add = FALSE, n = 251, nx = n, ny = n, f = 0.05, col = "red", ...) {
  
  if (is.character(x)) x <- mp(x)
  stopifnot( is.mpoly(x) )
  if (length(vars(x)) != 2) stop("`poly` must have exactly 2 variables.")
  if (missing(varorder)) varorder <- sort(vars(x))
  x <- reorder(x, varorder = varorder)
  
  if (isTRUE(add)) {
    if (missing(xlim)) xlim <- par("usr")[1:2] #|> extendrange(r = _, f = f)
    if (missing(ylim)) ylim <- par("usr")[3:4] #|> extendrange(r = _, f = f)
  } else {
    if (missing(xlim) || missing(ylim)) {
      stop("If `add = FALSE`, `xlim` and `ylim` must be specified.", call. = FALSE)
    }
  }
  
  xx <- seq(xlim[1], xlim[2], length.out = nx)
  yy <- seq(ylim[1], ylim[2], length.out = ny)

  df <- expand.grid(x = xx, y = yy)
  df$z <- as.function(x, varorder = varorder, silent = TRUE)(df)[,1]
  
  contour(
    xx, yy, matrix(df$z, nrow = nx), levels = 0, 
    drawlabels = FALSE, col = col, 
    xlab = varorder[1], ylab = varorder[2],  
    add = add,
    ...
  )
  
  
}

#' Enumerate the partitions of an integer
#'
#' Determine all unrestricted partitions of an integer.  This function is
#' equivalent to the function parts in the partitions package.
#'
#' @param n an integer
#' @return a matrix whose rows are the n-tuples
#' @author Robin K. S. Hankin, from package partitions
#' @export
#' @examples
#' partitions(5)
#' str(partitions(5))
partitions <- function(n){
  stopifnot(all(sapply(as.list(n), is.wholenumber)))
  stopifnot(all(n >= 1))
  if(is.vector(n) && length(n) > 1){
    return(lapply(as.list(n), function(x){
      as.matrix( t(parts(x)) )
    }))
  }
  as.matrix( t(parts(n)) )
}

context("partitions()")

test_that("basic partitions() functionality works", {
  expect_equal(
    partitions(3), 
    structure(c(3L, 2L, 1L, 0L, 1L, 1L, 0L, 0L, 1L), .Dim = c(3L, 3L))
  )
})


test_that("vectorized", {
  expect_equal(
    partitions(1:3), 
    list(
      structure(1L, .Dim = c(1L, 1L)), 
      structure(c(2L, 1L, 0L, 1L), .Dim = c(2L, 2L)), 
      structure(c(3L, 2L, 1L, 0L, 1L, 1L, 0L, 0L, 1L), .Dim = c(3L, 3L))
    )
  )
})

#' Determine all permutations of a set.
#'
#' An implementation of the Steinhaus-Johnson-Trotter permutation algorithm.
#'
#' @param set a set
#' @return a matrix whose rows are the permutations of set
#' @export
#' @examples
#' permutations(1:3)
#' permutations(c('first','second','third'))
#' permutations(c(1,1,3))
#' apply(permutations(letters[1:6]), 1, paste, collapse = '')
permutations <- function(set){
  r <- length(set)
  if(r == 1 && is.numeric(set)) return(permutations(1:set))
    
  row2diag <- function(row, direction){
  	np1 <- length(row) + 1
  	mat <- matrix(nrow = np1, ncol = np1)
    if(direction == -1) for(k in 1:np1) mat[k,] <- insert(np1, np1-k+1, row)
    if(direction == +1) for(k in 1:np1) mat[k,] <- insert(np1, k, row)

    mat
  } 
  # row2diag(matrix(c(1,2,2,1),2,2)[,1], -1)
  # row2diag(matrix(c(1,2,2,1),2,2)[,1], +1)  
  
  stepUp <- function(mat){ # an r x # matrix
    c <- ncol(mat)
    m <- NULL
    for(k in 1:nrow(mat)) m <- rbind(m, row2diag(mat[k,],(-1)^k))
    m    
  }
  # stepUp(matrix(1))  
  # stepUp(matrix(c(1,2,2,1),2,2))
  
  # iterate stepUp
  out <- matrix(1)
  for(k in 1:(r-1)) out <- stepUp(out)
  
  # substitute set values
  for(k in 1:r) out[out==k] <- paste('_', set[k], sep = '')
  
  out <- gsub('_', '', out) # clear PH
  if(is.numeric(set)){
  	d <- dim(out)
    out <- as.numeric(out)
    dim(out) <- d
  }
  
  # reorder outcomes
  out <- out[do.call("order", split(out, col(out))),]
  
  # return
  unique(out)
}







#' Insert an element into a vector.
#'
#' Insert an element into a vector.
#' 
#' @param elem element to insert
#' @param slot location of insert
#' @param v vector to insert into
#' @return vector with element inserted
#' @export
#' @examples
#' insert(2, 1, 1)
#' insert(2, 2, 1) 
#' insert('x', 5, letters) 
insert <- function(elem, slot, v){
  n <- length(v)
  if(slot == 1) return( c(elem, v) )
  if(slot == n+1) return( c(v, elem) )  	
  c(v[1:(slot-1)], elem, v[slot:n])    
}

context("permutations()")

test_that("permutations of a single integer works", {
  
  expect_equal(
    permutations(3L),
    structure(c(1, 1, 2, 2, 3, 3, 2, 3, 1, 3, 1, 2, 3, 2, 3, 1, 2, 1), .Dim = c(6L, 3L))
  )
  
})



test_that("permutations of a vector works", {
  
  expect_equal(
    permutations(letters[1:3]),
    structure(
      c("a", "a", "b", "b", "c", "c", "b", "c", "a", "c", 
      "a", "b", "c", "b", "c", "a", "b", "a"), 
      .Dim = c(6L, 3L)
    )
  )
  
})



test_that("permutations with duplicated entries works", {
  
  expect_equal(
    permutations(c(1, 1, 2)),
    structure(c(1, 1, 2, 1, 2, 1, 2, 1, 1), .Dim = c(3L, 3L))
  )
  
})

#' Determine the variables in a mpoly object.
#'
#' Determine the variables in a mpoly object.
#'
#' @param p An mpoly or mpolyList object.
#' @return A character vector of the variable names.
#' @export
#' @examples
#' 
#' p <- mp("x + y^2")
#' vars(p)
#' 
#' p <- mp(c("x + y^2", "y - 2 x"))
#' vars(p)
#' 
vars <- function(p) {
  stopifnot(is.mpoly(p) || is.mpolyList(p))
  flat_list <- unlist(p)
  flat_list <- flat_list[names(flat_list) != "coef"]
  flat_list <- unique(names(flat_list))
  unique(gsub('\\^[0-9]?', '', flat_list))
}

#' Homogenize a polynomial
#'
#' Homogenize a polynomial.
#'
#' @param x an mpoly object, see [mpoly()]
#' @param indeterminate name of homogenization
#' @return a (de/homogenized) mpoly or an mpolyList
#' @name homogenize
#' @examples
#'
#' x <- mp("x^4 + y + 2 x y^2 - 3 z")
#' is.homogeneous(x)
#' (xh <- homogenize(x))
#' is.homogeneous(xh)
#'
#' homogeneous_components(x)
#'
#' homogenize(x, "o")
#'
#' xh <- homogenize(x)
#' dehomogenize(xh) # assumes indeterminate = "t"
#' plug(xh, "t", 1) # same effect, but dehomogenize is faster
#'
#'
#'
#' # the functions are vectorized
#' (ps <- mp(c("x + y^2", "x + y^3")))
#' (psh <- homogenize(ps))
#' dehomogenize(psh)
#'
#'
#' # demonstrating a leading property of homogeneous polynomials
#' library(magrittr)
#' p  <- mp("x^2 + 2 x + 3")
#' (ph <- homogenize(p, "y"))
#' lambda <- 3
#' (d <- totaldeg(p))
#' ph %>%
#'   plug("x", lambda*mp("x")) %>%
#'   plug("y", lambda*mp("y"))
#' lambda^d * ph
#' 



#' @rdname homogenize
#' @export
homogenize <- function(x, indeterminate = "t"){
  
  if (is.mpolyList(x)) {
    hs <- lapply(x, homogenize, indeterminate = indeterminate)
    class(hs) <- "mpolyList"
    return(hs)
  }
  
  if(!is.mpoly(x)) stop("homogenize requires mpoly objects.", call. = FALSE)
  
  term_exps <- exponents(x)
  term_degs <- vapply(term_exps, sum, numeric(1))
  max_deg   <- max(term_degs)
  
  vars <- vars(x)
  p <- length(vars)
  
  homo_deg_per_term <- max_deg - term_degs

  for(k in 1:length(x)){
    if(homo_deg_per_term[[k]] == 0) next
    term <- x[[k]]
    coef_ndx <- which(names(term) == "coef")
    homo_piece <- homo_deg_per_term[[k]]
    names(homo_piece) <- indeterminate
    x[[k]] <- c(term[-coef_ndx], homo_piece, term[coef_ndx])
  }
  
  x
  
}







#' @rdname homogenize
#' @export
dehomogenize <- function(x, indeterminate = "t"){
  
  if (is.mpolyList(x)) {
    hs <- lapply(x, dehomogenize, indeterminate = indeterminate)
    class(hs) <- "mpolyList"
    return(hs)
  }
  
  if(!is.mpoly(x)) stop("dehomogenize requires mpoly objects.", call. = FALSE)
  # if(missing(indeterminate)) stop("dehomogenize requires a variable to dehomogenize.", call. = FALSE)
  
  # check for inhomogeneous
  if (!is.homogeneous(x)) {
    stop("polynomial ", print.mpoly(x, silent = TRUE), " is not homogeneous.", call. = FALSE)
  }
  
  # dehomogenize
  # plug(x, indeterminate, 1) # works, but is not optimized, so 
   xdh <- lapply(x, function(term){
    var_ndx <- which(names(term) == indeterminate)
    if (length(var_ndx) == 0) {
      return(term) # indeterminate not in term  
    } else {
      return(term[-var_ndx]) 
    }
  })
  
  class(xdh) <- "mpoly"
  xdh
}




#' @rdname homogenize
#' @export
is.homogeneous <- function(x) {
  if (is.mpolyList(x)) return(vapply(x, is.homogeneous, logical(1)))
  term_total_degs <- vapply(exponents(x), sum, numeric(1))
  all(term_total_degs == term_total_degs[1])
}



#' @rdname homogenize
#' @export
homogeneous_components <- function(x){
  exps <- exponents(x)
  exps_mat <- t(simplify2array(exps))
  term_total_degs <- rowSums(exps_mat)
  n_terms <- length(x)
  term_ndcs <- split(1:n_terms, term_total_degs)
  homo_comps <- lapply(term_ndcs, function(ndcs){
    x[ndcs]
  })
  homo_comps <- unname(homo_comps)
  class(homo_comps) <- "mpolyList"
  homo_comps
}

context("homogenizing functions")


test_that("is.homogeneous()", {
  
  expect_false( is.homogeneous(mp("x^2 + y")) )
  
  expect_true( is.homogeneous(mp("x^2 + y^2")) )
  
  expect_equal( 
    is.homogeneous(mp(c("x + y", "x + y^2"))),
    c(TRUE, FALSE)
  )
  
})




test_that("homogenize()", {
  
  expect_equal(
    homogenize(mp("x^3 + y")),
    mp("x^3 + y t^2")
  )
  
  expect_equal(
    homogenize(mp("x^3 + y"), indeterminate = "z"),
    mp("x^3 + y z^2")
  )
  
  expect_equal(
    homogenize(mp(c("x^3 + y", "x + 1"))),
    mp(c("x^3 + y t^2", "x + t"))
  )
  
  expect_error(
    homogenize("x + x^2"),
    "homogenize requires mpoly objects.",
    fixed = TRUE
  )
  
})



test_that("dehomogenize()", {
  
  expect_equal(
    dehomogenize(mp("x^3 + y t^2"), indeterminate = "t"),
    mp("x^3 + y")
  )
  
  expect_equal(
    dehomogenize(mp(c("x^2 + y t", "x t^2"))),
    mp(c("x^2 + y", "x"))
  )
  
  expect_error(
    dehomogenize(mp("x^3 + y t"), indeterminate = "t"),
    "polynomial x^3  +  y t is not homogeneous.",
    fixed = TRUE
  )
  
  expect_error(
    dehomogenize("x t + x^2"),
    "dehomogenize requires mpoly objects.",
    fixed = TRUE
  )
  
})




test_that("homogeneous_components()", {
  
  expect_equal(
    homogeneous_components(mp("1 + x + x^2 + x y + y")),
    mp(c("1", "x + y", "x^2 + x y"))
  )
  
})

#' mpoly predicate functions
#'
#' Various functions to deal with mpoly and mpolyList objects.
#'
#' @param x object to be tested
#' @return Vector of logicals.
#' @name predicates
#' @examples
#'
#' p <- mp("5")
#' is.mpoly(p)
#' is.constant(p)
#'
#' is.constant(mp(c("x + 1", "7", "y - 2")))
#'
#' p <- mp("x + y")
#' is.mpoly(p)
#'
#' is.mpolyList(mp(c("x + 1", "y - 1")))
#'
#'
#'
#' is.linear(mp("0"))
#' is.linear(mp("x + 1"))
#' is.linear(mp("x + y"))
#' is.linear(mp(c("0", "x + y")))
#'
#' is.linear(mp("x + x y"))
#' is.linear(mp(c("x + x y", "x")))
#'
#'
#' (p <- bernstein(2, 5))
#' is.mpoly(p)
#' is.bernstein(p)
#'
#' (p <- chebyshev(5))
#' is.mpoly(p)
#' is.chebyshev(p)
#' str(p)








#' @export
#' @rdname predicates
is.constant <- function(x){
  if (is.mpoly(x) && length(x) == 1 && length(x[[1]]) == 1) return(TRUE)
  if (is.mpolyList(x)) return(vapply(x, is.constant, logical(1)))
  FALSE
}




#' @export
#' @rdname predicates
is.mpoly <- function(x) inherits(x, "mpoly")




#' @export
#' @rdname predicates
is.unipoly <- function(x) {
  if(is.mpolyList(x)) return(vapply(x, is.unipoly, logical(1)))
  length(vars(x)) <= 1
}
# is.unipoly(mp("1"))
# is.unipoly(mp("x+1"))
# is.unipoly(mp("x+1+x^2"))
# is.unipoly(mp(c("x+1+x^2", "y")))
# is.unipoly(mp(c("x+1+x^2", "y", "x+y")))




#' @export
#' @rdname predicates
is.bernstein <- function(x) inherits(x, "bernstein")




#' @export
#' @rdname predicates
is.bezier <- function(x) inherits(x, "bezier")




#' @export
#' @rdname predicates
is.chebyshev <- function(x) inherits(x, "chebyshev")



       
#' @export
#' @rdname predicates
is.mpolyList <- function(x) inherits(x, "mpolyList")
  



#' @export
#' @rdname predicates
is.linear <- function(x){
  
  stopifnot(is.mpoly(x) || is.mpolyList(x))
  
  if(is.mpolyList(x)) return(vapply(x, is.linear, logical(1)))
  
  all(
    vapply(x, function(term) (length(term) <= 2L) && all(rev(term)[-1] <= 1L), logical(1))
  )  
  
}




#' Test whether an object is a whole number
#'
#' Test whether an object is a whole number.
#'
#' @param x object to be tested
#' @param tol tolerance within which a number is said to be whole
#' @return Vector of logicals.
#' @export
#' @examples
#' is.wholenumber(seq(-3,3,.5))
is.wholenumber <- function (x, tol = .Machine$double.eps^0.5){
  abs(x - round(x)) < tol
}

context("predicate functions")


test_that("is.constant() works - single", {
  expect_equal(is.constant(mp("5")), TRUE)
  expect_equal(is.constant(mp("5 + x")), FALSE)
})

test_that("is.constant() works - vector", {
  expect_equal(is.constant(mp(c("5", "5 + x"))), c(TRUE, FALSE))
})


test_that("is.mpoly() works", {
  expect_equal(is.mpoly(mp("5")), TRUE)
  expect_equal(is.mpoly(mp("5 + x")), TRUE)
  expect_equal(is.mpoly(mp(c("5", "5 + x"))), FALSE)
})


test_that("is.unipoly() works", {
  expect_equal(is.unipoly(mp("1")), TRUE)
  expect_equal(is.unipoly(mp("x+1")), TRUE)
  expect_equal(is.unipoly(mp("x+1+x^2")), TRUE)
  expect_equal(is.unipoly(mp(c("x+1+x^2", "y"))), c(TRUE, TRUE))
  expect_equal(is.unipoly(mp(c("x+1+x^2", "y", "x+y"))), c(TRUE, TRUE, FALSE))
})



test_that("is.bernstein() works", {
  expect_equal(is.bernstein(bernstein(0, 0)), TRUE)
  expect_equal(is.bernstein(1), FALSE)
  expect_equal(is.bernstein(mp("1")), FALSE)
})



test_that("is.bezier() works", {
  expect_equal(is.bezier(bezier(c(0, 0), c(1, 1), c(2, 0))), TRUE)
  expect_equal(is.bezier(mp(c("2 t", "-2 t^2 + 2 t"))), FALSE)
})



test_that("is.chebyshev() works", {
  expect_equal(is.chebyshev(chebyshev(1)), TRUE)
  expect_equal(is.chebyshev(mp("x")), FALSE)
})



test_that("is.mpolyList() works", {
  expect_equal(is.mpolyList(mp(c("x", "y"))), TRUE)
  
  expect_equal(is.mpolyList(   c("x", "y") ), FALSE)
  expect_equal(is.mpolyList(mp("x")), FALSE)
})




test_that("is.linear() works", {
  
  expect_equal(is.linear(mp("x + z")), TRUE)
  expect_equal(is.linear(mp(c("x + z", "y"))), c(TRUE, TRUE))
  
  expect_equal(is.linear(mp("x y")), FALSE)
  expect_equal(is.linear(mp("x^2")), FALSE)
  expect_equal(is.linear(mp(c("x + z", "y", "x^2"))), c(TRUE, TRUE, FALSE))

})



test_that("is.wholenumber() works", {
  
  expect_equal(is.wholenumber(1L), TRUE)
  expect_equal(is.wholenumber(1), TRUE)
  expect_equal(is.wholenumber(1 + 1e-5), FALSE)
  expect_equal(is.wholenumber(1 + 1e-10), TRUE)
  
})

#' Enumerate integer r-vectors summing to n
#'
#' Determine all r-vectors with nonnegative integer entries summing to n.  Note
#' that this is not intended to be optimized.
#'
#' @param n integer to sum to
#' @param r number of components
#' @return a matrix whose rows are the n-tuples
#' @export
#' @examples
#' burst(3)
#'
#' burst(3, 1)
#' burst(3, 2)
#' burst(3, 3)
#' burst(3, 4)
#'
#' rowSums(burst(4))
#' rowSums( burst(3, 2) )
#' rowSums( burst(3, 3) )
#' rowSums( burst(3, 4) )
#'
#'
#' burst(10, 4) # all possible 2x2 contingency tables with n=10
#' burst(10, 4) / 10 # all possible empirical relative frequencies
#' 
burst <- function(n, r = n){
  stopifnot(is.wholenumber(n))
  stopifnot(n >= 0)
  stopifnot(is.wholenumber(r))
  stopifnot(r >= 0)
  
  if (n == 0) return(t(rep(0, r)))
  
  if (r == 1) return(n)
   
  ## compute all partitions of the number (order does not matter)
  parts <- partitions(n)

  ## convert to list, remove 0's
  partsWOzeros <- apply(parts, 1, function(row) row[row != 0] )
  
  ## select all those with length less than or equal to r
  isAtLongest <- function(howLong){
    function(v) length(v) <= howLong
  }
  #isAtLongest(4)(1:4)
  #isAtLongest(4)(1:5)
  partsWOzeros <- Filter(isAtLongest(r), partsWOzeros)
  
  ## refill with 0's
  rvectors <- lapply(partsWOzeros, function(v){
    if(length(v) < r) return(c(v, rep(0, r - length(v))))
    v
  })
    
  ## compute permutations of each
  permsOfEach <- lapply(rvectors, permutations)
  
  ## make out matrix
  m <- sum(vapply(permsOfEach, nrow, integer(1)))
  n <- r
  out <- integer(length = m*n)
  dim(out) <- c(m, n)
  
  ## populate out matrix
  row <- 1
  for(i in seq_along(permsOfEach)){
    permsInPartition <- nrow(permsOfEach[[i]])
    out[row:(row+permsInPartition-1),] <- permsOfEach[[i]]
    row <- row + permsInPartition
  }
  
  ## return
  out
}

context("burst()")


test_that("basic burst works", {
  
  expect_equal(
    burst(2),
    structure(c(0, 2, 1, 2, 0, 1), .Dim = 3:2)
  )
  
})


test_that("bursts of a particular length are computed correctly", {
  
  expect_equal(
    burst(3, 2),
    structure(c(0, 3, 1, 2, 3, 0, 2, 1), .Dim = c(4L, 2L))
  )
  
})


test_that("bursts of length one is itself", {
  
  expect_equal(burst(3, 1), 3L)
  
})

#' Compute the least common multiple of two numbers.
#'
#' A simple algorithm to compute the least common multiple of two numbers
#'
#' @param x an object of class numeric
#' @param y an object of class numeric
#' @return The least common multiple of x and y.
#' @export
#' @examples
#' LCM(5,7)
#' LCM(5,8)
#' LCM(5,9)
#' LCM(5,10)
#' Reduce(LCM, 1:10) # -> 2520
LCM <- function(x, y){
  x0 <- x
  y0 <- y
  while(x != y){
    if(x < y){
      x <- x + x0
    } else {
      y <- y + y0	
    }
  }
  x
}

context("LCM() is working properly")

test_that("LCM works", {
  expect_equal(LCM(5L,  7L), 35L)
  expect_equal(LCM(5L,  8L), 40L)
  expect_equal(LCM(5L,  9L), 45L)
  expect_equal(LCM(5L, 10L), 10L)
})

#' Hermite polynomials
#'
#' Hermite polynomials as computed by orthopolynom.
#'
#' @param degree degree of polynomial
#' @param kind "he" (default, probabilists', see Wikipedia article) or "h"
#'   (physicists)
#' @param indeterminate indeterminate
#' @param normalized provide normalized coefficients
#' @return a mpoly object or mpolyList object
#' @author David Kahle calling code from the orthopolynom package
#' @seealso [orthopolynom::hermite.h.polynomials()],
#'   [orthopolynom::hermite.he.polynomials()],
#'   \url{https://en.wikipedia.org/wiki/Hermite_polynomials}
#' @export
#' @examples
#'
#' hermite(0)
#' hermite(1)
#' hermite(2)
#' hermite(3)
#' hermite(4)
#' hermite(5)
#' hermite(6)
#' hermite(10)
#'
#' hermite(0:5)
#' hermite(0:5, normalized = TRUE)
#' hermite(0:5, indeterminate = "t")
#'
#'
#'
#' # visualize the hermite polynomials
#'
#' library(ggplot2); theme_set(theme_classic())
#' library(tidyr)
#'
#' s <- seq(-3, 3, length.out = 201)
#' N <- 5 # number of hermite polynomials to plot
#' (hermPolys <- hermite(0:N))
#'
#' # see ?bernstein for a better understanding of
#' # how the code below works
#'
#' df <- data.frame(s, as.function(hermPolys)(s))
#' names(df) <- c("x", paste0("T_", 0:N))
#' mdf <- gather(df, degree, value, -x)
#' qplot(x, value, data = mdf, geom = "line", color = degree)
#'
#'
#' # hermite polynomials are orthogonal with respect to the gaussian kernel:
#' He2 <- as.function(hermite(2))
#' He3 <- as.function(hermite(3))
#' He4 <- as.function(hermite(4))
#'
#' w <- dnorm
#' integrate(function(x) He2(x) * He3(x) * w(x), lower = -Inf, upper = Inf)
#' integrate(function(x) He2(x) * He4(x) * w(x), lower = -Inf, upper = Inf)
#' integrate(function(x) He3(x) * He4(x) * w(x), lower = -Inf, upper = Inf)
#'
#'
#' 
hermite <- function(degree, kind = "he", indeterminate = "x", normalized = FALSE){
  
  stopifnot(all(is.wholenumber(degree)))
  stopifnot(all(degree >= 0))
  
  
  ## deal with kind
  stopifnot(kind %in% c("h","he"))
  
  ## make coefs
  if(kind == "h")  coefs <-  hermite.h.polynomials(max(degree), normalized)
  if(kind == "he") coefs <- hermite.he.polynomials(max(degree), normalized)

  
  ## if only one degree is wanted, return that
  if(length(degree) == 1){
    coefs <- rev.default(coefs)[[1]]
    p <- as.mpoly.polynomial(coefs, indeterminate)
    class(p) <- c("hermite", "mpoly")
    attr(p, "hermite") <- list(
      degree = length(polynomial)-1, 
      kind = kind, 
      indeterminate = indeterminate,
      normalized = normalized
    )
    return(p)
  }
  
  ## if several are wanted, return them
  coefs <- coefs[degree+1]
  ps <- lapply(coefs, function(polynomial){
    p <- as.mpoly.polynomial(polynomial, indeterminate)
    class(p) <- c("hermite", "mpoly")
    attr(p, "hermite") <- list(
      degree = length(polynomial)-1, 
      kind = kind, 
      indeterminate = indeterminate,
      normalized = normalized
    )
    p
  })
  class(ps) <- "mpolyList"
  ps
  
}

context("hermite()")


test_that("basic", {
  
  expect_equal(
    hermite(3),
    structure(
      list(
        c("x" = 1, "coef" = -3),
        c("x" = 3, "coef" = 1)
      ), 
      "class" = c("hermite", "mpoly"),
      "hermite" = list(
        "degree" = 0, 
        "kind" = "he", 
        "indeterminate" = "x",
        "normalized" = FALSE
      )
    )
  )
  
})



test_that("normalized", {
  
  expect_equal(
    hermite(3, normalized = TRUE),
    structure(
      list(
        c("x" = 1, "coef" = -0.7735719),
        c("x" = 3, "coef" = 0.2578573)
      ), 
      "class" = c("hermite", "mpoly"),
      "hermite" = list(
        "degree" = 0, 
        "kind" = "he", 
        "indeterminate" = "x",
        "normalized" = TRUE
      )
    ),
    tolerance = 1e-5
  )
  
})








test_that("indeterminate", {
  
  expect_equal(
    hermite(3, indeterminate = "t"),
    structure(
      list(
        c("t" = 1, "coef" = -3),
        c("t" = 3, "coef" = 1)
      ), 
      "class" = c("hermite", "mpoly"),
      "hermite" = list(
        "degree" = 0, 
        "kind" = "he", 
        "indeterminate" = "t",
        "normalized" = FALSE
      )
    ),
    tolerance = 1e-5
  )
  
})





test_that("kind = h", {
  
  expect_equal(
    hermite(3, kind = "h"),
    structure(
      list(
        c("x" = 1, "coef" = -12),
        c("x" = 3, "coef" = 8)
      ), 
      "class" = c("hermite", "mpoly"),
      "hermite" = list(
        "degree" = 0, 
        "kind" = "h", 
        "indeterminate" = "x",
        "normalized" = FALSE
      )
    ),
    tolerance = 1e-5
  )
  
})




test_that("vectorized", {
  
  expect_equal(
    hermite(0:2),
    structure(
      list(
        structure(
          list(
            c("coef" = 1)
          ), 
          "class" = c("hermite", "mpoly"),
          "hermite" = list(
            "degree" = 0, 
            "kind" = "he", 
            "indeterminate" = "x",
            "normalized" = FALSE
          )
        ),
        structure(
          list(
            c("x" = 1, "coef" = 1)
          ), 
          "class" = c("hermite", "mpoly"),
          "hermite" = list(
            "degree" = 1, 
            "kind" = "he", 
            "indeterminate" = "x",
            "normalized" = FALSE
          )
        ),
        structure(
          list(
            c("coef" = -1),
            c("x" = 2, "coef" = 1)
          ), 
          "class" = c("hermite", "mpoly"),
          "hermite" = list(
            "degree" = 2, 
            "kind" = "he", 
            "indeterminate" = "x",
            "normalized" = FALSE
          )
        )
      ),
      class = "mpolyList"
    )
  )
  
})

#' Switch indeterminates in a polynomial
#'
#' Switch indeterminates in a polynomial
#'
#' @param p a polynomial
#' @param indeterminate the indeterminate in the polynomial to switch
#' @param value the value/indeterminate to substitute
#' @return an mpoly object
#' @export
#' @examples
#'
#' # on an mpoly
#' (p <- mp("(x+y)^3"))
#' plug(p, "x", 5)
#' plug(p, "x", "t")
#' plug(p, "x", "y")
#' plug(p, "x", mp("2 y"))
#'
#' plug(p, "x", mp("x + y"))
#' mp("((x+y)+y)^3")
#'
#' # on an mpolyList
#' ps <- mp(c("x+y", "x+1"))
#' plug(ps, "x", 1)
#' 
plug <- function(p, indeterminate, value){
  
  # go recursive if an mpolyList
  if(is.mpolyList(p))
    return(structure(lapply(p, plug, indeterminate, value), class = "mpolyList"))
  
  # check args
  stopifnot(length(value) == 1 || is.mpoly(value))
  stopifnot(length(indeterminate) == 1)
  
  # if plugging in a number
  if(is.numeric(value)){    
    
    pList <- unclass(p)
    pList <- lapply(pList, function(term){
      # term <- pList[[2]]
      indetNdcs <- which(names(term) == indeterminate)
      term[indetNdcs] <- value^unname(term[indetNdcs])
      names(term)[indetNdcs] <- "coef"
      term
    })
    return(mpoly(pList)) # this can be optimized    
    
  }
  
  # if plugging in a value
  if(is.character(value)) value <- str_trim(value)
  if(is.mpoly(value))     value <- print.mpoly(value, silent = TRUE)
  
  if(is.character(value)){
    
    charPoly <- print.mpoly(p, silent = TRUE)
    charPoly <- str_replace_all(charPoly, indeterminate, paste0("(",value,")"))
    return(mp(charPoly))# this can be optimized    
    
  }

  
}

context("plug()")


test_that("plug() inserts numbers properly", {
  
  expect_equal(
    plug(mp("(x + y)^3"), "y", 5),
    mp("(x + 5)^3")
  )
  
})



test_that("plug() changes variables properly", {
  
  expect_equal(
    plug(mp("(x + y)^3"), "x", "t"),
    mp("(t + y)^3")
  )
  
  expect_equal(
    plug(mp("(x + y)^3"), "x", "t_2"),
    mp("(t_2 + y)^3")
  )
  
  expect_equal(
    plug(mp("(x + y)^3"), "x", "xx"),
    mp("(xx + y)^3")
  )
  
})



test_that("plug() inserts mpoly's properly", {
  
  expect_equal(
    plug(mp("(x + y)^3"), "x", mp("x^2")),
    mp("(x^2 + y)^3")
  )
  
})




test_that("plug() works on mpolyLists", {
  
  expect_equal(
    plug(mp(c("(x + y)^3", "2 x")), "x", mp("x^2")),
    mp(c("(x^2 + y)^3", "2 x^2"))
  )
  
})

#' Bezier polynomials
#'
#' Compute the Bezier polynomials of a given collection of points. Note that
#' using [mpoly::as.function.mpoly()] on the resulting Bezier polynomials is
#' made numerically stable by taking advantage of de Casteljau's algorithm; it
#' does not use the polynomial that is printed to the screen.  See
#' [bezier_function()] for details.
#'
#' @param ... either a sequence of points or a matrix/data frame of points, see
#'   examples
#' @param indeterminate the indeterminate of the resulting polynomial
#' @return a mpoly object
#' @author David Kahle
#' @seealso [bezier_function()]
#' @export
#' @examples
#'
#' p1 <- c(0,  0)
#' p2 <- c(1,  1)
#' p3 <- c(2, -1)
#' p4 <- c(3,  0)
#' bezier(p1, p2, p3, p4)
#'
#'
#' points <- data.frame(x = 0:3, y = c(0,1,-1,0))
#' bezier(points)
#'
#'
#' points <- data.frame(x = 0:2, y = c(0,1,0))
#' bezier(points)
#'
#'
#'
#'
#'
#'
#'
#' # visualize the bernstein polynomials
#'
#' library(ggplot2); theme_set(theme_bw())
#'
#' s <- seq(0, 1, length.out = 101)
#'
#'
#'
#' ## example 1
#' points <- data.frame(x = 0:3, y = c(0,1,-1,0))
#' (bezPolys <- bezier(points))
#'
#' f <- as.function(bezPolys)
#' df <- as.data.frame(f(s))
#'
#' ggplot(aes(x = x, y = y), data = df) +
#'   geom_point(data = points, color = "red") +
#'   geom_path(data = points, color = "red") +
#'   geom_path()
#'
#'
#'
#'
#' ## example 1 with weights
#' f <- as.function(bezPolys, weights = c(1,5,5,1))
#' df <- as.data.frame(f(s))
#'
#' ggplot(aes(x = x, y = y), data = df) +
#'   geom_point(data = points, color = "red") +
#'   geom_path(data = points, color = "red") +
#'   geom_path()
#'
#'
#'
#'
#'
#' ## example 2
#' points <- data.frame(x = 0:2, y = c(0,1,0))
#' (bezPolys <- bezier(points))
#' f <- as.function(bezPolys)
#' df <- as.data.frame(f(s))
#'
#' ggplot(aes(x = x, y = y), data = df) +
#'   geom_point(data = points, color = "red") +
#'   geom_path(data = points, color = "red") +
#'   geom_path()
#'
#'
#'
#'
#' ## example 3
#' points <- data.frame(x = c(-1,-2,2,1), y = c(0,1,1,0))
#' (bezPolys <- bezier(points))
#' f <- as.function(bezPolys)
#' df <- as.data.frame(f(s))
#'
#' ggplot(aes(x = x, y = y), data = df) +
#'   geom_point(data = points, color = "red") +
#'   geom_path(data = points, color = "red") +
#'   geom_path()
#'
#'
#'
#'
#' ## example 4
#' points <- data.frame(x = c(-1,2,-2,1), y = c(0,1,1,0))
#' (bezPolys <- bezier(points))
#' f <- as.function(bezPolys)
#' df <- as.data.frame(f(s))
#'
#' ggplot(aes(x = x, y = y), data = df) +
#'   geom_point(data = points, color = "red") +
#'   geom_path(data = points, color = "red") +
#'   geom_path()
#'
#'
#'
#'
#' ## example 5
#' qplot(speed, dist, data = cars)
#'
#' s <- seq(0, 1, length.out = 201)
#' p <- bezier(cars)
#' f <- as.function(p)
#' df <- as.data.frame(f(s))
#' qplot(speed, dist, data = cars) +
#'   geom_path(data = df, color = "red")
#'
#' # the curve is not invariant to permutations of the points
#' # but it always goes through the first and last points
#' permute_rows <- function(df) df[sample(nrow(df)),]
#' p <- bezier(permute_rows(cars))
#' f <- as.function(p)
#' df <- as.data.frame(f(s))
#' qplot(speed, dist, data = cars) +
#'   geom_path(data = df, color = "red")
#'
#'
#' 
bezier <- function(..., indeterminate = "t"){  
  
  ## grab input
  dots <- as.list(match.call(expand.dots = FALSE))$"..."
  names(dots) <- as.character(match.call(expand.dots = FALSE)$"...")
  dots <- lapply(dots, eval)
  
  ## parse input into a data frame
  if(length(dots) == 1 && (is.data.frame(dots[[1]]) || is.matrix(dots[[1]]))){    
    points <- dots[[1]]
    if(is.matrix(points)) points <- as.data.frame(points)
    if(is.null(names(points))) names(points) <- paste0("x", 1:ncol(points))
  } else if(
    all(vapply(dots, is.numeric, logical(1))) &&
    all(vapply(dots, length, numeric(1)) == length(dots[[1]]))
  ){
    points <- as.data.frame(do.call(rbind, dots))    
    if(is.null(names(dots[[1]]))){
      names(points) <- paste0("x", 1:ncol(points))
    } else {
      names(points) <- names(dots[[1]])
    }
    row.names(points) <- NULL
  }
  

  ## make polynomial
  n <- nrow(points) 
  bernPolys <- bernstein(0:(n-1), n-1, indeterminate)
  
  
  ## initialize bezPoly
  d <- ncol(points)
  bezPoly <- mp(rep("0", d))
  
  for(k in 1:n){
    ## convert vector of numerics into mpolyList
    polyWeights <- lapply(unname(points[k,]), function(.) mpoly(list(c(coef = .))))
    class(polyWeights) <- "mpolyList"
    
    ## duplicate the bern basis function to the same length
    basis <- replicate(d, bernPolys[[k]], simplify = FALSE)
    class(basis) <- "mpolyList"
    
    ## accumulate onto bezPoly
    bezPoly <- bezPoly + polyWeights * basis
  }
  
  ## class with meta-data
  class(bezPoly) <- c("bezier", "mpolyList")
  attr(bezPoly, "bezier") <- list(points = points)
  
  ## return
  bezPoly
}




































#' Bezier function
#'
#' Compute the Bezier function of a collection of polynomials.  By Bezier
#' function we mean the Bezier curve function, a parametric map running from t =
#' 0, the first point, to t = 1, the last point, where the coordinate mappings
#' are linear combinations of Bernstein polynomials.
#'
#' The function returned is vectorized and evaluates the Bezier curve in a
#' numerically stable way with de Castlejau's algorithm (implemented in R).
#'
#' @param points a matrix or data frame of numerics.  the rows represent points.
#' @param weights the weights in a weighted Bezier curve
#' @param ... ...; used internally
#' @return function of a single parameter
#' @author David Kahle
#' @references \url{https://en.wikipedia.org/wiki/Bezier_curve},
#'   \url{https://en.wikipedia.org/wiki/De_Casteljau's_algorithm}
#' @seealso [bezier()]
#' @name bezier_function
#' @examples
#'
#' library(ggplot2); theme_set(theme_bw())
#'
#'
#' t <- seq(0, 1, length.out = 201)
#' points <- data.frame(x = 0:3, y = c(0,1,-1,0))
#'
#'
#' f <- bezier_function(points)
#' df <- as.data.frame(f(t))
#'
#' ggplot(aes(x = x, y = y), data = df) +
#'   geom_point(data = points, color = "red", size = 8) +
#'   geom_path(data = points, color = "red") +
#'   geom_path()
#'
#'
#'
#'
#' f <- bezier_function(points, weights = c(1,5,5,1))
#' df <- as.data.frame(f(t))
#'
#' ggplot(aes(x = x, y = y), data = df) +
#'   geom_point(data = points, color = "red", size = 8) +
#'   geom_path(data = points, color = "red") +
#'   geom_path()
#'
#'
#'
#'
#' f <- bezier_function(points, weights = c(1,10,10,1))
#' df <- as.data.frame(f(t))
#'
#' ggplot(aes(x = x, y = y), data = df) +
#'   geom_point(data = points, color = "red", size = 8) +
#'   geom_path(data = points, color = "red") +
#'   geom_path()
#'
#'
#'
#'
#'
#'
#'
#'
#'
#'   




#' @rdname bezier_function
#' @export
bezier_function <- function(points, weights = rep(1L, nrow(points))){
  
  n <- nrow(points)
  degree <- n-1
  points <- cbind(1, as.matrix(points))
  points <- weights * points
  
  combineTwo <- function(vec, t) t*vec[-length(vec)] + (1-t)*vec[-1]  
  
  single_point_bezier <- function(.){
    for(i in 1:degree) points <- apply(points, 2, combineTwo, t = .)
    points[-1] / points[1]
  }  
  
  function(.){
    if(length(.) > 1) return(t(sapply(., single_point_bezier)))
    single_point_bezier(.)
  }
}


#' @rdname bezier_function
#' @export
bezierFunction <- function(...) {
  .Deprecated("bezier_function")
  bezier_function(...)
}





#' @rdname as-function
#' @exportS3Method base::as.function
as.function.bezier <- function(x, ...) bezier_function(attr(x, "bezier")$points, ...)

context("bezier_function()")


test_that("basic", {
  
  t <- seq(0, 1, length.out = 11)
  points <- data.frame(x = 0:3, y = c(0,1,-1,0))  
  
  expect_equal(
    as.data.frame(bezier_function(points)(t)),
    structure(
      list(
        "x" = c(3, 2.7, 2.4, 2.1, 1.8, 1.5, 1.2, 0.9, 0.6, 0.3, 0), 
        "y" = c(0, -0.216, -0.288, -0.252, -0.144, 0, 0.144, 0.252, 0.288, 0.216, 0)
      ), 
      class = "data.frame", 
      row.names = c(NA, -11L)
    )
  )
  
})



test_that("bezierFunction is deprecated", {
  
  t <- seq(0, 1, length.out = 11)
  points <- data.frame(x = 0:3, y = c(0,1,-1,0))  

  expect_equal(
    suppressWarnings(as.data.frame(bezierFunction(points)(t))),
    structure(
      list(
        "x" = c(3, 2.7, 2.4, 2.1, 1.8, 1.5, 1.2, 0.9, 0.6, 0.3, 0), 
        "y" = c(0, -0.216, -0.288, -0.252, -0.144, 0, 0.144, 0.252, 0.288, 0.216, 0)
      ), 
      class = "data.frame", 
      row.names = c(NA, -11L)
    )
  )
  
  expect_warning(
    bezierFunction(points),
    "'bezierFunction' is deprecated.",
    fixed = TRUE
  )
    
})

context("mpoly == mpoly")

test_that("basic", {
  
  expect_true(mp("x + y^2") == mp("x + y^2"))
  expect_false(mp("x + y^2") == mp("x"))
  expect_error(
    mp("2") == 2,
    "e1 and e2 must be of class mpoly.",
    fixed = TRUE
  )
  
})



context("mpoly != mpoly")

test_that("basic", {
  
  expect_false(mp("x + y^2") != mp("x + y^2"))
  expect_true( mp("x + y^2") != mp("y^2"))
  expect_error(
    mp("2") != 2,
    "e1 and e2 must be of class mpoly.",
    fixed = TRUE
  )
  
})

context("mpoly arithmetic")

test_that("Addition works", {
  
  expect_equal(0 + mp("1"), mp("1"))
  expect_equal(0 + mp("x"), mp("x"))
  expect_equal(1 + mp("1"), mp("2"))
  expect_equal(1 + mp("x"), mp("x + 1"))
  expect_equal(-1 + mp("1"), mp("0"))
  expect_equal(-1 + mp("x"), mp("-1 + x"))
  
  expect_equal(mp("1") + 0, mp("1"))
  expect_equal(mp("x") + 0, mp("x"))
  expect_equal(mp("1") + 1, mp("2"))
  expect_equal(mp("x") + 1, mp("x + 1"))
  expect_equal(mp("1") + -1, mp("0"))
  expect_equal(mp("x") + -1, mp("x - 1"))
  
})


test_that("Subtraction works", {
  
  expect_equal(0 - mp("1"), mp("-1"))
  expect_equal(0 - mp("x"), mp("-x"))
  expect_equal(1 - mp("1"), mp("0"))
  expect_equal(1 - mp("x"), mp("1 - x"))
  expect_equal(-1 - mp("1"), mp("-2"))
  expect_equal(-1 - mp("x"), mp("-1 - x"))
  
  expect_equal(mp("1") - 0, mp("1"))
  expect_equal(mp("x") - 0, mp("x"))
  expect_equal(mp("1") - 1, mp("0"))
  expect_equal(mp("x") - 1, mp("x - 1"))
  expect_equal(mp("1") - -1, mp("2"))
  expect_equal(mp("x") - -1, mp("x + 1"))
  
})


test_that("Multiplication works", {
  
  expect_equal(0 * mp("1"), mp("0"))
  expect_equal(0 * mp("x"), mp("0"))
  expect_equal(1 * mp("1"), mp("1"))
  expect_equal(1 * mp("x"), mp("x"))
  expect_equal(-1 * mp("1"), mp("-1"))
  expect_equal(-1 * mp("x"), mp("-x"))
  
  expect_equal(mp("1") * 0, mp("0"))
  expect_equal(mp("x") * 0, mp("0"))
  expect_equal(mp("1") * 1, mp("1"))
  expect_equal(mp("x") * 1, mp("x"))
  expect_equal(mp("1") * -1, mp("-1"))
  expect_equal(mp("x") * -1, mp("-x"))
  
})

context("mpolyList arithmetic")


test_that("addition", {
  
  expect_equal(
    mp(c("x", "y")) + mp(c("y^2", "x")),
    structure(
      list(
        structure(
          list(
            c(x = 1, coef = 1), 
            c(y = 2, coef = 1)
          ), 
          class = "mpoly"
        ), 
        structure(
          list(
            c(y = 1, coef = 1), 
            c(x = 1, coef = 1)
          ), 
          class = "mpoly")
        ), 
      class = "mpolyList"
    )
  )
  
  # expect_equal(
  #   mp("2") + mp(c("x", "y")),
  #   mp(c("x + 2", "y + 2"))
  # )
  
  expect_equal(
    2 + mp(c("x", "y")),
    mp(c("x + 2", "y + 2"))
  )
  
  # expect_equal(
  #   mp(c("x", "y")) + mp("2"),
  #   mp(c("x + 2", "y + 2"))
  # )
  
  expect_equal(
    mp(c("x", "y")) + 2,
    mp(c("x + 2", "y + 2"))
  )
  
  expect_equal(
    c(2, -3) + mp(c("x", "y")),
    mp(c("x + 2", "y - 3"))
  )
  
  expect_equal(
    mp(c("x", "y")) + c(2, -3),
    mp(c("x + 2", "y - 3"))
  )
  
  expect_error(
    c(2, -3) + mp(c("x", "y", "z")),
    "e1 and e2 must have equal length.",
    fixed = TRUE
  )
  
  expect_error(
    mp(c("x", "y", "z")) + c(2, -3),
    "e1 and e2 must have equal length.",
    fixed = TRUE
  )
    
  
})





test_that("subtraction", {
  
  expect_equal(
    mp(c("x", "y")) - mp(c("x", "x^2")),
    structure(
      list(
        structure(list(c(coef = 0)), class = "mpoly"), 
        structure(
          list(
            c(y = 1, coef = 1), 
            c(x = 2, coef = -1)
          ), 
          class = "mpoly")
        ), 
      class = "mpolyList"
    )
  )
  
  # expect_equal(
  #   mp("2") - mp(c("x", "y")),
  #   mp(c("-1 x + 2", "-1 y + 2"))
  # )
  
  expect_equal(
    2 - mp(c("x", "y")),
    mp(c("-1 x + 2", "-1 y + 2"))
  )
  
  # expect_equal(
  #   mp(c("x", "y")) - mp("2"),
  #   mp(c("x - 2", "y - 2"))
  # )
  
  expect_equal(
    mp(c("x", "y")) - 2,
    mp(c("x - 2", "y - 2"))
  )
  
  expect_equal(
    c(2, -3) - mp(c("x", "y")),
    mp(c("-1 x + 2", "-1 y - 3"))
  )
  
  expect_equal(
    mp(c("x", "y")) - c(2, -3),
    mp(c("x - 2", "y + 3"))
  )
  
  expect_error(
    c(2, -3) - mp(c("x", "y", "z")),
    "e1 and e2 must have equal length.",
    fixed = TRUE
  )
  
  expect_error(
    mp(c("x", "y", "z")) - c(2, -3),
    "e1 and e2 must have equal length.",
    fixed = TRUE
  )
  
})





test_that("multiplication", {
  
  expect_equal(
    mp(c("x", "y")) * mp(c("x", "x^2")),
    structure(
      list(
        structure(list(c(x = 2, coef = 1)), class = "mpoly"), 
        structure(list(c(y = 1, x = 2, coef = 1)), class = "mpoly")
      ), 
      class = "mpolyList"
    )
  )
  
  # expect_equal(
  #   mp("2") * mp(c("x", "y")),
  #   mp(c("2 x", "2 y"))
  # )
  
  expect_equal(
    2 * mp(c("x", "y")),
    mp(c("2 x", "2 y"))
  )
  
  # expect_equal(
  #   mp(c("x", "y")) * mp("2"),
  #   mp(c("2 x", "2 y"))
  # )
  
  expect_equal(
    mp(c("x", "y")) * 2,
    mp(c("2 x", "2 y"))
  )
  
  expect_equal(
    c(2, -3) * mp(c("x", "y")),
    mp(c("2 x", "-3 y"))
  )
  
  expect_error(
    c(2, -3) * mp(c("x", "y", "z")),
    "e1 and e2 must have equal length.",
    fixed = TRUE
  )
  
})











test_that("exponentiation", {
  
  expect_equal(
    mp(c("x", "y"))^2L,
    structure(
      list(
        structure(list(c(x = 2, coef = 1)), class = "mpoly"), 
        structure(list(c(y = 2, coef = 1)), class = "mpoly")
      ), 
      class = "mpolyList"
    )
  )
  
  
  expect_equal(
    mp(c("x", "y"))^0L,
    structure(
      list(
        structure(list(c(coef = 1)), class = "mpoly"), 
        structure(list(c(coef = 1)), class = "mpoly")
      ), 
      class = "mpolyList"
    )
  )
  
  expect_error(
    mp(c("x", "y"))^1.5,
    "exponent must be a positive integer.",
    fixed = TRUE
  )
  
})

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/bezier.R
\name{bezier}
\alias{bezier}
\title{Bezier polynomials}
\usage{
bezier(..., indeterminate = "t")
}
\arguments{
\item{...}{either a sequence of points or a matrix/data frame of points, see
examples}

\item{indeterminate}{the indeterminate of the resulting polynomial}
}
\value{
a mpoly object
}
\description{
Compute the Bezier polynomials of a given collection of points. Note that
using \code{\link[=as.function.mpoly]{as.function.mpoly()}} on the resulting Bezier polynomials is
made numerically stable by taking advantage of de Casteljau's algorithm; it
does not use the polynomial that is printed to the screen.  See
\code{\link[=bezier_function]{bezier_function()}} for details.
}
\examples{

p1 <- c(0,  0)
p2 <- c(1,  1)
p3 <- c(2, -1)
p4 <- c(3,  0)
bezier(p1, p2, p3, p4)


points <- data.frame(x = 0:3, y = c(0,1,-1,0))
bezier(points)


points <- data.frame(x = 0:2, y = c(0,1,0))
bezier(points)







# visualize the bernstein polynomials

library(ggplot2); theme_set(theme_bw())

s <- seq(0, 1, length.out = 101)



## example 1
points <- data.frame(x = 0:3, y = c(0,1,-1,0))
(bezPolys <- bezier(points))

f <- as.function(bezPolys)
df <- as.data.frame(f(s))

ggplot(aes(x = x, y = y), data = df) +
  geom_point(data = points, color = "red") +
  geom_path(data = points, color = "red") +
  geom_path()




## example 1 with weights
f <- as.function(bezPolys, weights = c(1,5,5,1))
df <- as.data.frame(f(s))

ggplot(aes(x = x, y = y), data = df) +
  geom_point(data = points, color = "red") +
  geom_path(data = points, color = "red") +
  geom_path()





## example 2
points <- data.frame(x = 0:2, y = c(0,1,0))
(bezPolys <- bezier(points))
f <- as.function(bezPolys)
df <- as.data.frame(f(s))

ggplot(aes(x = x, y = y), data = df) +
  geom_point(data = points, color = "red") +
  geom_path(data = points, color = "red") +
  geom_path()




## example 3
points <- data.frame(x = c(-1,-2,2,1), y = c(0,1,1,0))
(bezPolys <- bezier(points))
f <- as.function(bezPolys)
df <- as.data.frame(f(s))

ggplot(aes(x = x, y = y), data = df) +
  geom_point(data = points, color = "red") +
  geom_path(data = points, color = "red") +
  geom_path()




## example 4
points <- data.frame(x = c(-1,2,-2,1), y = c(0,1,1,0))
(bezPolys <- bezier(points))
f <- as.function(bezPolys)
df <- as.data.frame(f(s))

ggplot(aes(x = x, y = y), data = df) +
  geom_point(data = points, color = "red") +
  geom_path(data = points, color = "red") +
  geom_path()




## example 5
qplot(speed, dist, data = cars)

s <- seq(0, 1, length.out = 201)
p <- bezier(cars)
f <- as.function(p)
df <- as.data.frame(f(s))
qplot(speed, dist, data = cars) +
  geom_path(data = df, color = "red")

# the curve is not invariant to permutations of the points
# but it always goes through the first and last points
permute_rows <- function(df) df[sample(nrow(df)),]
p <- bezier(permute_rows(cars))
f <- as.function(p)
df <- as.data.frame(f(s))
qplot(speed, dist, data = cars) +
  geom_path(data = df, color = "red")



}
\seealso{
\code{\link[=bezier_function]{bezier_function()}}
}
\author{
David Kahle
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/eq_mp.R
\name{eq_mp}
\alias{eq_mp}
\title{Convert an equation to a polynomial}
\usage{
eq_mp(string, ...)
}
\arguments{
\item{string}{a character string containing a polynomial, see examples}

\item{...}{arguments to pass to \code{\link[=mpoly]{mpoly()}}}
}
\value{
An object of class mpoly or mpolyList.
}
\description{
Convert characters of the form \code{"p1 = p2"} (or similar) to an mpoly
object representing \code{p1 - p2}.
}
\examples{

eq_mp(c("y = x", "y ==  (x + 2)"))



}
\seealso{
\code{\link[=mpoly]{mpoly()}}
}
\author{
David Kahle \email{david@kahle.io}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mpoly-equal.R
\name{mpoly-equal}
\alias{mpoly-equal}
\alias{==.mpoly}
\alias{==}
\alias{!=.mpoly}
\title{Determine whether two multivariate polynomials are equal.}
\usage{
\method{==}{mpoly}(e1, e2)

\method{!=}{mpoly}(e1, e2)
}
\arguments{
\item{e1}{an object of class mpoly}

\item{e2}{an object of class mpoly}
}
\value{
A logical value.
}
\description{
Determine whether two multivariate polynomials are equal.
}
\examples{

p1 <- mp("x + y + 2 z")
p1 == p1

p2 <- reorder(p1, order = "lex", varorder = c("z","y","x"))
p1 == p2
p2 <- reorder(p1, order = "lex", varorder = c("z","w","y","x"))
p1 == p2
p1 == ( 2 * p2 )

p1 <- mp("x + 1")
p2 <- mp("x + 1")
identical(p1, p2)
p1 == p2

mp("x + 1") == mp("y + 1")
mp("2") == mp("1")
mp("1") == mp("1")
mp("0") == mp("-0")


}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/jacobi.R
\name{jacobi}
\alias{jacobi}
\title{Jacobi polynomials}
\usage{
jacobi(
  degree,
  alpha = 1,
  beta = 1,
  kind = "p",
  indeterminate = "x",
  normalized = FALSE
)
}
\arguments{
\item{degree}{degree of polynomial}

\item{alpha}{the first parameter, also called p}

\item{beta}{the second parameter, also called q}

\item{kind}{"g" or "p"}

\item{indeterminate}{indeterminate}

\item{normalized}{provide normalized coefficients}
}
\value{
a mpoly object or mpolyList object
}
\description{
Jacobi polynomials as computed by orthopolynom.
}
\examples{

jacobi(0)
jacobi(1)
jacobi(2)
jacobi(3)
jacobi(4)
jacobi(5)
jacobi(6)
jacobi(10, 2, 2, normalized = TRUE)

jacobi(0:5)
jacobi(0:5, normalized = TRUE)
jacobi(0:5, kind = "g")
jacobi(0:5, indeterminate = "t")



# visualize the jacobi polynomials

library(ggplot2); theme_set(theme_classic())
library(tidyr)

s <- seq(-1, 1, length.out = 201)
N <- 5 # number of jacobi polynomials to plot
(jacPolys <- jacobi(0:N, 2, 2))

df <- data.frame(s, as.function(jacPolys)(s))
names(df) <- c("x", paste0("P_", 0:N))
mdf <- gather(df, degree, value, -x)
qplot(x, value, data = mdf, geom = "line", color = degree)

qplot(x, value, data = mdf, geom = "line", color = degree) +
  coord_cartesian(ylim = c(-30, 30))



}
\seealso{
\code{\link[orthopolynom:jacobi.g.polynomials]{orthopolynom::jacobi.g.polynomials()}},
\code{\link[orthopolynom:jacobi.p.polynomials]{orthopolynom::jacobi.p.polynomials()}},
\url{https://en.wikipedia.org/wiki/Jacobi_polynomials}
}
\author{
David Kahle calling code from the orthopolynom package
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/print.mpoly.R
\name{print.mpoly}
\alias{print.mpoly}
\title{Pretty printing of multivariate polynomials.}
\usage{
\method{print}{mpoly}(x, varorder, order, stars = FALSE, silent =
  FALSE, ..., plus_pad = 2L, times_pad = 1L)
}
\arguments{
\item{x}{an object of class mpoly}

\item{varorder}{the order of the variables}

\item{order}{a total order used to order the monomials in the printing}

\item{stars}{print the multivariate polynomial in the more computer-friendly
asterisk notation (default FALSE)}

\item{silent}{logical; if TRUE, suppresses output}

\item{...}{additional parameters to go to \code{\link[base:cat]{base::cat()}}}

\item{plus_pad}{number of spaces to the left and right of plus sign}

\item{times_pad}{number of spaces to the left and right of times sign}
}
\value{
Invisible string of the printed object.
}
\description{
This is the major function used to view multivariate polynomials.
}
\examples{

mp("-x^5 - 3 y^2 + x y^3 - 1")


(p <- mp("2 x^5  -  3 y^2  +  x y^3"))
print(p) # same
print(p, silent = TRUE)
s <- print(p, silent = TRUE)
s

print(p, order = "lex") # -> 2 x^5  +  x y^3  -  3 y^2
print(p, order = "lex", varorder = c("y","x")) # -> y^3 x  -  3 y^2  +  2 x^5
print(p, varorder = c("y","x")) # -> y^3 x  -  3 y^2  +  2 x^5

# this is mostly used internally
print(p, stars = TRUE)
print(p, stars = TRUE, times_pad = 0L)
print(p, stars = TRUE, times_pad = 0L, plus_pad = 1L)
print(p, stars = TRUE, times_pad = 0L, plus_pad = 0L)
print(p, plus_pad = 1L)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/as.function.R, R/bezier.R
\name{as-function}
\alias{as-function}
\alias{as.function.mpoly}
\alias{as.function.bernstein}
\alias{as.function.mpolyList}
\alias{as.function.bezier}
\title{Change polynomials into functions.}
\usage{
\method{as.function}{mpoly}(x, varorder = vars(x), vector = TRUE,
  silent = FALSE, ..., plus_pad = 1L, times_pad = 1L, squeeze = TRUE)

\method{as.function}{bernstein}(x, ...)

\method{as.function}{mpolyList}(x, varorder = vars(x), vector = TRUE,
  silent = FALSE, name = FALSE, ..., plus_pad = 1L, times_pad = 1L, squeeze = TRUE)

\method{as.function}{bezier}(x, ...)
}
\arguments{
\item{x}{an object of class mpoly}

\item{varorder}{the order of the variables}

\item{vector}{whether the function should take a vector argument (TRUE) or a
series of arguments (FALSE)}

\item{silent}{logical; if TRUE, suppresses output}

\item{...}{any additional arguments}

\item{plus_pad}{number of spaces to the left and right of plus sign}

\item{times_pad}{number of spaces to the left and right of times sign}

\item{squeeze}{minify code in the created function}

\item{name}{should the returned object be named? only for \code{mpolyList} objects}
}
\description{
Transform \code{mpoly} and \code{mpolyList} objects into function that can be
evaluated.
}
\details{
Convert polynomials to functions mapping Rn to Rm that are vectorized in the
following way \code{\link[=as.function.mpoly]{as.function.mpoly()}} governs this behavior:

\describe{

\item{[m = 1, n = 1, e.g. f(x) = x^2 + 2x] }{Ordinary vectorized function
returned, so that if you input a numeric vector x, the returned function is
evaluated at each of the input values, and a numeric vector is returned.}

\item{[m = 1, n = 2+, e.g. f(x,y) = x^2 + 2x + 2xy] }{A function of a
single vector argument f(v) = f(c(x,y)) is returned. If a N x n matrix is
input to the returned function, the function will be applied across the rows
and return a numeric vector of length N. If desired, setting \code{vector = FALSE}
changes this behavior so that an arity-n function is returned, i.e. the
function f(x,y) of two arguments.} In this case, the returned function will
accept equal-length numeric vectors and return a numeric vector, vectorizing
it.

}

And \code{\link[=as.function.mpolyList]{as.function.mpolyList()}} governs this behavior:

\describe{

\item{[m = 2+, n = 1, e.g. f(x) = (x, x^2)] }{Ordinary vectorized function
returned, so that if you input a numeric vector x, the function is evaluated
at each of the input values, and a numeric matrix N x m, where N is the
length of the input vector.}

\item{[m = 2+, n = 2+, e.g. f(x,y) = (1, x, y)] }{When \code{vector = FALSE}
(the default), the created function accepts a numeric vector and returns a
numeric vector. The function will also accept an N x n matrix, in which case
the function is applied to its rows to return a N x m matrix.}

}
}
\examples{

# basic usage. m = # polys/eqns, n = # vars

# m = 1, n = 1, `as.function.mpoly()`
p <- mp("x^2 + 1")
(f <- as.function(p))
f(2)
f(1:3) # vectorized


# m = 1, n = 2 , `as.function.mpoly()`
p <- mp("x y")
(f <- as.function(p))
f(1:2) 
(mat <- matrix(1:6, ncol = 2))
f(mat) # vectorized across rows of input matrix

 
# m = 2, n = 1, `as.function.mpolyList()`
p <- mp(c("x", "x^2"))
(f <- as.function(p))
f(2)
f(1:3) # vectorized

(f <- as.function(p, name = TRUE))
f(2)
f(1:3) # vectorized


# m = 3, n = 2, `as.function.mpolyList()`
p <- mp("(x + y)^2")
(p <- monomials(p))

(f <- as.function(p))
f(1:2) 
(mat <- cbind(x = 1:3, y = 4:6))
f(mat) # vectorized across rows of input matrix

(f <- as.function(p, name = TRUE))
f(1:2) 
f(mat)



# setting vector = FALSE changes the function to a sequence of arguments
# this is only of interest if n = # of vars > 1

# m = 1, n = 2, `as.function.mpoly()`
p <- mp("x y")
(f <- as.function(p, vector = FALSE))
f(1, 2) 
(mat <- matrix(1:6, ncol = 2))
f(mat[,1], mat[,2]) # vectorized across input vectors

# m = 3, n = 2, `as.function.mpolyList()`
p <- mp(c("x", "y", "x y"))
(f <- as.function(p, vector = FALSE))
f(1, 2) 
(mat <- matrix(1:4, ncol = 2))
f(mat[,1], mat[,2]) # vectorized across rows of input matrix
(f <- as.function(p, vector = FALSE, name = TRUE))
f(1, 2) 
(mat <- matrix(1:4, ncol = 2))
f(mat[,1], mat[,2]) # vectorized across rows of input matrix



# it's almost always a good idea to use the varorder argument, 
# otherwise, mpoly has to guess at the order of the arguments
invisible( as.function(mp("y + x")) )
invisible( as.function(mp("x + y")) )
invisible( as.function(mp("y + x"), varorder = c("x","y")) )


# constant polynomials have some special rules
f <- as.function(mp("1"))
f(2)
f(1:10)
f(matrix(1:6, nrow = 2))



# you can use this to create a gradient function, useful for optim()
p <- mp("x + y^2 + y z")
(ps <- gradient(p))
(f <- as.function(ps, varorder = c("x","y","z")))
f(c(0,2,3)) # -> [1, 7, 2]



# a m = 1, n = 2+ mpolyList creates a vectorized function
# whose rows are the evaluated quantities
(ps <- basis_monomials("x", 3))
(f <- as.function(ps))
s <- seq(-1, 1, length.out = 11)
f(s)

# another example
(ps <- chebyshev(1:3))
f <- as.function(ps)
f(s)

# the binomial pmf as an algebraic (polynomial) map
# from [0,1] to [0,1]^size
# p |-> {choose(size, x) p^x (1-p)^(size-x)}_{x = 0, ..., size}
abinom <- function(size, indet = "p"){
  chars4mp <- vapply(0:size, function(x){
    sprintf("\%d \%s^\%d (1-\%s)^\%d", choose(size, x), indet, x, indet, size-x)
  }, character(1))
  mp(chars4mp)
}
(ps <- abinom(2, "p")) # = mp(c("(1-p)^2", "2 p (1-p)", "p^2"))
f <- as.function(ps)

f(.50) # P[X = 0], P[X = 1], and P[X = 2] for X ~ Bin(2, .5)
dbinom(0:2, 2, .5)

f(.75) # P[X = 0], P[X = 1], and P[X = 2] for X ~ Bin(2, .75)
dbinom(0:2, 2, .75)

f(c(.50, .75)) # the above two as rows

# as the degree gets larger, you'll need to be careful when evaluating
# the polynomial.  as.function() is not currently optimized for
# stable numerical evaluation of polynomials; it evaluates them in
# the naive way
all.equal(
  as.function(abinom(10))(.5),
  dbinom(0:10, 10, .5)
)

all.equal(
  as.function(abinom(30))(.5),
  dbinom(0:30, 20, .5)
)


# the function produced is vectorized:
number_of_probs <- 11
probs <- seq(0, 1, length.out = number_of_probs)
(mat <- f(probs))
colnames(mat) <- sprintf("P[X = \%d]", 0:2)
rownames(mat) <- sprintf("p = \%.2f", probs)
mat

}
\seealso{
\code{\link[=plug]{plug()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/hermite.R
\name{hermite}
\alias{hermite}
\title{Hermite polynomials}
\usage{
hermite(degree, kind = "he", indeterminate = "x", normalized = FALSE)
}
\arguments{
\item{degree}{degree of polynomial}

\item{kind}{"he" (default, probabilists', see Wikipedia article) or "h"
(physicists)}

\item{indeterminate}{indeterminate}

\item{normalized}{provide normalized coefficients}
}
\value{
a mpoly object or mpolyList object
}
\description{
Hermite polynomials as computed by orthopolynom.
}
\examples{

hermite(0)
hermite(1)
hermite(2)
hermite(3)
hermite(4)
hermite(5)
hermite(6)
hermite(10)

hermite(0:5)
hermite(0:5, normalized = TRUE)
hermite(0:5, indeterminate = "t")



# visualize the hermite polynomials

library(ggplot2); theme_set(theme_classic())
library(tidyr)

s <- seq(-3, 3, length.out = 201)
N <- 5 # number of hermite polynomials to plot
(hermPolys <- hermite(0:N))

# see ?bernstein for a better understanding of
# how the code below works

df <- data.frame(s, as.function(hermPolys)(s))
names(df) <- c("x", paste0("T_", 0:N))
mdf <- gather(df, degree, value, -x)
qplot(x, value, data = mdf, geom = "line", color = degree)


# hermite polynomials are orthogonal with respect to the gaussian kernel:
He2 <- as.function(hermite(2))
He3 <- as.function(hermite(3))
He4 <- as.function(hermite(4))

w <- dnorm
integrate(function(x) He2(x) * He3(x) * w(x), lower = -Inf, upper = Inf)
integrate(function(x) He2(x) * He4(x) * w(x), lower = -Inf, upper = Inf)
integrate(function(x) He3(x) * He4(x) * w(x), lower = -Inf, upper = Inf)



}
\seealso{
\code{\link[orthopolynom:hermite.h.polynomials]{orthopolynom::hermite.h.polynomials()}},
\code{\link[orthopolynom:hermite.he.polynomials]{orthopolynom::hermite.he.polynomials()}},
\url{https://en.wikipedia.org/wiki/Hermite_polynomials}
}
\author{
David Kahle calling code from the orthopolynom package
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/solve_unipoly.R
\name{solve_unipoly}
\alias{solve_unipoly}
\title{Solve a univariate mpoly with polyroot}
\usage{
solve_unipoly(mpoly, real_only = FALSE)
}
\arguments{
\item{mpoly}{an mpoly}

\item{real_only}{return only real solutions?}
}
\description{
Solve a univariate mpoly with polyroot
}
\examples{

solve_unipoly(mp("x^2 - 1")) # check x = -1, 1
solve_unipoly(mp("x^2 - 1"), real_only = TRUE)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/legendre.R
\name{legendre}
\alias{legendre}
\title{Legendre polynomials}
\usage{
legendre(degree, indeterminate = "x", normalized = FALSE)
}
\arguments{
\item{degree}{degree of polynomial}

\item{indeterminate}{indeterminate}

\item{normalized}{provide normalized coefficients}
}
\value{
a mpoly object or mpolyList object
}
\description{
Legendre polynomials as computed by orthopolynom.
}
\examples{

legendre(0)
legendre(1)
legendre(2)
legendre(3)
legendre(4)
legendre(5)
legendre(6)

legendre(2)
legendre(2, normalized = TRUE)

legendre(0:5)
legendre(0:5, normalized = TRUE)
legendre(0:5, indeterminate = "t")



# visualize the legendre polynomials

library(ggplot2); theme_set(theme_classic())
library(tidyr)

s <- seq(-1, 1, length.out = 201)
N <- 5 # number of legendre polynomials to plot
(legPolys <- legendre(0:N))

# see ?bernstein for a better understanding of
# how the code below works

df <- data.frame(s, as.function(legPolys)(s))
names(df) <- c("x", paste0("P_", 0:N))
mdf <- gather(df, degree, value, -x)
qplot(x, value, data = mdf, geom = "line", color = degree)


# legendre polynomials and the QR decomposition
n <- 201
x <- seq(-1, 1, length.out = n)
mat <- cbind(1, x, x^2, x^3, x^4, x^5)
Q <- qr.Q(qr(mat))
df <- as.data.frame(cbind(x, Q))
names(df) <- c("x", 0:5)
mdf <- gather(df, degree, value, -x)
qplot(x, value, data = mdf, geom = "line", color = degree)

Q <- apply(Q, 2, function(x) x / x[n])
df <- as.data.frame(cbind(x, Q))
names(df) <- c("x", paste0("P_", 0:5))
mdf <- gather(df, degree, value, -x)
qplot(x, value, data = mdf, geom = "line", color = degree)


# chebyshev polynomials are orthogonal in two ways:
P2 <- as.function(legendre(2))
P3 <- as.function(legendre(3))
P4 <- as.function(legendre(4))

integrate(function(x) P2(x) * P3(x), lower = -1, upper = 1)
integrate(function(x) P2(x) * P4(x), lower = -1, upper = 1)
integrate(function(x) P3(x) * P4(x), lower = -1, upper = 1)

n <- 10000L
u <- runif(n, -1, 1)
2 * mean(P2(u) * P3(u))
2 * mean(P2(u) * P4(u))
2 * mean(P3(u) * P4(u))

(2/n) * sum(P2(u) * P3(u))
(2/n) * sum(P2(u) * P4(u))
(2/n) * sum(P3(u) * P4(u))

}
\seealso{
\code{\link[orthopolynom:legendre.polynomials]{orthopolynom::legendre.polynomials()}},
\url{https://en.wikipedia.org/wiki/Legendre_polynomials}
}
\author{
David Kahle calling code from the orthopolynom package
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/vars.R
\name{vars}
\alias{vars}
\title{Determine the variables in a mpoly object.}
\usage{
vars(p)
}
\arguments{
\item{p}{An mpoly or mpolyList object.}
}
\value{
A character vector of the variable names.
}
\description{
Determine the variables in a mpoly object.
}
\examples{

p <- mp("x + y^2")
vars(p)

p <- mp(c("x + y^2", "y - 2 x"))
vars(p)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/as.mpoly.R
\name{as.mpoly}
\alias{as.mpoly}
\title{Convert an object to an mpoly}
\usage{
as.mpoly(x, ...)
}
\arguments{
\item{x}{an object}

\item{...}{additional arguments to pass to methods}
}
\value{
the object formated as a mpoly object.
}
\description{
mpoly is the most basic function used to create objects of class mpoly.
}
\examples{

library(ggplot2); theme_set(theme_classic())
library(dplyr)

n <- 101
s <- seq(-5, 5, length.out = n)



# one dimensional case

df <- data.frame(x = seq(-5, 5, length.out = n)) \%>\%
  mutate(y = -x^2 + 2*x - 3 + rnorm(n, 0, 2))

(mod <- lm(y ~ x + I(x^2), data = df))
(p <- as.mpoly(mod))
qplot(x, y, data = df) +
  stat_function(fun = as.function(p), colour = "red", size = 1)

(mod <- lm(y ~ poly(x, 2, raw = TRUE), data = df))
(p <- as.mpoly(mod))
qplot(x, y, data = df) +
  stat_function(fun = as.function(p), colour = "red", size = 1)

(mod <- lm(y ~ poly(x, 1, raw = TRUE), data = df))
(p <- as.mpoly(mod))
qplot(x, y, data = df) +
  stat_function(fun = as.function(p), colour = "red", size = 1)



# two dimensional case with ggplot2

df <- expand.grid(x = s, y = s) \%>\%
  mutate(z = x^2 - y^2 + 3*x*y + rnorm(n^2, 0, 3))
qplot(x, y, data = df, geom = "raster", fill = z)

(mod <- lm(z ~ x + y + I(x^2) + I(y^2) + I(x*y), data = df))
(mod <- lm(z ~ poly(x, y, degree = 2, raw = TRUE), data = df))
(p <- as.mpoly(mod))
df$fit <- apply(df[,c("x","y")], 1, as.function(p))

qplot(x, y, data = df, geom = "raster", fill = fit)

qplot(x, y, data = df, geom = "raster", fill = z - fit) # residuals







}
\seealso{
\code{\link[=mp]{mp()}}
}
\author{
David Kahle \email{david@kahle.io}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mpoly-package.R, R/mpoly.R
\docType{package}
\name{mpoly}
\alias{mpoly}
\alias{mpoly-package}
\title{Multivariate polynomials in R.}
\usage{
mpoly(list, varorder)
}
\arguments{
\item{list}{a list from which to construct an mpoly object}

\item{varorder}{(optional) a character vector setting the intrinsic variable
order of the polynomial}
}
\value{
Object of class mpoly.
}
\description{
A package for symbolic computation and more with multivariate polynomials

mpoly is the most basic function used to create objects of class mpoly.
However, it is not a general purpose function; for that see \code{\link[=mp]{mp()}}.
}
\examples{

list <- list(
  c(coef = 1),
  c(x = 1, coef = 1)
)
mpoly(list)


list <- list(
  c(x = 1, coef = 1, y = 0),
  c(x = 0, y = 1, coef = 2),
  c(y = 1, coef = -6),
  c(z = 1, coef = -3, x = 2),
  c(x = 1, coef = 0, x = 3),
  c(t = 1, coef = 4, t = 2, y = 4),
  c(x = 1),
  c(x = 1),
  c(coef = 5),
  c(coef = 5),
  c(coef = -5)
)

mpoly(list) # 3 x  -  4 y  -  3 x^2 z  +  4 y^4 t^3  +  5
mpoly(list, varorder = c("y", "z", "t", "x"))

list <- list(  c(x = 5, x = 2, coef = 5, coef = 6, y = 0) )
mpoly(list)


}
\seealso{
Useful links:
\itemize{
  \item \url{https://github.com/dkahle/mpoly}
  \item Report bugs at \url{https://github.com/dkahle/mpoly/issues}
}


\code{\link[=mp]{mp()}}
}
\author{
\strong{Maintainer}: David Kahle \email{david@kahle.io} (\href{https://orcid.org/0000-0002-9999-1558}{ORCID})


David Kahle \email{david@kahle.io}
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mpolyArithmetic.R, R/mpolyListArithmetic.R
\name{mpolyArithmetic}
\alias{mpolyArithmetic}
\alias{+.mpoly}
\alias{-.mpoly}
\alias{*.mpoly}
\alias{^.mpoly}
\alias{^.mpolyList}
\title{Arithmetic with multivariate polynomials}
\usage{
\method{+}{mpoly}(e1, e2)

\method{-}{mpoly}(e1, e2)

\method{*}{mpoly}(e1, e2)

\method{^}{mpoly}(e1, e2)

\method{^}{mpolyList}(e1, e2)
}
\arguments{
\item{e1}{an object of class mpoly}

\item{e2}{an object of class mpoly}
}
\value{
object of class mpoly
}
\description{
Arithmetic with multivariate polynomials
}
\examples{

p <- mp("x + y")
p + p
p - p
p * p
p^2
p^10


mp("(x+1)^10")
p + 1
2*p



}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/components.R
\name{components}
\alias{components}
\alias{[.mpoly}
\alias{LT}
\alias{LC}
\alias{LM}
\alias{multideg}
\alias{totaldeg}
\alias{monomials}
\alias{exponents}
\alias{coef.mpoly}
\alias{normalize_coefficients}
\alias{coef_lift}
\title{Polynomial components}
\usage{
\method{[}{mpoly}(x, ndx)

LT(x, varorder = vars(x), order = "lex")

LC(x, varorder = vars(x), order = "lex")

LM(x, varorder = vars(x), order = "lex")

multideg(x, varorder = vars(x), order = "lex")

totaldeg(x)

monomials(x, unit = FALSE)

exponents(x, reduced = FALSE)

\method{coef}{mpoly}(object, ...)

normalize_coefficients(p, norm = function(x) sqrt(sum(x^2)))

coef_lift(p)
}
\arguments{
\item{x}{an object of class mpoly}

\item{ndx}{a subsetting index}

\item{varorder}{the order of the variables}

\item{order}{a total order used to order the terms}

\item{unit}{for \code{\link[=monomials]{monomials()}}, should the monomials have coefficient 1?}

\item{reduced}{if TRUE, don't include zero degrees}

\item{object}{mpoly object to pass to \code{\link[=coef]{coef()}}}

\item{...}{In \code{\link[=coef]{coef()}}, additional arguments passed to \code{\link[=print.mpoly]{print.mpoly()}} for
the names of the resulting vector}

\item{p}{an object of class mpoly or mpolyList}

\item{norm}{a norm (function) to normalize the coefficients of a polynomial,
defaults to the Euclidean norm}
}
\value{
An object of class mpoly or mpolyList, depending on the context
}
\description{
Compute quantities/expressions related to a multivariate polynomial.
}
\examples{
(p <- mp("x y^2 + x (x+1) (x+2) x z + 3 x^10"))
p[2]
p[-2]
p[2:3]

LT(p)
LC(p)
LM(p)

multideg(p)
totaldeg(p)
monomials(p)
monomials(p, unit = TRUE)
coef(p)

p[1:2]
coef_lift(p[1:2])

exponents(p)
exponents(p, reduce = TRUE)
lapply(exponents(p), is.integer)

homogeneous_components(p)

(p <- mp("(x + y)^2"))
normalize_coefficients(p)
norm <- function(v) sqrt(sum(v^2))
norm(coef( normalize_coefficients(p) ))

abs_norm <- function(x) sum(abs(x))
normalize_coefficients(p, norm = abs_norm)

p <- mp(c("x", "2 y"))
normalize_coefficients(p)

# normalize_coefficients on the zero polynomial returns the zero polynomial
normalize_coefficients(mp("0"))


}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/reorder.mpoly.R
\name{reorder.mpoly}
\alias{reorder.mpoly}
\title{Reorder a multivariate polynomial.}
\usage{
\method{reorder}{mpoly}(x, varorder = vars(x), order, ...)
}
\arguments{
\item{x}{an object of class mpoly}

\item{varorder}{the order of the variables}

\item{order}{a total order used to order the terms, \code{"lex"}, \code{"glex"}, or \code{"grlex"}}

\item{...}{additional arguments}
}
\value{
An object of class mpoly.
}
\description{
This function is used to set the intrinsic order of a multivariate
polynomial. It is used for both the in-term variables and the
terms.
}
\examples{


p <- mp("x y^2 z  +  x^2 z^2  +  z^2  +  x^3")	
reorder(p) # -> x y^2 z  +  x^2 z^2  +  z^2  +  x^3
reorder(p, varorder = c("x","y","z"), order = "lex")
    # -> x^3  +  x^2 z^2  +  x y^2 z  +  z^2
reorder(p, varorder = c("x","y","z"), order = "glex")
    # -> x^2 z^2  +  x y^2 z  +  x^3  +  z^2
reorder(p, varorder = c("x","y","z"), order = "grlex")
    # -> x y^2 z  +  x^2 z^2  +  x^3  +  z^2

reorder(mp("x + 1"), varorder = c("y","x","z"), order = "lex")
reorder(mp("x + y"), varorder = c("y","x","z"), order = "lex")
reorder(mp("x y + y + 2 x y z^2"), varorder = c("y","x","z"))
reorder(mp("x^2 + y x + y"), order = "lex")


}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/lissajous.R
\name{lissajous}
\alias{lissajous}
\title{Lissajous polynomials}
\usage{
lissajous(m, n, p, q, digits = 3)
}
\arguments{
\item{m, n, p, q}{Trigonometric coefficients, see examples for description}

\item{digits}{The number of digits to round coefficients to, see
\code{\link[=round.mpoly]{round.mpoly()}}. This is useful for cleaning terms that are numerically
nonzero, but should be.}
}
\value{
a mpoly object
}
\description{
The Lissajous polynomials are the implicit (variety) descriptions of the
image of the parametric map x = cos(m t + p), y = sin(n t + q).
}
\examples{

lissajous(3, 2,  -pi/2, 0)
lissajous(4, 3,  -pi/2, 0)

}
\seealso{
\code{\link[=chebyshev]{chebyshev()}}, Merino, J. C (2003). Lissajous figures and Chebyshev
polynomials. The College Mathematics Journal, 34(2), pp. 122-127.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/chebyshev.R
\name{chebyshev}
\alias{chebyshev}
\alias{chebyshev_roots}
\title{Chebyshev polynomials}
\usage{
chebyshev(degree, kind = "t", indeterminate = "x", normalized = FALSE)

chebyshev_roots(k, n)
}
\arguments{
\item{degree}{degree of polynomial}

\item{kind}{\code{"t"} or \code{"u"} (Chebyshev polynomials of the first and
second kinds), or \code{"c"} or \code{"s"}}

\item{indeterminate}{indeterminate}

\item{normalized}{provide normalized coefficients}

\item{k, n}{the k'th root of the n'th chebyshev polynomial}
}
\value{
a mpoly object or mpolyList object
}
\description{
Chebyshev polynomials as computed by orthopolynom.
}
\examples{

chebyshev(0)
chebyshev(1)
chebyshev(2)
chebyshev(3)
chebyshev(4)
chebyshev(5)
chebyshev(6)
chebyshev(10)

chebyshev(0:5)
chebyshev(0:5, normalized = TRUE)
chebyshev(0:5, kind = "u")
chebyshev(0:5, kind = "c")
chebyshev(0:5, kind = "s")
chebyshev(0:5, indeterminate = "t")



# visualize the chebyshev polynomials

library(ggplot2); theme_set(theme_classic())
library(tidyr)

s <- seq(-1, 1, length.out = 201)
N <- 5 # number of chebyshev polynomials to plot
(cheb_polys <- chebyshev(0:N))

# see ?bernstein for a better understanding of
# how the code below works

df <- data.frame(s, as.function(cheb_polys)(s))
names(df) <- c("x", paste0("T_", 0:N))
mdf <- gather(df, degree, value, -x)
qplot(x, value, data = mdf, geom = "line", color = degree)



# roots of chebyshev polynomials
N <- 5
cheb_roots <- chebyshev_roots(1:N, N)
cheb_fun <- as.function(chebyshev(N))
cheb_fun(cheb_roots)



# chebyshev polynomials are orthogonal in two ways:
T2 <- as.function(chebyshev(2))
T3 <- as.function(chebyshev(3))
T4 <- as.function(chebyshev(4))

w <- function(x) 1 / sqrt(1 - x^2)
integrate(function(x) T2(x) * T3(x) * w(x), lower = -1, upper = 1)
integrate(function(x) T2(x) * T4(x) * w(x), lower = -1, upper = 1)
integrate(function(x) T3(x) * T4(x) * w(x), lower = -1, upper = 1)

(cheb_roots <- chebyshev_roots(1:4, 4))
sum(T2(cheb_roots) * T3(cheb_roots) * w(cheb_roots))
sum(T2(cheb_roots) * T4(cheb_roots) * w(cheb_roots))
sum(T3(cheb_roots) * T4(cheb_roots) * w(cheb_roots))

sum(T2(cheb_roots) * T3(cheb_roots))
sum(T2(cheb_roots) * T4(cheb_roots))
sum(T3(cheb_roots) * T4(cheb_roots))


}
\seealso{
\code{\link[orthopolynom:chebyshev.t.polynomials]{orthopolynom::chebyshev.t.polynomials()}},
\code{\link[orthopolynom:chebyshev.u.polynomials]{orthopolynom::chebyshev.u.polynomials()}},
\code{\link[orthopolynom:chebyshev.c.polynomials]{orthopolynom::chebyshev.c.polynomials()}},
\code{\link[orthopolynom:chebyshev.s.polynomials]{orthopolynom::chebyshev.s.polynomials()}},
\url{https://en.wikipedia.org/wiki/Chebyshev_polynomials}
}
\author{
David Kahle calling code from the orthopolynom package
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mp.R
\name{mp}
\alias{mp}
\alias{make_indeterminate_list}
\title{Define a multivariate polynomial.}
\usage{
make_indeterminate_list(vars)

mp(string, varorder, stars_only = FALSE)
}
\arguments{
\item{vars}{a character vector of indeterminates}

\item{string}{a character string containing a polynomial, see examples}

\item{varorder}{(optional) order of variables in string}

\item{stars_only}{if you format your multiplications using asterisks, setting
this to \code{TRUE} will reduce preprocessing time}
}
\value{
An object of class mpoly.
}
\description{
mp is a smart function which attempts to create a formal mpoly object from a
character string containing the usual representation  of a multivariate
polynomial.
}
\examples{

( m <- mp("x + y + x y") )
is.mpoly( m )
unclass(m)

mp("1 + x")


mp("x + 2 y + x^2 y + x y z")
mp("x + 2 y + x^2 y + x y z", varorder = c("y", "z", "x"))

( ms <- mp(c("x + y", "2 x")) )
is.mpolyList(ms)


gradient( mp("x + 2 y + x^2 y + x y z") )
gradient( mp("(x + y)^10") )

# mp and the print methods are kinds of inverses of each other
( polys <- mp(c("x + y", "x - y")) )
strings <- print(polys, silent = TRUE)
strings
mp(strings)

}
\seealso{
\code{\link[=mpoly]{mpoly()}}
}
\author{
David Kahle \email{david@kahle.io}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/round.mpoly.R
\name{round.mpoly}
\alias{round.mpoly}
\title{Round the coefficients of a polynomial}
\usage{
\method{round}{mpoly}(x, digits = 3)
}
\arguments{
\item{x}{an mpoly object}

\item{digits}{number of digits to round to}
}
\value{
the rounded mpoly object
}
\description{
Round the coefficients of an mpoly object.
}
\examples{

p <- mp("x + 3.14159265")^4
p
round(p)
round(p, 0)

\dontrun{
library(plyr)
library(ggplot2)
library(stringr)

n <- 101
s <- seq(-5, 5, length.out = n)

# one dimensional case
df <- data.frame(x = s)
df <- mutate(df, y = -x^2 + 2*x - 3 + rnorm(n, 0, 2))
qplot(x, y, data = df)
mod <- lm(y ~ x + I(x^2), data = df)
p <- as.mpoly(mod)
qplot(x, y, data = df) +
  stat_function(fun = as.function(p), colour = 'red')

p
round(p, 1)
qplot(x, y, data = df) +
  stat_function(fun = as.function(p), colour = 'red') +
  stat_function(fun = as.function(round(p,1)), colour = 'blue')


}

}
\seealso{
\code{\link[=mp]{mp()}}
}
\author{
David Kahle \email{david@kahle.io}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/burst.R
\name{burst}
\alias{burst}
\title{Enumerate integer r-vectors summing to n}
\usage{
burst(n, r = n)
}
\arguments{
\item{n}{integer to sum to}

\item{r}{number of components}
}
\value{
a matrix whose rows are the n-tuples
}
\description{
Determine all r-vectors with nonnegative integer entries summing to n.  Note
that this is not intended to be optimized.
}
\examples{
burst(3)

burst(3, 1)
burst(3, 2)
burst(3, 3)
burst(3, 4)

rowSums(burst(4))
rowSums( burst(3, 2) )
rowSums( burst(3, 3) )
rowSums( burst(3, 4) )


burst(10, 4) # all possible 2x2 contingency tables with n=10
burst(10, 4) / 10 # all possible empirical relative frequencies

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/predicates.R
\name{predicates}
\alias{predicates}
\alias{is.constant}
\alias{is.mpoly}
\alias{is.unipoly}
\alias{is.bernstein}
\alias{is.bezier}
\alias{is.chebyshev}
\alias{is.mpolyList}
\alias{is.linear}
\title{mpoly predicate functions}
\usage{
is.constant(x)

is.mpoly(x)

is.unipoly(x)

is.bernstein(x)

is.bezier(x)

is.chebyshev(x)

is.mpolyList(x)

is.linear(x)
}
\arguments{
\item{x}{object to be tested}
}
\value{
Vector of logicals.
}
\description{
Various functions to deal with mpoly and mpolyList objects.
}
\examples{

p <- mp("5")
is.mpoly(p)
is.constant(p)

is.constant(mp(c("x + 1", "7", "y - 2")))

p <- mp("x + y")
is.mpoly(p)

is.mpolyList(mp(c("x + 1", "y - 1")))



is.linear(mp("0"))
is.linear(mp("x + 1"))
is.linear(mp("x + y"))
is.linear(mp(c("0", "x + y")))

is.linear(mp("x + x y"))
is.linear(mp(c("x + x y", "x")))


(p <- bernstein(2, 5))
is.mpoly(p)
is.bernstein(p)

(p <- chebyshev(5))
is.mpoly(p)
is.chebyshev(p)
str(p)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/permutations.R
\name{permutations}
\alias{permutations}
\title{Determine all permutations of a set.}
\usage{
permutations(set)
}
\arguments{
\item{set}{a set}
}
\value{
a matrix whose rows are the permutations of set
}
\description{
An implementation of the Steinhaus-Johnson-Trotter permutation algorithm.
}
\examples{
permutations(1:3)
permutations(c('first','second','third'))
permutations(c(1,1,3))
apply(permutations(letters[1:6]), 1, paste, collapse = '')
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/basis_monomials.R
\name{basis_monomials}
\alias{basis_monomials}
\title{Enumerate basis monomials}
\usage{
basis_monomials(indeterminates, d)
}
\arguments{
\item{indeterminates}{a character vector}

\item{d}{the highest total degree}
}
\value{
a \code{\link[=mpolyList]{mpolyList()}} object of monomials
}
\description{
Enumerate basis monomials in the standard basis up to a given degree.
}
\examples{

basis_monomials(c("x", "y"), 2)
basis_monomials(c("x", "y"), 3)
basis_monomials(c("x", "y", "z"), 2)
basis_monomials(c("x", "y", "z"), 3)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/deriv.mpoly.R
\name{gradient}
\alias{gradient}
\title{Compute gradient of a multivariate polynomial.}
\usage{
gradient(mpoly)
}
\arguments{
\item{mpoly}{an object of class mpoly}
}
\value{
An object of class mpoly or mpolyList.
}
\description{
This is a wrapper for deriv.mpoly.
}
\examples{
m <- mp("x y + y z + z^2")
gradient(m)


# gradient descent illustration using the symbolically
# computed gradient of the rosenbrock function (shifted)
rosenbrock <- mp("(1 - x)^2 + 100 (y - x^2)^2")
fn <- as.function(rosenbrock)
(rosenbrock_gradient <- gradient(rosenbrock))
gr <- as.function(rosenbrock_gradient)

# visualize the function
library(ggplot2)
s <- seq(-5, 5, .05)
df <- expand.grid(x = s, y = s)
df$z <- apply(df, 1, fn)
ggplot(df, aes(x = x, y = y)) +
  geom_raster(aes(fill = z + 1e-10)) +
  scale_fill_continuous(trans = "log10")
  
# run the gradient descent algorithm using line-search
# step sizes computed with optimize()
current <- steps <- c(-3,-4)
change <- 1
tol <- 1e-5
while(change > tol){
  last  <- current
  delta <- optimize(
    function(delta) fn(current - delta*gr(current)),
    interval = c(1e-10, .1)
  )$minimum
  current <- current - delta*gr(current)
  steps   <- unname(rbind(steps, current))
  change  <- abs(fn(current) - fn(last))
}
steps <- as.data.frame(steps)
names(steps) <- c("x", "y")
  
# visualize steps, note the optim at c(1,1)
# the routine took 5748 steps
ggplot(df, aes(x = x, y = y)) +
  geom_raster(aes(fill = z + 1e-10)) +
  geom_path(data = steps, color = "red") +
  geom_point(data = steps, color = "red", size = .5) +
  scale_fill_continuous(trans = "log10")

# it gets to the general region of space quickly
# but once it gets on the right arc, it's terrible
# here's what the end game looks like
last_steps <- tail(steps, 100)
rngx <- range(last_steps$x); sx <- seq(rngx[1], rngx[2], length.out = 201)
rngy <- range(last_steps$y); sy <- seq(rngy[1], rngy[2], length.out = 201)
df <- expand.grid(x = sx, y = sy)
df$z <- apply(df, 1, fn) 
ggplot(df, aes(x = x, y = y)) +
  geom_raster(aes(fill = z)) +
  geom_path(data = last_steps, color = "red", size = .25) +
  geom_point(data = last_steps, color = "red", size = 1) +
  scale_fill_continuous(trans = "log10") 
  

}
\seealso{
\code{\link[=deriv.mpoly]{deriv.mpoly()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/predicates.R
\name{is.wholenumber}
\alias{is.wholenumber}
\title{Test whether an object is a whole number}
\usage{
is.wholenumber(x, tol = .Machine$double.eps^0.5)
}
\arguments{
\item{x}{object to be tested}

\item{tol}{tolerance within which a number is said to be whole}
}
\value{
Vector of logicals.
}
\description{
Test whether an object is a whole number.
}
\examples{
is.wholenumber(seq(-3,3,.5))
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mpolyList.R
\name{mpolyList}
\alias{mpolyList}
\title{Define a collection of multivariate polynomials.}
\usage{
mpolyList(...)
}
\arguments{
\item{...}{a series of mpoly objects.}
}
\value{
An object of class mpolyList.
}
\description{
Combine a series of mpoly objects into a mpolyList.
}
\examples{
( p1 <- mp("t^4 - x") )
( p2 <- mp("t^3 - y") )
( p3 <- mp("t^2 - z") )
( ms <- mpolyList(p1, p2, p3) )
is.mpolyList( ms )

mpolyList(mp("x + 1"))
p <- mp("x + 1")
mpolyList(p)

ps <- mp(c("x + 1", "y + 2"))
is.mpolyList(ps)


f <- function(){
  a <- mp("1")
  mpolyList(a)
}
f()


}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot.R
\name{plot.mpoly}
\alias{plot.mpoly}
\title{Plot the (real) variety of a polynomial}
\usage{
\method{plot}{mpoly}(x, xlim, ylim, varorder, add = FALSE, n = 251, nx
  = n, ny = n, f = 0.05, col = "red", ...)
}
\arguments{
\item{x}{an mpoly object}

\item{xlim, ylim}{numeric(2) vectors; x and y limits}

\item{varorder}{character(2); first element is x, second is y, defaults to
\code{sort(vars(poly))}}

\item{add}{logical; should the plot be added to the current device?}

\item{n, nx, ny}{integer specifying number of points in the x and y dimensions}

\item{f}{argument to pass to \code{\link[=extendrange]{extendrange()}}}

\item{col}{color of curve}

\item{...}{arguments to pass to \code{\link[=contour]{contour()}}}
}
\value{
\link{NULL}
}
\description{
The variety must have only 2 variables; it is plotted over a finite window;
and it will not discover zero-dimensional components.
}
\examples{

p <- mp("x^2 + y^2 - 1")
plot(p, xlim = c(-1, 1), ylim = c(-1, 1))
plot(p, xlim = c(-1, 1), ylim = c(-1, 1), asp = 1)

p <- mp("x^2 + 16 y^2 - 1")
plot(p, xlim = c(-1, 1), ylim = c(-1, 1))

p <- mp("u^2 + 16 v^2 - 1")
plot(p, xlim = c(-1, 1), ylim = c(-1, 1))

p <- mp("v^2 + 16 u^2 - 1")
plot(p, xlim = c(-1, 1), ylim = c(-1, 1))

p <- mp("u^2 + 16 v^2 - 1")
plot(p, xlim = c(-1, 1), ylim = c(-1, 1), varorder = c("v","u"))

p <- mp("y^2 - (x^3 + x^2)")
plot(p, xlim = c(-1.5, 1.5), ylim = c(-1.5, 1.5))

plot(lissajous(3, 3, 0, 0), xlim = c(-1, 1), ylim = c(-1, 1), asp = 1)
plot(lissajous(5, 5, 0, 0), col = "steelblue", add = TRUE)

# how it works - inefficient
p <- lissajous(5, 5, 0, 0)
df <- expand.grid(
  x = seq(-1, 1, length.out = 26),
  y = seq(-1, 1, length.out = 26)
)
pf <- as.function(p)
df$z <- structure(pf(df), .Names = "p")
head(df)
with(df, plot(x, y, cex = .75, pch = 16, col = c("firebrick", "steelblue")[(z >= 0) + 1]))
plot(lissajous(5, 5, 0, 0), col = "black", add = TRUE)


}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mpoly-defunct.R
\name{mpoly-defunct}
\alias{mpoly-defunct}
\title{Defunct mpoly functions}
\description{
This is a list of past functions of the mpoly package.
}
\details{
The following are defunct mpoly functions:

\itemize{

\item grobner

}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plug.R
\name{plug}
\alias{plug}
\title{Switch indeterminates in a polynomial}
\usage{
plug(p, indeterminate, value)
}
\arguments{
\item{p}{a polynomial}

\item{indeterminate}{the indeterminate in the polynomial to switch}

\item{value}{the value/indeterminate to substitute}
}
\value{
an mpoly object
}
\description{
Switch indeterminates in a polynomial
}
\examples{

# on an mpoly
(p <- mp("(x+y)^3"))
plug(p, "x", 5)
plug(p, "x", "t")
plug(p, "x", "y")
plug(p, "x", mp("2 y"))

plug(p, "x", mp("x + y"))
mp("((x+y)+y)^3")

# on an mpolyList
ps <- mp(c("x+y", "x+1"))
plug(ps, "x", 1)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/bernstein_approx.R
\name{bernstein-approx}
\alias{bernstein-approx}
\alias{bernstein_approx}
\alias{bernsteinApprox}
\title{Bernstein polynomial approximation}
\usage{
bernstein_approx(f, n, lower = 0, upper = 1, indeterminate = "x")

bernsteinApprox(...)
}
\arguments{
\item{f}{the function to approximate}

\item{n}{Bernstein polynomial degree}

\item{lower}{lower bound for approximation}

\item{upper}{upper bound for approximation}

\item{indeterminate}{indeterminate}

\item{...}{...}
}
\value{
a mpoly object
}
\description{
Bernstein polynomial approximation
}
\examples{





\dontrun{  # visualize the bernstein polynomials

library(ggplot2); theme_set(theme_bw())
library(reshape2)




f <- function(x) sin(2*pi*x)
p <- bernstein_approx(f, 20)
round(p, 3)

x <- seq(0, 1, length.out = 101)
df <- data.frame(
  x = rep(x, 2),
  y = c(f(x), as.function(p)(x)),
  which = rep(c("actual", "approx"), each = 101)
)
qplot(x, y, data = df, geom = "line", color = which)






p <- bernstein_approx(sin, 20, pi/2, 1.5*pi)
round(p, 4)

x <- seq(0, 2*pi, length.out = 101)
df <- data.frame(
  x = rep(x, 2),
  y = c(sin(x), as.function(p)(x)),
  which = rep(c("actual", "approx"), each = 101)
)
qplot(x, y, data = df, geom = "line", color = which)








p <- bernstein_approx(dnorm, 15, -1.25, 1.25)
round(p, 4)

x <- seq(-3, 3, length.out = 101)
df <- data.frame(
  x = rep(x, 2),
  y = c(dnorm(x), as.function(p)(x)),
  which = rep(c("actual", "approx"), each = 101)
)
qplot(x, y, data = df, geom = "line", color = which)






}

}
\author{
David Kahle
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/print.mpolyList.R
\name{print.mpolyList}
\alias{print.mpolyList}
\title{Pretty printing of a list of multivariate polynomials.}
\usage{
\method{print}{mpolyList}(
  x,
  varorder = vars(x),
  stars = FALSE,
  order,
  silent = FALSE,
  ...,
  plus_pad = 2L,
  times_pad = 1L
)
}
\arguments{
\item{x}{an object of class mpoly}

\item{varorder}{the order of the variables}

\item{stars}{print the multivariate polynomial in the more computer-friendly
asterisk notation (default FALSE)}

\item{order}{a total order used to order the monomials in the printing}

\item{silent}{logical; if TRUE, suppresses output}

\item{...}{additional parameters to go to \code{\link[base:cat]{base::cat()}}}

\item{plus_pad}{number of spaces to the left and right of plus sign}

\item{times_pad}{number of spaces to the left and right of times sign}
}
\value{
Invisible character vector of the printed objects.
}
\description{
This function iterates print.mpoly on an object of class mpolyList.
}
\examples{

mL <- mp(c("x + 1", "y - 1", "x y^2 z  +  x^2 z^2  +  z^2  +  x^3"))
mL
print(mL, order = "lex")
print(mL, order = "glex")
print(mL, order = "grlex")
print(mL, order = "glex", varorder = c("z","y","x"))
print(mL, order = "grlex", varorder = c("z","y","x"))
print(mL, varorder = c("z","y","x"))

(print(mL, varorder = c("z","y","x"), plus_pad = 1L, silent = TRUE))

(print(mL, silent = TRUE, stars = TRUE, plus_pad = 1L, times_pad = 0L))

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/homogenize.R
\name{homogenize}
\alias{homogenize}
\alias{dehomogenize}
\alias{is.homogeneous}
\alias{homogeneous_components}
\title{Homogenize a polynomial}
\usage{
homogenize(x, indeterminate = "t")

dehomogenize(x, indeterminate = "t")

is.homogeneous(x)

homogeneous_components(x)
}
\arguments{
\item{x}{an mpoly object, see \code{\link[=mpoly]{mpoly()}}}

\item{indeterminate}{name of homogenization}
}
\value{
a (de/homogenized) mpoly or an mpolyList
}
\description{
Homogenize a polynomial.
}
\examples{

x <- mp("x^4 + y + 2 x y^2 - 3 z")
is.homogeneous(x)
(xh <- homogenize(x))
is.homogeneous(xh)

homogeneous_components(x)

homogenize(x, "o")

xh <- homogenize(x)
dehomogenize(xh) # assumes indeterminate = "t"
plug(xh, "t", 1) # same effect, but dehomogenize is faster



# the functions are vectorized
(ps <- mp(c("x + y^2", "x + y^3")))
(psh <- homogenize(ps))
dehomogenize(psh)


# demonstrating a leading property of homogeneous polynomials
library(magrittr)
p  <- mp("x^2 + 2 x + 3")
(ph <- homogenize(p, "y"))
lambda <- 3
(d <- totaldeg(p))
ph \%>\%
  plug("x", lambda*mp("x")) \%>\%
  plug("y", lambda*mp("y"))
lambda^d * ph

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/permutations.R
\name{insert}
\alias{insert}
\title{Insert an element into a vector.}
\usage{
insert(elem, slot, v)
}
\arguments{
\item{elem}{element to insert}

\item{slot}{location of insert}

\item{v}{vector to insert into}
}
\value{
vector with element inserted
}
\description{
Insert an element into a vector.
}
\examples{
insert(2, 1, 1)
insert(2, 2, 1) 
insert('x', 5, letters) 
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/terms.mpoly.R
\name{terms.mpoly}
\alias{terms.mpoly}
\title{Extract the terms of a multivariate polynomial.}
\usage{
\method{terms}{mpoly}(x, ...)
}
\arguments{
\item{x}{an object of class mpoly}

\item{...}{additional parameters}
}
\value{
An object of class mpolyList.
}
\description{
Compute the terms of an mpoly object as a mpolyList.
}
\examples{

\dontrun{ .Deprecated issues a warning

x <- mp("x^2 - y + x y z")
terms(x)
monomials(x)

}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/bernstein.R
\name{bernstein}
\alias{bernstein}
\title{Bernstein polynomials}
\usage{
bernstein(k, n, indeterminate = "x")
}
\arguments{
\item{k}{Bernstein polynomial k}

\item{n}{Bernstein polynomial degree}

\item{indeterminate}{indeterminate}
}
\value{
a mpoly object
}
\description{
Bernstein polynomials
}
\examples{

bernstein(0, 0)

bernstein(0, 1)
bernstein(1, 1)

bernstein(0, 1, "t")

bernstein(0:2, 2)
bernstein(0:3, 3)
bernstein(0:3, 3, "t")


bernstein(0:4, 4)
bernstein(0:10, 10)
bernstein(0:10, 10, "t")
bernstein(0:20, 20, "t")

\dontrun{  # visualize the bernstein polynomials

library(ggplot2); theme_set(theme_classic())
library(tidyr)

s <- seq(0, 1, length.out = 101)
N <- 10 # number of bernstein polynomials to plot
(bernPolys <- bernstein(0:N, N))

df <- data.frame(s, as.function(bernPolys)(s))
names(df) <- c("x", paste0("B_", 0:N))
head(df)

mdf <- gather(df, degree, value, -x)
head(mdf)

qplot(x, value, data = mdf, geom = "line", color = degree)

}


}
\author{
David Kahle
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/partitions.R
\name{partitions}
\alias{partitions}
\title{Enumerate the partitions of an integer}
\usage{
partitions(n)
}
\arguments{
\item{n}{an integer}
}
\value{
a matrix whose rows are the n-tuples
}
\description{
Determine all unrestricted partitions of an integer.  This function is
equivalent to the function parts in the partitions package.
}
\examples{
partitions(5)
str(partitions(5))
}
\author{
Robin K. S. Hankin, from package partitions
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/deriv.mpoly.R
\name{deriv.mpoly}
\alias{deriv.mpoly}
\title{Compute partial derivatives of a multivariate polynomial.}
\usage{
\method{deriv}{mpoly}(expr, var, ..., bring_power_down = TRUE)
}
\arguments{
\item{expr}{an object of class mpoly}

\item{var}{character - the partial derivative desired}

\item{...}{any additional arguments}

\item{bring_power_down}{if \code{FALSE}, x^n -> x^(n-1), not n x^(n-1)}
}
\value{
An object of class mpoly or mpolyList.
}
\description{
This is a deriv method for mpoly objects.  It does not call the
\code{\link[stats:deriv]{stats::deriv()}}.
}
\examples{
p <- mp("x y + y z + z^2")
deriv(p, "x")
deriv(p, "y")
deriv(p, "z")
deriv(p, "t")
deriv(p, c("x","y","z"))

is.mpoly(deriv(p, "x"))
is.mpolyList( deriv(p, c("x","y","z")) )

p <- mp("x^5")
deriv(p, "x")
deriv(p, "x", bring_power_down = FALSE)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/swap.R
\name{swap}
\alias{swap}
\title{Swap polynomial indeterminates}
\usage{
swap(p, variable, replacement)
}
\arguments{
\item{p}{polynomial}

\item{variable}{the variable in the polynomial to replace}

\item{replacement}{the replacement variable}
}
\value{
a mpoly object
}
\description{
Swap polynomial indeterminates
}
\examples{

(p <- mp("(x + y)^2"))
swap(p, "x", "t")

## the meta data is retained
(p <- bernstein(3, 5))
(p2 <- swap(p, "x", "t"))
is.bernstein(p2)

(p <- chebyshev(3))
(p2 <- swap(p, "x", "t"))
is.chebyshev(p2)


}
\author{
David Kahle
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/LCM.R
\name{LCM}
\alias{LCM}
\title{Compute the least common multiple of two numbers.}
\usage{
LCM(x, y)
}
\arguments{
\item{x}{an object of class numeric}

\item{y}{an object of class numeric}
}
\value{
The least common multiple of x and y.
}
\description{
A simple algorithm to compute the least common multiple of two numbers
}
\examples{
LCM(5,7)
LCM(5,8)
LCM(5,9)
LCM(5,10)
Reduce(LCM, 1:10) # -> 2520
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/laguerre.R
\name{laguerre}
\alias{laguerre}
\title{Generalized Laguerre polynomials}
\usage{
laguerre(degree, alpha = 0, indeterminate = "x", normalized = FALSE)
}
\arguments{
\item{degree}{degree of polynomial}

\item{alpha}{generalization constant}

\item{indeterminate}{indeterminate}

\item{normalized}{provide normalized coefficients}
}
\value{
a mpoly object or mpolyList object
}
\description{
Generalized Laguerre polynomials as computed by orthopolynom.
}
\examples{

laguerre(0)
laguerre(1)
laguerre(2)
laguerre(3)
laguerre(4)
laguerre(5)
laguerre(6)

laguerre(2)
laguerre(2, normalized = TRUE)

laguerre(0:5)
laguerre(0:5, normalized = TRUE)
laguerre(0:5, indeterminate = "t")



# visualize the laguerre polynomials

library(ggplot2); theme_set(theme_classic())
library(tidyr)

s <- seq(-5, 20, length.out = 201)
N <- 5 # number of laguerre polynomials to plot
(lagPolys <- laguerre(0:N))

# see ?bernstein for a better understanding of
# how the code below works

df <- data.frame(s, as.function(lagPolys)(s))
names(df) <- c("x", paste0("L_", 0:N))
mdf <- gather(df, degree, value, -x)
qplot(x, value, data = mdf, geom = "line", color = degree)

qplot(x, value, data = mdf, geom = "line", color = degree) +
  coord_cartesian(ylim = c(-25, 25))


# laguerre polynomials are orthogonal with respect to the exponential kernel:
L2 <- as.function(laguerre(2))
L3 <- as.function(laguerre(3))
L4 <- as.function(laguerre(4))

w <- dexp
integrate(function(x) L2(x) * L3(x) * w(x), lower = 0, upper = Inf)
integrate(function(x) L2(x) * L4(x) * w(x), lower = 0, upper = Inf)
integrate(function(x) L3(x) * L4(x) * w(x), lower = 0, upper = Inf)


}
\seealso{
\code{\link[orthopolynom:glaguerre.polynomials]{orthopolynom::glaguerre.polynomials()}},
\url{https://en.wikipedia.org/wiki/Laguerre_polynomials}
}
\author{
David Kahle calling code from the orthopolynom package
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tuples.R
\name{tuples}
\alias{tuples}
\title{Determine all n-tuples using the elements of a set.}
\usage{
tuples(set, n = length(set), repeats = FALSE, list = FALSE)
}
\arguments{
\item{set}{a set}

\item{n}{length of each tuple}

\item{repeats}{if set contains duplicates, should the result?}

\item{list}{tuples as list?}
}
\value{
a matrix whose rows are the n-tuples
}
\description{
Determine all n-tuples using the elements of a set.  This is really just a
simple wrapper for expand.grid, so it is not optimized.
}
\examples{

tuples(1:2, 3)
tuples(1:2, 3, list = TRUE)

apply(tuples(c("x","y","z"), 3), 1, paste, collapse = "")

# multinomial coefficients
r <- 2 # number of variables, e.g. x, y
n <- 2 # power, e.g. (x+y)^2
apply(burst(n,r), 1, function(v) factorial(n)/ prod(factorial(v))) # x, y, xy
mp("x + y")^n

r <- 2 # number of variables, e.g. x, y
n <- 3 # power, e.g. (x+y)^3
apply(burst(n,r), 1, function(v) factorial(n)/ prod(factorial(v)))
mp("x + y")^n

r <- 3 # number of variables, e.g. x, y, z
n <- 2 # power, e.g. (x+y+z)^2
apply(burst(n,r), 1, function(v) factorial(n)/ prod(factorial(v))) # x, y, z, xy, xz, yz
mp("x + y + z")^n


}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mpolyListArithmetic.R
\name{mpolyListArithmetic}
\alias{mpolyListArithmetic}
\alias{+.mpolyList}
\alias{-.mpolyList}
\alias{*.mpolyList}
\title{Element-wise arithmetic with vectors of multivariate polynomials.}
\usage{
\method{+}{mpolyList}(e1, e2)

\method{-}{mpolyList}(e1, e2)

\method{*}{mpolyList}(e1, e2)
}
\arguments{
\item{e1}{an object of class mpolyList}

\item{e2}{an object of class mpolyList}
}
\value{
An object of class mpolyList.
}
\description{
Element-wise arithmetic with vectors of multivariate polynomials.
}
\examples{

( ms1 <- mp( c("x", 'y') ) )
( ms2 <- mp( c("y", '2 x^2') ) )
ms1 + ms2
ms1 - ms2
ms1 * ms2
ms1^3

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/bezier.R
\name{bezier_function}
\alias{bezier_function}
\alias{bezierFunction}
\title{Bezier function}
\usage{
bezier_function(points, weights = rep(1L, nrow(points)))

bezierFunction(...)
}
\arguments{
\item{points}{a matrix or data frame of numerics.  the rows represent points.}

\item{weights}{the weights in a weighted Bezier curve}

\item{...}{...; used internally}
}
\value{
function of a single parameter
}
\description{
Compute the Bezier function of a collection of polynomials.  By Bezier
function we mean the Bezier curve function, a parametric map running from t =
0, the first point, to t = 1, the last point, where the coordinate mappings
are linear combinations of Bernstein polynomials.
}
\details{
The function returned is vectorized and evaluates the Bezier curve in a
numerically stable way with de Castlejau's algorithm (implemented in R).
}
\examples{

library(ggplot2); theme_set(theme_bw())


t <- seq(0, 1, length.out = 201)
points <- data.frame(x = 0:3, y = c(0,1,-1,0))


f <- bezier_function(points)
df <- as.data.frame(f(t))

ggplot(aes(x = x, y = y), data = df) +
  geom_point(data = points, color = "red", size = 8) +
  geom_path(data = points, color = "red") +
  geom_path()




f <- bezier_function(points, weights = c(1,5,5,1))
df <- as.data.frame(f(t))

ggplot(aes(x = x, y = y), data = df) +
  geom_point(data = points, color = "red", size = 8) +
  geom_path(data = points, color = "red") +
  geom_path()




f <- bezier_function(points, weights = c(1,10,10,1))
df <- as.data.frame(f(t))

ggplot(aes(x = x, y = y), data = df) +
  geom_point(data = points, color = "red", size = 8) +
  geom_path(data = points, color = "red") +
  geom_path()









  
}
\references{
\url{https://en.wikipedia.org/wiki/Bezier_curve},
\url{https://en.wikipedia.org/wiki/De_Casteljau's_algorithm}
}
\seealso{
\code{\link[=bezier]{bezier()}}
}
\author{
David Kahle
}
